-- ATE Tool database upgrade script 
-- From version     : 1.9.x
-- To version       : 1.10.0
-- Target DB engine : MySQL 5.5.x

-- -----------------------------------------------------
-- Table `tool`.`TaskPONumber`
-- -----------------------------------------------------
DROP TABLE IF EXISTS `tool`.`TaskPONumber` ;

CREATE TABLE IF NOT EXISTS `tool`.`TaskPONumber` (
  `TaskPONumber_ID` INT NOT NULL AUTO_INCREMENT,
  `PONumber` VARCHAR(20) CHARACTER SET 'utf8' COLLATE 'utf8_unicode_ci' NOT NULL,
  `Task_ID` INT NOT NULL,
  `Year` INT NOT NULL,
  PRIMARY KEY (`TaskPONumber_ID`),
  CONSTRAINT `TaskPONumber_Task_fk`
    FOREIGN KEY (`Task_ID`)
    REFERENCES `tool`.`Task` (`Task_ID`)
    ON DELETE CASCADE
    ON UPDATE NO ACTION)
ENGINE = InnoDB;

-- Create new table containing ECO states
DROP TABLE IF EXISTS `tool`.`ECOState`;
CREATE TABLE IF NOT EXISTS `tool`.`ECOState` (
    `ECOState_ID` INT(11) NOT NULL AUTO_INCREMENT,
    `State` VARCHAR(15) CHARACTER SET 'utf8' COLLATE 'utf8_unicode_ci',
    `Value` TINYINT NOT NULL,
    `Type` ENUM('Project','Person') NOT NULL,
    PRIMARY KEY (`ECOState_ID`),
    UNIQUE(`State`, `Type`)
) ENGINE = InnoDB;

-- Insert default values into the table ECOState
INSERT INTO `ECOState` (`State`, `Value`, `Type`) VALUES
('CZ', 1, 'Project'),
('EU', 2, 'Project'),
('NLR', 3, 'Project'),
('CZ', 1, 'Person'),
('EU', 2, 'Person'),
('Other', 3, 'Person');

-- Bind column AccessLevel from table Person to ECOState table
ALTER TABLE `Person` ADD COLUMN `temp_AccessLevel` INT(11);

UPDATE `Person` SET `temp_AccessLevel` = 4 WHERE `AccessLevel` = 'CZ';
UPDATE `Person` SET `temp_AccessLevel` = 5 WHERE `AccessLevel` = 'EU';
UPDATE `Person` SET `temp_AccessLevel` = 6 WHERE `AccessLevel` = 'Other';

ALTER TABLE `Person` DROP COLUMN `AccessLevel`;
ALTER TABLE `Person` CHANGE `temp_AccessLevel` `AccessLevel` INT(11) NOT NULL;
ALTER TABLE `Person` ADD CONSTRAINT `Person_ECOState_fk` FOREIGN KEY (`AccessLevel`) REFERENCES ECOState(`ECOState_ID`) ON UPDATE CASCADE ON DELETE RESTRICT;

-- Bind column SecurityLevel from table Project to ECOState table
ALTER TABLE `Project` ADD COLUMN `temp_SecurityLevel` INT(11);

UPDATE `Project` SET `temp_SecurityLevel` = 1 WHERE `SecurityLevel` = 'CZ';
UPDATE `Project` SET `temp_SecurityLevel` = 2 WHERE `SecurityLevel` = 'EU';
UPDATE `Project` SET `temp_SecurityLevel` = 3 WHERE `SecurityLevel` = 'NLR';

ALTER TABLE `Project` DROP COLUMN `SecurityLevel`;
ALTER TABLE `Project` CHANGE `temp_SecurityLevel` `SecurityLevel` INT(11) NOT NULL;
ALTER TABLE `Project` ADD CONSTRAINT `Project_ECOState_fk` FOREIGN KEY (`SecurityLevel`) REFERENCES ECOState(`ECOState_ID`) ON UPDATE CASCADE ON DELETE RESTRICT;

-- Add column to table ProjectCharter - implementing ECO
ALTER TABLE `ProjectCharter` ADD COLUMN `SecurityLevel` INT(11);

UPDATE `ProjectCharter` SET `SecurityLevel` = 1; -- ECOState CZ of type Project

ALTER TABLE `ProjectCharter` CHANGE `SecurityLevel` `SecurityLevel` INT(11) NOT NULL;
ALTER TABLE `ProjectCharter` ADD CONSTRAINT `ProjectCharter_ECOState_fk` FOREIGN KEY (`SecurityLevel`) REFERENCES ECOState(`ECOState_ID`) ON UPDATE CASCADE ON DELETE RESTRICT;

-- Add column to table StrapItem - implementing ECO
ALTER TABLE `StrapItem` ADD COLUMN `SecurityLevel` INT(11);

UPDATE `StrapItem` SET `SecurityLevel` = 1; -- ECOState CZ of type Project

ALTER TABLE `StrapItem` CHANGE `SecurityLevel` `SecurityLevel` INT(11) NOT NULL;
ALTER TABLE `StrapItem` ADD CONSTRAINT `StrapItem_ECOState_fk` FOREIGN KEY (`SecurityLevel`) REFERENCES ECOState(`ECOState_ID`) ON UPDATE CASCADE ON DELETE RESTRICT;

-- Add column to table StrapItemActivity - implementing ECO
ALTER TABLE `StrapItemActivity` ADD COLUMN `SecurityLevel` INT(11);

UPDATE `StrapItemActivity` SET `SecurityLevel` = 1; -- ECOState CZ of type Project

ALTER TABLE `StrapItemActivity` CHANGE `SecurityLevel` `SecurityLevel` INT(11) NOT NULL;       
ALTER TABLE `StrapItemActivity` ADD CONSTRAINT `StrapItemActivity_ECOState_fk` FOREIGN KEY (`SecurityLevel`) REFERENCES ECOState(`ECOState_ID`) ON UPDATE CASCADE ON DELETE RESTRICT;

-- Insert default start date of range for iASAP actuals import
INSERT INTO `GlobalSetting` (`Key`, `Value`, `Default`, `DataType`)
VALUES ('iASAP.actuals.update.startDate', '2014-01-01', '', 'string');

-- Insert credentials for iASAP actuals auto import
INSERT INTO `GlobalSetting` (`Key`, `Value`, `Default`, `DataType`)
VALUES ('iASAP.actuals.credentials.cronToken', '65a88332ff952eb99699597bb4d696b6', '', 'string');

-- Add column to table Actuals - will determine data source
ALTER TABLE `Actuals` ADD COLUMN `Source` TINYINT NOT NULL DEFAULT 0;

-- Update source of data in table Actuals - all current records has the same source (eCharge)
UPDATE `Actuals`
SET `Source` = 0;


-- Drop fk to table ECO State
ALTER TABLE `Person` DROP FOREIGN KEY `Person_ECOState_fk`;
ALTER TABLE `ProjectCharter` DROP FOREIGN KEY `ProjectCharter_ECOState_fk`;
ALTER TABLE `StrapItem` DROP FOREIGN KEY `StrapItem_ECOState_fk`;
ALTER TABLE `StrapItemActivity` DROP FOREIGN KEY `StrapItemActivity_ECOState_fk`;
ALTER TABLE `Project` DROP FOREIGN KEY `Project_ECOState_fk`;

-- Drop table ECOState
DROP TABLE IF EXISTS `tool`.`ECOState`;

-- Create new table instead ECOState
DROP TABLE IF EXISTS `tool`.`ECLicence`;
CREATE TABLE IF NOT EXISTS `tool`.`ECLicence` (
    `ECLicence_ID` INT(11) NOT NULL AUTO_INCREMENT,
    `Name` VARCHAR(30) CHARACTER SET 'utf8' COLLATE 'utf8_unicode_ci',
    `Value` TINYINT NOT NULL,
    `ECLicenceNamePrefix` VARCHAR(15) CHARACTER SET 'utf8' COLLATE 'utf8_unicode_ci',
    PRIMARY KEY (`ECLicence_ID`),
    UNIQUE(`Name`),
    UNIQUE(`ECLicenceNamePrefix`)
) ENGINE = InnoDB;

-- Insert default values into the table ECLicence
INSERT INTO `ECLicence` (`Name`, `Value`, `ECLicenceNamePrefix`) VALUES
('MILITARY', 1, 'MIL'),
('DUAL', 2, 'DU'),
('ITAR', 0, 'ITAR'),
('EAR', 0, 'EAR'),
('NLR', 3, 'NLR');

-- Bind dropped fk to table ECOState to ECLicence
ALTER TABLE `ProjectCharter` CHANGE `SecurityLevel` `ECLicence_ID` INT(11) NOT NULL;
ALTER TABLE `ProjectCharter` ADD CONSTRAINT `ProjectCharter_ECLicence_fk` FOREIGN KEY (`ECLicence_ID`) REFERENCES ECLicence(`ECLicence_ID`) ON UPDATE CASCADE ON DELETE RESTRICT;
ALTER TABLE `StrapItem` CHANGE `SecurityLevel` `ECLicence_ID` INT(11) NOT NULL;
ALTER TABLE `StrapItem` ADD CONSTRAINT `StrapItem_ECLicence_fk` FOREIGN KEY (`ECLicence_ID`) REFERENCES ECLicence(`ECLicence_ID`) ON UPDATE CASCADE ON DELETE RESTRICT;
ALTER TABLE `StrapItemActivity` CHANGE `SecurityLevel` `ECLicence_ID` INT(11) NOT NULL;
ALTER TABLE `StrapItemActivity` ADD CONSTRAINT `StrapItemActivity_ECLicence_fk` FOREIGN KEY (`ECLicence_ID`) REFERENCES ECLicence(`ECLicence_ID`) ON UPDATE CASCADE ON DELETE RESTRICT;
ALTER TABLE `Project` CHANGE `SecurityLevel` `ECLicence_ID` INT(11);
ALTER TABLE `Project` ADD CONSTRAINT `Project_ECLicence_fk` FOREIGN KEY (`ECLicence_ID`) REFERENCES ECLicence(`ECLicence_ID`) ON UPDATE CASCADE ON DELETE RESTRICT;

-- Defalut ECLicence_ID for projects
UPDATE `Project`
SET `ECLicence_ID` = 5;

-- Create new table for relation between Person and EC licence
DROP TABLE IF EXISTS `tool`.`Person_ECLicence`;
CREATE TABLE IF NOT EXISTS `tool`.`Person_ECLicence` (
    `EID` VARCHAR(7) NOT NULL,
    `ECLicence_ID` INT(11) NOT NULL,
    UNIQUE (`EID`, `ECLicence_ID`),
	CONSTRAINT `Person_ECLicence_Person_fk`
		FOREIGN KEY (`EID`)
		REFERENCES `tool`.`Person` (`EID`)
		ON DELETE CASCADE
		ON UPDATE CASCADE,
	CONSTRAINT `Person_ECLicence_ECLicence_fk`
		FOREIGN KEY (`ECLicence_ID`)
		REFERENCES `tool`.`ECLicence` (`ECLicence_ID`)
		ON DELETE CASCADE
		ON UPDATE CASCADE
) ENGINE = InnoDB;

-- Create new table for EC licences
DROP TABLE IF EXISTS `tool`.`ECLicenceName`;
CREATE TABLE IF NOT EXISTS `tool`.`ECLicenceName` (
    `ECLicenceName_ID` INT(11) NOT NULL AUTO_INCREMENT,
    `Name` VARCHAR(50) CHARACTER SET 'utf8' COLLATE 'utf8_unicode_ci' NOT NULL,
    PRIMARY KEY (`ECLicenceName_ID`),
    UNIQUE(`Name`)
) ENGINE = InnoDB;

-- Create new table for relation between Projects and EC licences
DROP TABLE IF EXISTS `tool`.`Project_ECLicenceName`;
CREATE TABLE IF NOT EXISTS `tool`.`Project_ECLicenceName` (
    `Project_ID` INT(11) NOT NULL,
    `ECLicenceName_ID` INT(11) NOT NULL,
    UNIQUE (`Project_ID`, `ECLicenceName_ID`),
	CONSTRAINT `Project_ECLicenceName_Project_fk`
		FOREIGN KEY (`Project_ID`)
		REFERENCES `tool`.`Project` (`Project_ID`)
		ON DELETE CASCADE
		ON UPDATE CASCADE,
	CONSTRAINT `Project_ECLicenceName_ECLicenceName_fk`
		FOREIGN KEY (`ECLicenceName_ID`)
		REFERENCES `tool`.`ECLicenceName` (`ECLicenceName_ID`)
		ON DELETE CASCADE
		ON UPDATE CASCADE
) ENGINE = InnoDB;

-- Create new table Nationality
DROP TABLE IF EXISTS `tool`.`Nationality`;
CREATE TABLE IF NOT EXISTS `tool`.`Nationality` (
    `Nationality_ID` INT(11) NOT NULL AUTO_INCREMENT,
    `Name` VARCHAR(30) CHARACTER SET 'utf8' COLLATE 'utf8_unicode_ci',
    `Value` TINYINT NOT NULL,
    PRIMARY KEY (`Nationality_ID`),
    UNIQUE(`Name`)
) ENGINE = InnoDB;

-- Insert default values into the table Nationality
INSERT INTO `Nationality` (`Name`, `Value`) VALUES
('CZ', 1),
('EU', 2),
('Other', 3);

-- Transform Person.AccessLevel values to interval of values of Nationality.Nationality_ID
UPDATE `Person`
SET `AccessLevel` = `AccessLevel` - 3;

-- Bind person access level to nationality
ALTER TABLE `Person` CHANGE `AccessLevel` `Nationality_ID` INT(11) NOT NULL;
ALTER TABLE `Person` ADD CONSTRAINT `Person_Nationality_fk` FOREIGN KEY (`Nationality_ID`) REFERENCES Nationality(`Nationality_ID`) ON UPDATE CASCADE ON DELETE RESTRICT;

-- Update project attribute ResponsibleUnit
UPDATE `Project`
SET `ResponsibleUnit` = (SELECT `OrganizationUnit`.`OrganizationUnit_ID`
                         FROM `OrganizationUnit`
                         WHERE `Path` = (SELECT SUBSTR(`Path`, 1, 3)
                                         FROM `OrganizationUnit`
                                         WHERE `OrganizationUnit`.`OrganizationUnit_ID` = (SELECT `Person`.`OrganizationUnit_ID`
                                                                                           FROM `Person`
                                                                                           WHERE `EID` = `Project`.`TM`
                                                                                           )
                                        )
                         );

-- Bug fix - deleting of Person and OrganizationUnit
ALTER TABLE `SettingDisplayedContent` DROP FOREIGN KEY `settingdisplayedcontent_ibfk_3`, ADD CONSTRAINT `SettingDisplayedContent_Person_fk` FOREIGN KEY (`EID`) REFERENCES `tool`.`person`(`EID`) ON DELETE CASCADE ON UPDATE CASCADE;
ALTER TABLE `SettingDisplayedContent` DROP FOREIGN KEY `settingdisplayedcontent_ibfk_4`, ADD CONSTRAINT `SettingDisplayedContent_OrganizationUnit_fk` FOREIGN KEY (`OrganizationUnit_ID`) REFERENCES `tool`.`organizationunit`(`OrganizationUnit_ID`) ON DELETE CASCADE ON UPDATE CASCADE;

-- Create email list for testers
INSERT INTO `GlobalSetting` (`Key`, `Default`, `DataType`)
VALUES ('mail.developmentMode', '0', 'boolean');

-- Create email list for testers
INSERT INTO `GlobalSetting` (`Key`, `Value`, `Default`, `DataType`)
VALUES ('mail.testerList', 'vojtech.sedlak@honeywell.com', '', 'string');

-- Login switcher
INSERT INTO `GlobalSetting` (`Key`, `Default`, `DataType`)
VALUES ('login.developmentMode', '0', 'boolean');

-- Workable hours remark added.
ALTER TABLE `Hour` ADD COLUMN `Remark` TEXT DEFAULT NULL;

-- Export Control Month Check - Token
INSERT INTO `GlobalSetting` (`Key`, `Value`, `Default`, `DataType`)
VALUES ('exportControlMonthCheck.credentials.cronToken', '25279b5d35c6e89b94e2eb5cb36d5172', '', 'string');

-- Adds new column into the table Organizaiton Unit
ALTER TABLE `OrganizationUnit` ADD COLUMN `Locked` TINYINT NOT NULL DEFAULT 0;

-- Adds new column into the table Hour
ALTER TABLE `Hour` ADD COLUMN `IsAOP` TINYINT NULL AFTER `Contract`;

UPDATE `Hour` SET `Hour`.`IsAOP` = 1 WHERE `Hour`.`Yield` IS NOT NULL;
UPDATE `Hour` SET `Hour`.`IsAOP` = 0 WHERE `Hour`.`Yield` IS NULL;
UPDATE `Hour` SET `Hour`.`Yield` = 0.94 WHERE `Hour`.`Yield` IS NULL;
ALTER TABLE `Hour` CHANGE `Yield` `Yield` DECIMAL(3,2) NULL DEFAULT '0.94';

-- Change of OrganizationUnit Parent_ID
ALTER TABLE `OrganizationUnit` DROP FOREIGN KEY `OrganizationUnit_fk1`;
ALTER TABLE `OrganizationUnit` CHANGE `Parent_ID` `Parent_ID` INT(11);
ALTER TABLE `OrganizationUnit` ADD CONSTRAINT `OrganizationUnit_OrganizationUnit_fk` FOREIGN KEY (`Parent_ID`) REFERENCES `OrganizationUnit`(`OrganizationUnit_ID`) ON DELETE SET NULL ON UPDATE CASCADE;

-- Subcontractors update
INSERT INTO `Subcontractor` (`Name`, `Short`) VALUES
('Howard Glover', 'Glover'),
('Nationaal Luchten', 'NLR'),
('Ottawa', 'OTT'),
('Torrance', 'TOR'),
('Tempe', 'TEM'),
('Aguadilla - Aerospace', 'AGD');

ALTER TABLE `POAttributesAndBudget` DROP FOREIGN KEY `POAttributesAndBudget_Project_fk`; ALTER TABLE `POAttributesAndBudget` ADD CONSTRAINT `POAttributesAndBudget_Project_fk` FOREIGN KEY (`Project_ID`) REFERENCES `tool`.`project`(`Project_ID`) ON DELETE CASCADE ON UPDATE NO ACTION;

-- Adds new column into the table Task
ALTER TABLE `Task` ADD COLUMN `CostAsLabour` TINYINT NOT NULL DEFAULT 0;

-- HTS tasks are now considered as labour task. There is need sets cost center and labour flag to all of them.
UPDATE `Task` SET `Task`.`CostCenter_ID` = 'CZxxxxxx' WHERE `Task`.`Name` LIKE 'HTS%';
UPDATE `Task` SET `Task`.`LabourTask` = '1' WHERE `Task`.`Name` LIKE 'HTS%';
UPDATE `Task` SET `Task`.`CostAsLabour` = '1' WHERE `Task`.`Name` LIKE 'HTS%';