-- ATE Tool database upgrade script 
-- From version: 1.1.0
-- To version: 1.2.0
-- Target DB engine: MySQL 5.1.x

SET SQL_MODE="NO_AUTO_VALUE_ON_ZERO";
SET time_zone = "+00:00";


/*!40101 SET @OLD_CHARACTER_SET_CLIENT=@@CHARACTER_SET_CLIENT */;
/*!40101 SET @OLD_CHARACTER_SET_RESULTS=@@CHARACTER_SET_RESULTS */;
/*!40101 SET @OLD_COLLATION_CONNECTION=@@COLLATION_CONNECTION */;
/*!40101 SET NAMES utf8 */;

/* Prefix for guessing */
ALTER TABLE `CostCenter` ADD `TaskNamePrefix` VARCHAR(20) NOT NULL;
UPDATE `CostCenter` SET `TaskNamePrefix`='BRN' WHERE `CostCenter_ID`='EEEPZZA1';
UPDATE `CostCenter` SET `TaskNamePrefix`='BRNCHW' WHERE `CostCenter_ID`='EEEPZZA1_CAP_HW';
UPDATE `CostCenter` SET `TaskNamePrefix`='BRNCSW' WHERE `CostCenter_ID`='EEEPZZA1_CAP_SW';
UPDATE `CostCenter` SET `TaskNamePrefix`='PRG' WHERE `CostCenter_ID`='EEEPZZA2';
UPDATE `CostCenter` SET `TaskNamePrefix`='PRGCHW' WHERE `CostCenter_ID`='EEEPZZA2_CAP_HW';
UPDATE `CostCenter` SET `TaskNamePrefix`='PRGCSW' WHERE `CostCenter_ID`='EEEPZZA2_CAP_SW';
UPDATE `CostCenter` SET `TaskNamePrefix`='D&S' WHERE `CostCenter_ID`='EESDZZC4';
UPDATE `CostCenter` SET `TaskNamePrefix`='D&SGNCOE' WHERE `CostCenter_ID`='EESDZZC5';
ALTER TABLE `CostCenter` ADD UNIQUE (`TaskNamePrefix`);

/* New cost center */
INSERT INTO `CostCenter` (`CostCenter_ID` , `Name` ,`TaskNamePrefix`)
VALUES ('EEGRZZC1', 'EHCOE - Board Layout', 'EHCOE');

ALTER TABLE `Task` ADD `Project_ID` INT NOT NULL AFTER `SAP_ID`;
UPDATE `Task` SET `Project_ID`=`GetProjectIDFromTaskID`(`Task_ID`);
ALTER TABLE `Task` ADD CONSTRAINT `Task_ibfk_3` FOREIGN KEY (`Project_ID`) REFERENCES `Project` (`Project_ID`) ON DELETE CASCADE;

--- We need to ensure consistency between WorkPackage_ID and Project_ID
DELIMITER //
DROP FUNCTION IF EXISTS `GetProjectIDFromWorkPackageID`//
CREATE FUNCTION `GetProjectIDFromWorkPackageID`(WPID INT) RETURNS int(11)
    DETERMINISTIC
BEGIN
        DECLARE WID INT;
        DECLARE PID INT;
        IF ((SELECT WorkPackage_ID FROM WorkPackage WHERE WorkPackage_ID = WPID) IS NULL) THEN
            SET PID = NULL;
            RETURN PID;
        ELSE
            SET PID = (SELECT Project_ID FROM WorkPackage WHERE WorkPackage_ID = WPID);
			SET WID = WPID;
            WHILE (PID IS NULL) DO
                SET WID = (SELECT Parent_ID FROM WorkPackage WHERE WorkPackage_ID = WID);
                SET PID = (SELECT Project_ID FROM WorkPackage WHERE WorkPackage_ID = WID);
            END WHILE;
            RETURN PID;
        END IF;
END//

CREATE TRIGGER `SetTaskProjectID` BEFORE INSERT ON `Task` FOR EACH ROW 
BEGIN
        SET NEW.`Project_ID`=`GetProjectIDFromWorkPackageID`(NEW.`WorkPackage_ID`);
END//

CREATE TRIGGER `UpdateTaskProjectID` BEFORE UPDATE ON `Task` FOR EACH ROW 
BEGIN
	IF (NEW.`WorkPackage_ID`<>OLD.`WorkPackage_ID`) THEN
        SET NEW.`Project_ID`=`GetProjectIDFromWorkPackageID`(NEW.`WorkPackage_ID`);
	END IF;
END//
DELIMITER ;

--- Fixing of Person ID foreign key constraints
ALTER TABLE `Project` DROP FOREIGN KEY `Project_fk`;
ALTER TABLE `Project` ADD CONSTRAINT `Project_fk` FOREIGN KEY (`TM`) REFERENCES `Person` (`EID`) ON DELETE SET NULL ON UPDATE CASCADE;
ALTER TABLE `Project` DROP FOREIGN KEY `Project_ibfk_1`;
ALTER TABLE `Project` ADD CONSTRAINT `Project_ibfk_1` FOREIGN KEY (`PIPM`) REFERENCES `Person` (`EID`) ON DELETE SET NULL ON UPDATE CASCADE;
ALTER TABLE `Project` DROP FOREIGN KEY `Project_ibfk_2`;
ALTER TABLE `Project` ADD CONSTRAINT `Project_ibfk_2` FOREIGN KEY (`PMOE`) REFERENCES `Person` (`EID`) ON DELETE SET NULL ON UPDATE CASCADE;
ALTER TABLE `Project` DROP FOREIGN KEY `Project_ibfk_3`;
ALTER TABLE `Project` ADD CONSTRAINT `Project_ibfk_3` FOREIGN KEY (`PPaC`) REFERENCES `Person` (`EID`) ON DELETE SET NULL ON UPDATE CASCADE;

ALTER TABLE `Hour` DROP FOREIGN KEY `Hour_ibfk_1`;
ALTER TABLE `Hour` ADD CONSTRAINT `Hour_ibfk_1` FOREIGN KEY (`EID`) REFERENCES `Person` (`EID`) ON DELETE CASCADE ON UPDATE CASCADE;

ALTER TABLE `Person_Skill` DROP FOREIGN KEY `Person_Skill_ibfk_1`;
ALTER TABLE `Person_Skill` ADD CONSTRAINT `Person_Skill_ibfk_1` FOREIGN KEY (`EID`) REFERENCES `Person` (`EID`) ON DELETE CASCADE ON UPDATE CASCADE;

ALTER TABLE `Person_Task` DROP FOREIGN KEY `Person_Task_ibfk_1`;
ALTER TABLE `Person_Task` ADD CONSTRAINT `Person_Task_ibfk_1` FOREIGN KEY (`EID`) REFERENCES `Person` (`EID`) ON DELETE CASCADE ON UPDATE CASCADE;

ALTER TABLE `Setting` DROP FOREIGN KEY `Setting_fk`;
ALTER TABLE `Setting` ADD CONSTRAINT `Setting_fk` FOREIGN KEY (`EID`) REFERENCES `Person` (`EID`) ON DELETE CASCADE ON UPDATE CASCADE;

ALTER TABLE `User` ADD CONSTRAINT `User_fk` FOREIGN KEY (`EID`) REFERENCES `Person` (`EID`) ON DELETE CASCADE ON UPDATE CASCADE;

--- Constraint for cascade deleting from Person_Task table if TaskPlan (for Task_ID and Year) is deleted
ALTER TABLE `TaskPlan` ADD UNIQUE `uniq_Task_ID_Year` (`Task_ID`, `Year`);
ALTER TABLE `Person_Task` CHANGE `Year` `Year` YEAR(4) NOT NULL;
ALTER TABLE `Person_Task` CHANGE `Month` `Month` INT(11) NOT NULL;
ALTER TABLE `Person_Task` ADD  CONSTRAINT `Person_Task_ibfk_3` FOREIGN KEY (`Task_ID`, `Year`) REFERENCES `TaskPlan` (`Task_ID`, `Year`) ON DELETE CASCADE;

--- eCharge credentials into database
INSERT INTO `GlobalSetting` (`Key`, `Value`, `Default`, `DataType`) VALUES
('eCharge.urlBase', 'http://elcsaero.honeywell.com/CET/', '', 'string'),
('eCharge.credentials.user', NULL, '', 'string'),
('eCharge.credentials.password', NULL, '', 'string'),
('eCharge.actualsUpdate.startDate', NULL, '', 'string');
