<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * MaintenancePresenter presents data and forms for application
 * maintenance tasks like database backup, logging etc
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 * @todo Remove coupling with Dibi if possible.
 */
class MaintenancePresenter extends SecuredPresenter
{

	private $zeroTask = false;
	private $assignedPerson = false;
	/**
	 * Renders default info.
	 */
	public function renderDefault()
	{
	}
	
	/**
	 * Renders system chceck.
	 */
	public function renderSystemCheck()
	{
		$year = Date('Y');
		if($this->zeroTask)
		{
			Debugger::log('ZeroTaskCheck render.', Debugger::DEBUG); 
			$zeroTask = $this->context->systemCheckRepository->findZeroTaskCheck($year);
			if(!is_null($zeroTask))
				$this->template->zeroTask = $zeroTask;
			else
				$this->template->zeroTask = 'none';
		}
		$this->zeroTask = false;
		
		if($this->assignedPerson)
		{
			Debugger::log('assignedPersonCheck render.', Debugger::DEBUG); 
			$assignedPerson = $this->context->systemCheckRepository->findAssignedPersonCheck($year);
			if(!is_null($assignedPerson))
				$this->template->assignedPerson = $assignedPerson;
			else
				$this->template->assignedPerson = 'none';
		}
		$this->assignedPerson = false;
		$this->template->form = $this['systemCheckForm'];
	}
	
	/**
	 * Factory method for creating component into template.
	 * Creates form for running the backup & restore operation. 
	 */
	public function createComponentBackupRestoreDatabaseForm()
	{
		$form = new AppForm();
		$form->addSubmit('process', 'Backup & Restore');
		return $form;
	}	
	
	/**
	 * Renders a form for starting DB backup.
	 */
	public function renderBackupRestoreDatabaseForm()
	{
 		if (!$this->testUserRoles('ADMIN'))
		{
			$this->flashMessage("You are not authorized to use this functionality.", 'info');
			$this->redirect('default');
		}

		$gsettings = $this->context->globalSetting;
		if ($gsettings->get('menu.import.BackupAndRestore.enabled') !== TRUE)
		{
			$this->flashMessage('Operation not allowed!', 'error');
			$this->redirect('default');
		}
		else
		{
			$this->template->targetDatabase = dibi::getDatabaseInfo()->name;
			$source_database = $gsettings->get('menu.import.BackupAndRestore.sourcedb');
			if ($source_database != '')
			{
				$this->template->sourceDatabase = $source_database;
				$this->template->form = $this['backupRestoreDatabaseForm'];
				$this->template->form->getElementPrototype()->action = '/' . TOOL_DIR . '/?presenter=Maintenance&action=backupRestoreDatabase';
			}
			else
				$this->flashMessage('Source database is not set! Cannot run operation.', 'warning');
		}
	}

	/**
	 * Renders a form for starting DB backup.
	 */
	public function actionBackupRestoreDatabase()
	{
		$gsettings = $this->context->globalSetting;
		if ($gsettings->get('menu.import.BackupAndRestore.enabled') === TRUE)
		{
			$src_db_name = $gsettings->get('menu.import.BackupAndRestore.sourcedb');
			$target_db_name = $this->template->targetDatabase = dibi::getDatabaseInfo()->name;
			if (strpos(strtolower(dibi::getConnection()->getDriver()->getClass()), 'mysql') !== FALSE)
			{
				$root_dir = Environment::getVariable('tempDir') . '/backup-restore';
				if (!file_exists($root_dir))
					if (!mkdir($root_dir))
					{
						$this->flashMessage('Failed to create working directory: '.$root_dir, 'error');
						$this->redirect('default');
					}
				$backup_script = $root_dir . '/mysqldump-' . date('Ymd-His') . '.sql';
				$backup_errors = $root_dir . '/mysqldump-' . date('Ymd-His') . '.err';
				$db_user = dibi::getConnection()->getConfig('username');
				$db_password = dibi::getConnection()->getConfig('password');
				// step 1: dump source database
				$exec_command = 'mysqldump -R -h localhost -u '.$db_user.' -p'.$db_password.' '.$src_db_name.' > ' . $backup_script . ' 2> ' . $backup_errors;
				exec($exec_command, $exec_output, $exec_return_value);
				if ($exec_return_value > 0)
				{
					$this->flashMessage('Backup of database '.$src_db_name.' failed! Skipping restore.', 'error');
					Debugger::log('Backup of database '.$src_db_name.' failed! See error log: ' . $backup_errors, Debugger::WARNING);
					$this->redirect('default');
				}
				else // dump was ok => import
				{
					$this->flashMessage('Backup of database '.$src_db_name.' was successful.', 'success');
					Debugger::log('Backup of database '.$src_db_name.' was successful. Dump file: ' . $backup_script, Debugger::INFO);
					if (dibi::isConnected())
						dibi::getConnection()->disconnect();
					// step 2: import to current database
					$exec_command = 'mysql -h localhost -u '.$db_user.' -p'.$db_password.' '.$target_db_name.' < ' . $backup_script . ' 2>> ' . $backup_errors;
					exec($exec_command, $exec_output, $exec_return_value);
					if ($exec_return_value > 0)
					{
						$this->flashMessage('Restore of database '.$target_db_name.' failed! CONTACT ADMNISTRATOR TO FIX DATABASE!!!', 'error');
						Debugger::log('Restore of database '.$target_db_name.' failed! Data may be invalid! See error log: ' . $backup_errors, Debugger::ERROR);
					}
					else
					{
						Debugger::log('Restore of database '.$target_db_name.' was successfull! Source dump file: '.$backup_script, Debugger::INFO);
						$this->flashMessage('Restore of database '.$target_db_name.' was successful.', 'success');
						// restore global setting
						$gsettings->set('menu.import.BackupAndRestore.sourcedb', $src_db_name, true);
						$gsettings->set('menu.import.BackupAndRestore.enabled', true,  true, IGlobalSetting::OPTVAL_BOOLEAN);
					}
					$this->redirect('default');
				}
			}
			else
			{
				$this->flashMessage('Operation works only for MySql database!', 'error');
				$this->redirect('default');
			}
		}
		else
		{
			$this->flashMessage('Operation not allowed!', 'error');
			$this->redirect('default');
		}
	}

	/**
	 * Factory method for creating grid with legacy log.
	 */
	protected function createComponentLogGrid()
	{
		$table_name = $this->context->parameters['logger']['dbtable'];
		return new LogDataGrid($this->context->database->table($table_name));
	}
	
	/**
	 * Factory method for creating grid with legacy log.
	 */
	protected function createComponentLegacyLogGrid()
	{
		return new LegacyLogGrid($this->context->database->table("Log"));
	}
	
	/**
	 * Renders log grid.
	 */
	public function renderLog()
	{

	}
	
	/**
	 * Factory method for creating component into template.
	 * Creates form for running the backup & restore operation. 
	 */
	public function createComponentCleanLogForm()
	{
		$form = new AppForm();
        $form->addText('DateUpto', 'Delete log up to')
            ->addCondition(Form::FILLED, TRUE)
            ->addRule(Form::PATTERN, '%value is not valid date!', '2[0-9]{3}-(0[0-9]{1}|1[012]{1})-([012]{1}[0-9]|3[01]{1})');
        $form['DateUpto']->getControlPrototype()->addClass('datepicker');
		$form->addSubmit('process', 'Clean Logs');
        $form->onSuccess[] = callback($this, 'cleanLogFormSubmitted');
		return $form;
	}	

	/**
	 * Cleans up log
	 * @param string	$dateTo	Date upto which logs are to be deleted.
	 */
	public function cleanLogFormSubmitted(AppForm $form)
	{
 		if (!$this->testUserRoles('ADMIN'))
		{
			$this->flashMessage("You are not authorized to use this functionality. ADMIN role required.", 'info');
			$this->redirect('log');
		}
		//TODO: use Log repository (instead of dibi access)
		$values = $form->getValues();
		if (dibi::query('DELETE FROM [LogData] WHERE [Created] <= %d', $values['DateUpto']))
		{
			$this->context->createLogger(NULL, "")->log('Log cleanup up to '.$values['DateUpto'].' was successful.', ILogger::NOTICE);
			$this->flashMessage('Log cleanup up to '.$values['DateUpto'], 'success');
		}
		else
			$this->flashMessage('Log cleanup up to '.$values['DateUpto'].' failed.', 'error');
	}
	
	/**
	 * Renders log grid.
	 */
	public function renderTest()
	{
		$project_id = 34;
		$tree = $this->context->workPackageRepository->getProjectWBSTree($project_id);
		echo '<table>';
		echo '<tr><th>Type</td><th>SAP ID</td><td><b>Element Name</b></td><td>Level</td></tr>';
		foreach ($tree->getTreeIterator(false) as $item)
			echo '<tr><td>'.$item->getWBSType() . '</td><td>'.$item->getSAPID() . '</td><td><b>' . $item->getName() . '</b></td><td>'. $item->getLevel() . '</td></tr>';
		echo '</table>';
		
		exit;
	}

	/**
	 * Factory method for creating component into template.
	 * Creates form buttons for system chcek. 
	 */
	public function createComponentSystemCheckForm()
	{
		$form = new AppForm();
		
		$form->addSubmit('assignedPersonCheck', 'Check')
				->setValidationScope(FALSE)
                ->onClick[] = callback($this, 'assignedPersonCheckClicked');
        if (!$this->testUserRoles('ADMIN'))        
            $form['assignedPersonCheck']->setDisabled(TRUE);
		
		$form->addSubmit('zeroTaskCheck', 'Check')
				->setValidationScope(FALSE)
                ->onClick[] = callback($this, 'zeroTaskCheckClicked');
        if (!$this->testUserRoles('ADMIN'))        
            $form['zeroTaskCheck']->setDisabled(TRUE);
		
		$form->getElementPrototype()->class('ajax');
		
		return $form;
	}
	
    /**
     * Assigned person button check if people are assigned only on the task with the same work center.
     * 
     * @param SubmitButton $button Assigned person check button.
     */
    public function assignedPersonCheckClicked(SubmitButton $button)
    {
		if($this->isAjax())
		{
			$this->assignedPerson = true;
			$this->invalidateControl('assignedPersonCheck');
			Debugger::log('assignedPersonCheckClicked by AJAX.', Debugger::DEBUG); 
		}
    }
	
    /**
     * Zero task button check if there are tasks with zero hours effort and zero budget having task plan for current year in the database.
     * 
     * @param SubmitButton $button Zero task button.
     */
    public function zeroTaskCheckClicked(SubmitButton $button)
    {
		if($this->isAjax())
		{
			$this->zeroTask = true;
			$this->invalidateControl('zeroTaskCheck');
			Debugger::log('zeroTaskCheckClicked by AJAX.', Debugger::DEBUG); 
		}
    }
	
}

