<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2016 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Presents interface for importing data from iASAP.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class TLCActualsImportPresenter extends SecuredPresenter
{
    /**
     * Service for iASAP TLC import.
     * @var IASAPActualsImportServiceTLC
     */
	private $iASAPActualsImportServiceTLC;
    
    /**
     * Nette default startup method 
     */
    public function startup()
    {
        parent::startup();
		
        if (!$this->isAllowed(self::RESOURCE_IMPORT, self::ACTION_VIEW))
        {
            $this->flashMessage('You don\'t have permission to use TLC actuals import. You were redirected.');
            $this->redirect('Dashboard:');
        }
        
    }
    
    /**
     * Nette default render method 
     */
	public function renderDefault()
	{
        $this->template->lastUpdated = $this->context->actualsRepository->getLastImportDate(Actuals::SOURCE_IASAP_TIMEADMIN);
        $this->template->maxExecTime = IASAPActualsImportServiceTLC::MAXIMUM_EXECUTION_TIME;
		$lock = $this->context->createLock(IASAPActualsImportServiceTLC::LOCK_KEY);
		$this->template->running = $lock->locked();
	}
    
    /**
     * Run method 
     */
    public function renderRunTLC()
    {
		if (!$this->isAllowed(self::RESOURCE_IMPORT, self::ACTION_RUN))
        {
            $this->flashMessage('You don\'t have permission to use iASAP actuals import. You were redirected.');
            $this->redirect('Dashboard:');
        }
		
		// init logging
		$logs_suffix = time().'-'. rand(1, 9);
		$current_log_file_name = 'tlc_actuals_import-'.$logs_suffix.'.html';
		$batch_id = 'tlc_actuals_import-'.$logs_suffix;
		$filelog_path = TEMP_DIR . DIRECTORY_SEPARATOR . $current_log_file_name;
        $filelog = $this->context->createFileLogger($filelog_path, $batch_id, "TLC ACTUALS IMPORT") ;
		$multilog = new MultiLogger();
		$multilog->add($filelog);
		$multilog->add($this->context->createLogger($batch_id, "TLC ACTUALS IMPORT"));

		//  time admin table import
        $this->iASAPActualsImportServiceTLC = new IASAPActualsImportServiceTLC($this->context, $multilog);
        
        try
        {
			switch ($this->iASAPActualsImportServiceTLC->runTimeAdminImport())
			{
				case IASAPActualsImportServiceTLC::RESULT_SUCCESS:
					$this->flashMessage('TLC (time admin) actuals import was successful.', 'success');
					break;
				case IASAPActualsImportServiceTLC::RESULT_NO_DATA:
					$this->flashMessage('TLC (time admin) actuals import was successful but no actuals was found.', 'warning');
					break;
				case IASAPActualsImportServiceTLC::RESULT_WARNING:
					$this->flashMessage('TLC (time admin) actuals import was successful with warnings.', 'warning');
					break;
				case IASAPActualsImportServiceTLC::RESULT_ERROR:
					$this->flashMessage('TLC (time admin) actuals import failed!', 'error');
					break;
				default: break;
			}
        }
        catch (IASAPActualsImportServiceTLCException $e)
        {
			$this->flashMessage("TLC (time admin) actuals import failed: " . $e->getMessage(), 'error');
			$multilog->log('TLC (time admin) actuals import failed: ' . $e->getMessage(), ILogger::ERROR);
			Debugger::log('TLC (time admin) actuals import failed: ' . $e->getMessage(), Debugger::ERROR);
		}
		
		// ysp detail csc table import
//        $this->iASAPActualsImportServiceTLC = new IASAPActualsImportServiceTLC($this->context, $multilog);
//        
//        try
//        {
//			switch ($this->iASAPActualsImportServiceTLC->runYPSDetailImport())
//			{
//				case IASAPActualsImportServiceTLC::RESULT_SUCCESS:
//					$this->flashMessage('TLC (ysp detail csc) actuals import was successful.', 'success');
//					break;
//				case IASAPActualsImportServiceTLC::RESULT_NO_DATA:
//					$this->flashMessage('TLC (ysp detail csc) actuals import was successful but no actuals was found.', 'warning');
//					break;
//				case IASAPActualsImportServiceTLC::RESULT_WARNING:
//					$this->flashMessage('TLC (ysp detail csc) actuals import was successful with warnings.', 'warning');
//					break;
//				case IASAPActualsImportServiceTLC::RESULT_ERROR:
//					$this->flashMessage('TLC (ysp detail csc) actuals import failed!', 'error');
//					break;
//				default: break;
//			}
//        }
//        catch (IASAPActualsImportServiceTLCException $e)
//        {
//			$this->flashMessage("TLC (ysp detail csc) actuals import failed: " . $e->getMessage(), 'error');
//			$multilog->log('TLC (ysp detail csc) actuals import failed: ' . $e->getMessage(), ILogger::ERROR);
//			Debugger::log('TLC (ysp detail csc) actuals import failed: ' . $e->getMessage(), Debugger::ERROR);
//		}
        $filelog->close();
		
        $this->template->lastUpdated    = $this->context->actualsRepository->getLastImportDate(Actuals::SOURCE_IASAP_TIMEADMIN);
        $this->template->logfilePath    = basename(TEMP_DIR) . '/' . $current_log_file_name;
		$this->template->batchNumber	= $batch_id;
    }
}
