<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Presenter for exporting tracking table.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class TrackingTablePresenter extends SecuredPresenter
{
	
    /**
     * Nette default render method.
     */
    public function renderDefault()
    {
        $formDefaultReport = array();
		$lock = $this->context->createLock('TT_Report');
        $downloadReport = $this['downloadForm'];
		if($lock->locked())
		{
			$downloadReport['download']->setDisabled(TRUE);
			$this->flashMessage('Report is generated right now.', 'notice');
		}
        $downloadReport->setDefaults($formDefaultReport);
        $this->template->downloadReport = $downloadReport;
		
		$dir = null;
        if (file_exists(Environment::getConfig('trackingTable_storage_dir'))) 
			$dir = opendir(Environment::getConfig('trackingTable_storage_dir'));
		$files = array();
		//List files in directory
		if($dir != null)
		{
			while (($file = readdir($dir)) !== false)
			{
				if(preg_match('#.xlsx#', $file))
				{
					$files[$file]['file'] = $file;
					$files[$file]['modDate'] = date("Y-d-m H:i:s.", filemtime(Environment::getConfig('trackingTable_storage_dir') . $file));
				}
			}
			closedir($dir); 
		}
		
		$this->template->path = Environment::getConfig('trackingTable_storage_dir');
		$this->template->files = $files;
    }

    /**
     * Creates form for download.
     * 
     * @param string $name Name of the form.
     * @return AppForm Returns AppForm.
     */
    public function createComponentDownloadForm($name)
    {
        $form = new AppForm($this, $name);
        $years = array();
		for ($i = 2012; $i <= (date('Y')+1); $i++)
			$years[$i] = $i;
        $form->addHidden('download_token_value_id');
		$form->addSelect('Year', 'Year', $years)->setDefaultValue(date('Y'));
		
		$allSbe = $this->context->organizationUnitRepository->findAllOrgUnitByType(OrganizationUnit::ORG_UNIT_TYPE_SBE);
        $sbes = array();
		foreach ($allSbe as $sbe)
		{
			$sbes[$sbe->ID] = $sbe->Name;
		}
        $form->addSelect('sbe', 'Strategic Business Unit', $sbes)->setDefaultValue(date('Y'));
        $form->addSubmit('download', 'Regenerate Tracking Table');
        $form->onSuccess[] = array($this, 'downloadFormSubmitted');
    }

    /**
     * Component downloadForm submit download tracking table.
     * 
     * @param AppForm $form 
     */
    public function downloadFormSubmitted(AppForm $form)       
    {
        $values = $form->getValues();
		$year = $values['Year'];
		$sbe = $this->context->organizationUnitRepository->find($values['sbe']);
		if(is_null($sbe))
		{
			$this->flashMessage('SBE: ' . $values['sbe'] . ' was not found.', 'error');
			$this->redirect('default');
		}
		$multilog = new MultiLogger();
		$batch_id = 'TT-'.time().'-'. rand(1, 9);
		$multilog->add($this->context->createLogger($batch_id, "TT manual generation"));
        $TTReportService = new TTReportService($this->context, $multilog, $year, $sbe, $values['download_token_value_id']);
        try
        {
			switch ($TTReportService->runTrackingTableGeneration())
			{
				case IASAPActualsImportServicePO::RESULT_SUCCESS:
				{
					$multilog->log('TT manual generation completed.', ILogger::NOTICE);
					$this->flashMessage('Report regeneration was successful.', 'success');
					break;
				}
				case IASAPActualsImportServicePO::RESULT_NO_DATA:
					$this->flashMessage('TT has no data to report.', 'warning');
					break;
				case IASAPActualsImportServicePO::RESULT_WARNING:
					//not applicable
					break;
				case IASAPActualsImportServicePO::RESULT_ERROR:
					$multilog->log('TT manual generation completed.', ILogger::NOTICE);
					$this->flashMessage('Report regeneration was successful.', 'success');
					$this->flashMessage('There is/are missing work center or labour rate. Calculated values may not be correct. For more info see menu item Maintenance -> Log Viewer or contact HON PT administrator.', 'error');
					break;
				default: break;
			}
		}
        catch (Exception $e)
        {
            Debugger::log('TT manual generation failed: ' . $e->getMessage(), Debugger::ERROR);
			$multilog->log('TT manual generation failed: ' . $e->getMessage(), ILogger::ERROR);
			$this->flashMessage('TT manual generation failed: ' . $e->getMessage(), 'error');
		}
		$this->redirect('default');
    }
}

