<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Presenter for work package structure.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class WBSPresenter extends SecuredPresenter
{
    /**
     * Nette default render method.
     */
	public function renderDefault($id, $year)
	{
        $project_id = $id;
        if($year == null)
            $year = date("Y");

        $availableYears = $this->context->projectRepository->findProjectYears($project_id);
        $this->template->AvailableYears = $availableYears;
		if (!in_array($year, $availableYears))
		{
			$alternative_year = 0;
			if (in_array($year + 1, $availableYears))
				$alternative_year = $year + 1;
			elseif (in_array($year - 1, $availableYears))
				$alternative_year = $year - 1;
			elseif (count($availableYears))
				$alternative_year = $availableYears[0]; // first year of project
			if ($alternative_year != 0)
			{
				$this->flashMessage("There is no plan for current year $year. Year $alternative_year selected.", 'info');
				$this->redirect('default', 
					array(	'id' => $project_id, 
							'year' => $alternative_year
						));
			}
			else
				$this->redirect('Error:404');
		}
		
        $setting = $this->context->userSettingRepository->find(Environment::getUser()->getId());
        $project = $this->context->projectRepository->find($project_id);
        $reporter = new ProjectEACReporter($this->context, $project, $year);
        $this->template->ActualYear = $year;
        $this->template->id = $project_id;
        $eac_start_from_month = $reporter->getFirstEACMonth();
        
        $this->template->project =  $project->Name;
        $this->template->SAP_ID =  $project->SAP_ID;

        $wbsTree = $this->context->workPackageRepository->getProjectWBSTree($project_id);
        $items = array();
        $costCenters = $this->context->costCenterRepository->findAllCostCenters();
        
        foreach ($wbsTree->getTreeIterator(false) as $item)
        {
            if($item->getWBSType() == "workpackage" AND $item->getSAPID() != null AND substr($item->getName(), 0, 7) != "NOT USE")
            {
                $items[] = array("lvl" => $item->getLevel(),
                                    "type" => $item->getWBSType(),
                                    "name" => $item->getName(),
                                    "SAP_ID" => $item->getSAPID(),);
            }
            elseif($item->getWBSType() == "task" AND substr($item->getName(), 0, 7) != "NOT USE")
            {      
                $task_plan = $this->context->taskPlanRepository->findTaskYearPlan($item->getData()->ID, $year);  
                if($task_plan != NULL)                
                    $task_plan_id = $task_plan->taskPlanID;                
                else
                    $task_plan_id = NULL;
                if ($task_plan === NULL) // task is not planned but has some work/costs for this year
                    $task_plan = new TaskYearPlan($year);
                
                // work for labour tasks
                $task_plan_values = $task_actuals = $task_deltas = array();
                if($item->getData()->isLabourTask())
                    $task_work = $reporter->getTaskLabourEACHoursMonthTotals($item->getData());
                else
                    $monthly_task_costs = $this->context->costCenterRepository->findActualCostsYearSummary($item->getData()->ID, $year, 'USD');
                for ($i = 1; $i <= 12; $i++)
                {
                    if($item->getData()->isLabourTask())
                    {
                        $task_plan_values[$i] = $task_plan->getMonthPlan($i)->Hours;
                        $task_actuals[$i] = $task_work[$i];
                        $task_deltas[$i] = $task_plan->getMonthPlan($i)->Hours - $task_work[$i];
                        $costShare = $item->getData()->costShare;
                        $isLabour = true;
                        $costCenter = (isset($costCenters[$item->getData()->CostCenterID])) ? $costCenters[$item->getData()->CostCenterID]->Name : '-';
                        $costCenterID = (isset($costCenters[$item->getData()->CostCenterID])) ? $costCenters[$item->getData()->CostCenterID]->ID : NULL;
// TODO How to get technical group??
//                        $techGroup = $this->context->taskRepository->findTechnicalGroup($item->getData()->ID);                       
                        $techGroup = NULL;                       
                    }
                    else
                    {
        				$task_plan_values[$i] = $task_plan->getMonthPlan($i)->Budget;
                        $task_actuals[$i] = $monthly_task_costs[$i];
                		if(($i > $eac_start_from_month AND $year == date("Y"))OR($year > date("Y")))
                            $task_deltas[$i] = 0;  // EAC for non-labour cost is same as plan -> delta is zero
                        else
                            $task_deltas[$i] = $task_plan->getMonthPlan($i)->Budget - $monthly_task_costs[$i];
                        $isLabour = false;
                        $costCenter = "-";
                        $costCenterID = NULL;
                        $costShare = $item->getData()->costShare;                       
                        $techGroup = NULL;
                    }
                }
                if($setting->WBSHideTasksWithoutEffort == 1)
                {
                    $sum = 0;
                    for($i = 1; $i<=12; $i++)
                    {
                        $sum += abs($task_plan_values[$i]) + abs($task_actuals[$i]);
                    }

                    if($sum == 0)
                    {
                        $task_deltas = "hide";
                    }
                }
                else
                {
                    $sum = 0;
                    for($i = 1; $i<=12; $i++)
                    {
                        $sum += abs($task_plan_values[$i]) + abs($task_actuals[$i]);
                    }

                    if($sum == 0)
                    {
                        $task_deltas = "zero";
                    }
                }
                
                $isTrv = null;
                if(substr($item->getName(), 0, 3) == "TRV")
                    $isTrv = true;
                
                $items[] = array("lvl" => $item->getLevel(),
                                    "type" => $item->getWBSType(),
                                    "name" => $item->getName(),
                                    "SAP_ID" => $item->getData()->getEChargeID(),
                                    "CONCUR_ID" => $item->getData()->getConcurID(),
                                    "costShare" => $costShare,
                                    "costCenter" => $costCenter,
                                    "costCenterID" => $costCenterID,
                                    "values" => $task_deltas,
                                    "techGroup" => $techGroup,
                                    "isLabour" => $isLabour,
                                    "isTrv" => $isTrv,
                                    "taskPlanID" => $task_plan_id,);

            }
        }
        $this->template->tree = $items;
        
        $this->template->WBSShowLabourTasksOnly = $setting->WBSShowLabourTasksOnly;
        $this->template->WBSHideTasksWithoutEffort = $setting->WBSHideTasksWithoutEffort;
        $this->template->WBSHideAllWP = $setting->WBSHideAllWP;
        
        $formDefaultOptions = array();
        $formDefaultOptions['WBSShowLabourTasksOnly'] = $setting->WBSShowLabourTasksOnly;
        $formDefaultOptions['WBSHideTasksWithoutEffort'] = $setting->WBSHideTasksWithoutEffort;
        $formDefaultOptions['WBSHideAllWP'] = $setting->WBSHideAllWP;
        $formOptions = $this['optionsForm'];
        $formOptions->setDefaults($formDefaultOptions);
        $this->template->formOptions = $formOptions;       
        
        $formDefaultReport = array();
        $buttonReport = $this['buttonForm'];
        $buttonReport->setDefaults($formDefaultReport);
        $this->template->buttonReport = $buttonReport;
	}         
    
    /**
     * Creates form for buttons.
     * 
     * @param string $name Name of the form.
     * @return AppForm Returns AppForm.
     */
    public function createComponentButtonForm($name)
    {
        $form = new AppForm($this, $name);
        $form->addSubmit('back', 'Back to project information')
            ->setValidationScope(FALSE)
            ->onClick[] = callback($this, 'backClicked');
        $form->addSubmit('plannedAndActuals', 'Report assigned resources and actuals')
            ->setValidationScope(FALSE)
            ->onClick[] = callback($this, 'plannedAndActualsClicked');
        $form->addSubmit('plannedAndEAC', 'Report assigned resources and EAC')
            ->setValidationScope(FALSE)
            ->onClick[] = callback($this, 'plannedAndEACClicked');
        $form->addSubmit('taskEACForSAPUpdate', 'Report task EAC for SAP update')
            ->setValidationScope(FALSE)
            ->onClick[] = callback($this, 'taskEACForSAPUpdateClicked');
        $form->addHidden('download_token_value_id');
    }
    
    /**
     * Creates form for user options(setting).
     * 
     * @param string $name Name of the form.
     * @return AppForm Returns AppForm.
     */
    public function createComponentOptionsForm($name)
    {
        $form = new AppForm($this, $name);
        $form->addCheckbox('WBSShowLabourTasksOnly', 'Show labour tasks only');
        $form->addCheckbox('WBSHideTasksWithoutEffort', 'Hide tasks with no effort for current year');
        $form->addCheckbox('WBSHideAllWP', 'Hide All workpackages')
                ->setAttribute('class', 'tree');;
        $form->addSubmit('Save', 'Save')
            ->setHtmlId("optionsSave")
            ->setValidationScope(FALSE);
        $form->onSuccess[] = array($this, 'optionsFormSubmittedSave');        
    }
    
    /**
     * Options(setting) form submitted.
     * 
     * @param AppForm $form 
     */
    public function optionsFormSubmittedSave($form)
    {
        $eid = Environment::getUser()->getId();
        $data = $form->getValues();
        $setting = new UserSetting();
        $setting = $this->context->userSettingRepository->find($eid);        
        $setting->setWBSHideTasksWithoutEffort($data['WBSHideTasksWithoutEffort']);
        $setting->setWBSHideAllWP($data['WBSHideAllWP']);
        $setting->setWBSShowLabourTasksOnly($data['WBSShowLabourTasksOnly']);
        if($form->isValid())
        {                                     
            if ($this->context->userSettingRepository->save($setting))
            {              
                Debugger::log("Setting of person: ".$eid." updated.", Debugger::INFO);
                $this->flashMessage("Work Breakdown Structure display options updated.", 'success');
            }            
        }
        else
        {  
            $this->flashMessage("Invalid setting values.");
        }
        $Project_ID = $this->getHttpRequest()->getQuery('id');
        $year = $this->getHttpRequest()->getQuery('year');
        $this->redirect('WBS:default', $Project_ID, $year);
    }
    
    public function taskEACForSAPUpdateClicked(SubmitButton $button)
    {
        $Project_ID = $this->getHttpRequest()->getQuery('id');
        $year = $this->getHttpRequest()->getQuery('year');
        $this->redirect('ProjectReport:ProjectTaskEACTSV', $Project_ID, $year);
    }
    
    public function plannedAndActualsClicked(SubmitButton $button)
    {
        $Project_ID = $this->getHttpRequest()->getQuery('id');
        $year = $this->getHttpRequest()->getQuery('year');
        $this->redirect('ProjectReport:ProjectAssignmentsAndPlansWithActualsXLS', $Project_ID, $year, time());
    }  
    
    public function plannedAndEACClicked(SubmitButton $button)
    {
        $Project_ID = $this->getHttpRequest()->getQuery('id');
        $year = $this->getHttpRequest()->getQuery('year');
        $this->redirect('ProjectReport:ProjectAssignmentsAndPlansWithEACXLS', $Project_ID, $year, time());
    }
    
    public function backClicked(SubmitButton $button)
    {
        $Project_ID = $this->getHttpRequest()->getQuery('id');
        $this->redirect('Project:detail', $Project_ID);
    }
}