<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * Presenter for worker resource plan.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class WorkerPlanPresenter extends SecuredPresenter
{
    /** @persistent */
    public $year;
    /** @persistent */
    public $person_id;
    
    /**
     * Nette default render method.
     */
    public function renderDefault($id, $year)
    {
        if($year == null)
            $this->year = date("Y");
        else
            $this->year = $year;
        $years = array();
		for ($i = 2010; $i <= (date('Y')+1); $i++)
			$years[$i] = $i;
        $actualMonth = FiscalCalendar::getFiscalMonthByTimestamp(time())->index+1;
        $this->template->availableYears= $years;
        $this->template->actualYear = $this->year;
        $this->person_id = $id;  
        
        /* --- all projects tasks hours of preson and total year sum--- */
		$projects = array();
		$projects = $this->context->projectRepository->findPersonAssignedAtProjectsInYear($this->person_id, $this->year); 
		// lest find projects where persons are not assigned
		$notAssignedProjects = $this->context->projectRepository->findPersonNotAssignedAtProjectsInYear($this->person_id, $this->year);
		foreach($notAssignedProjects as $project)
		{
			if(!isset($projects[$project->ID]))
				$projects[$project->ID] = $project;  
		}
        $assignedAndActualHours = array();
        $assignedAndActualHoursYearTotal = array(); 
        for($i = 1; $i < 13; $i++)
        {
            $assignedAndActualHoursYearTotal[$i] = 0;
        }
        $projectNames = array();
        $taskNames = array();
        $taskSapID = array();
        foreach ($projects as $project)
        {
            $projectNames[$project->ID] = $project->Name;
            $tasks = $this->context->taskRepository->findPersonTasksInYear($this->person_id, $project->ID, $this->year);
			$activeTasks = $this->context->taskRepository->findPersonActiveTasksInYear($this->person_id, $project->ID, $this->year);
			foreach($activeTasks as $task)
			{
				if(!isset($tasks[$task->ID]))
				{
					$tasks[$task->ID] = $task;
				}
			}
            foreach($tasks as $task)
            {
                $assigned = $this->context->personTaskAssignmentRepository->findPersonTaskYearAssignment($task->ID, $this->year, $this->person_id);
                $actual = $this->context->workRepository->findPersonTaskYearWork($this->person_id, $task->ID, $this->year);
                $taskNames[$task->ID] = $task->Name;
                $taskSapID[$task->ID] = $task->SAPID;
                foreach($assigned as $month => $hour)
                {
					if($year == Date('Y'))
					{
						if($month >= $actualMonth)
						{
							$assignedAndActualHours[$project->ID][$task->ID][$month] = $hour->Hours;
							$assignedAndActualHoursYearTotal[$month] += $hour->Hours;
						}
						else
						{
							$assignedAndActualHours[$project->ID][$task->ID][$month] = $actual[$month];
							$assignedAndActualHoursYearTotal[$month] += $actual[$month];
						}
					}
					elseif($year > Date('Y'))
					{
						$assignedAndActualHours[$project->ID][$task->ID][$month] = $hour->Hours;
						$assignedAndActualHoursYearTotal[$month] += $hour->Hours;
					}
					else
					{
						$assignedAndActualHours[$project->ID][$task->ID][$month] = $actual[$month];
						$assignedAndActualHoursYearTotal[$month] += $actual[$month];
					}
                }
            }
        }

        /* --- Total task hours sum --- */
        $assignedAndActualHoursTaskTotal = array();
        
        foreach ($assignedAndActualHours as $projectName => $project)
        {
            $assignedAndActualHoursTaskTotal[$projectName] = array();
            for($i = 1; $i < 13; $i++)
            {
                $assignedAndActualHoursTaskTotal[$projectName][$i] = 0;
            }
            foreach($project as $task)
            {
                foreach($task as $month => $hour)
                {
                    $assignedAndActualHoursTaskTotal[$projectName][$month] += $hour;
                }
            }
        }
        
        if($assignedAndActualHours == null)
        {
            $this->flashMessage("Resource is not planned for selected year.", 'info');
        }

        $this->template->workableHours = $this->context->aopRepository->findForPerson($this->year, $this->person_id);
        $this->template->assignedAndActualHours = $assignedAndActualHours;
        $this->template->projectNames = $projectNames;
        $this->template->taskNames = $taskNames;
        $this->template->taskSapID = $taskSapID;
        $this->template->assignedAndActualHoursTaskTotal = $assignedAndActualHoursTaskTotal;
        $this->template->assignedAndActualHoursYearTotal = $assignedAndActualHoursYearTotal;
        $this->template->person = $this->context->personRepository->find($this->person_id);
        $this->template->assignedAndActualHoursForm = $this['assignedAndActualHoursForm'];
    }
    
    /**
     * Creates form for assigned and actuals hours.
     * 
     * @param string $name Name of the form.
     * @return AppForm Returns AppForm.
     */
    public function createComponentAssignedAndActualHoursForm($name)
    {
        $form = new AppForm($this, $name);

        $actualYear = Date('Y');
        $actualMonth = FiscalCalendar::getFiscalMonthByTimestamp(time())->index+1;
       	$projects = array();
		$projects = $this->context->projectRepository->findPersonAssignedAtProjectsInYear($this->person_id, $this->year); 
		// lest find projects where persons are not assigned
		$notAssignedProjects = $this->context->projectRepository->findPersonNotAssignedAtProjectsInYear($this->person_id, $this->year);
		foreach($notAssignedProjects as $project)
		{
			if(!isset($projects[$project->ID]))
				$projects[$project->ID] = $project;  
		}
        $assignedAndActualHours = array();
        
        foreach ($projects as $project)
        {
            $tasks = $this->context->taskRepository->findPersonTasksInYear($this->person_id, $project->ID, $this->year);
			$activeTasks = $this->context->taskRepository->findPersonActiveTasksInYear($this->person_id, $project->ID, $this->year);
			foreach($activeTasks as $task)
			{
				if(!isset($tasks[$task->ID]))
				{
					$tasks[$task->ID] = $task;
				}
			}
            foreach($tasks as $task)
            {
                $assigned = $this->context->personTaskAssignmentRepository->findPersonTaskYearAssignment($task->ID, $this->year, $this->person_id);
                $actual = $this->context->workRepository->findPersonTaskYearWork($this->person_id, $task->ID, $this->year);

                foreach($assigned as $month => $hour)
                {
					if($this->year == Date('Y'))
					{
						if($month >= $actualMonth)
						{
							$assignedAndActualHours[$project->ID][$task->ID][$month] = $hour->Hours;
						}
						else
						{
							$assignedAndActualHours[$project->ID][$task->ID][$month] = $actual[$month];
						}
					}
					elseif($this->year > Date('Y'))
					{
						$assignedAndActualHours[$project->ID][$task->ID][$month] = $hour->Hours;
					}
					else
					{
						$assignedAndActualHours[$project->ID][$task->ID][$month] = $actual[$month];
					}
                }
            }
        }
        
        foreach($assignedAndActualHours as $projectName => $project)
        {
            foreach($project as $taskName => $task)
            {
                foreach($task as $month => $hour)
                {
                    if($this->year < $actualYear)
                    {
                        $form->addtext($projectName . '_' . $taskName . '_' . $month, NULL, 3)
                            ->setDefaultValue($assignedAndActualHours[$projectName][$taskName][$month])
                            ->setAttribute('readonly', 'readonly');
                    }
                    elseif ($this->year == $actualYear)
                    {
                        if($month >= $actualMonth)
                            $form->addText($projectName . '_' . $taskName . '_' . $month, NULL, 3)
                                    ->setDefaultValue($assignedAndActualHours[$projectName][$taskName][$month])
                                    ->addRule(Form::FLOAT, 'Assigned hours must be float.')
                                    ->addRule(Form::RANGE, 'Assigned hours must be positive number from 0 to 999.', array(0, 999));
                        else
                            $form->addtext($projectName . '_' . $taskName . '_' . $month, NULL, 3)
                                ->setDefaultValue($assignedAndActualHours[$projectName][$taskName][$month])
                                ->setAttribute('readonly', 'readonly');
                    }
                    else
                    {
                        $form->addText($projectName . '_' . $taskName . '_' . $month, NULL, 3)
                                ->setDefaultValue($assignedAndActualHours[$projectName][$taskName][$month])
                                ->addRule(Form::FLOAT, 'Assigned hours must be float.')
                                ->addRule(Form::RANGE, 'Assigned hours must be positive number from 0 to 999.', array(0, 999));
                    }
                }
            }
        }
        
        $form->addSubmit('save', 'Save');        
        if($this->year < date('Y'))
        $form['save']->setDisabled(TRUE);      
        $form->addSubmit('back', 'Go back')
                ->setValidationScope(FALSE)
                ->onClick[] = callback($this, 'backClicked');
        $form->onSuccess[] = array($this, 'assignedAndActualHoursFormSubmitted');
        
        return $form;
    }
    
    /**
     * Assigned and actuals hours form submitted.
     * 
     * @param AppForm $form 
     */
    public function assignedAndActualHoursFormSubmitted(AppForm $form)
    {
        $actualMonth = FiscalCalendar::getFiscalMonthByTimestamp(time())->index+1;
        $values = $form->getValues();
        $tasks_id = array();
        $assigned = array();
        $i = 0;

        foreach($values as $task_id => $value)
        {
            $assigned[] = $value;
            if(!($i % 12))
            {
                $a = strpos($task_id, '_') + 1;
                $b = strrpos($task_id, '_');
                $tasks_id[] = (substr($task_id, $a, $b - $a));
            }
            $i++;
        }
		$i = 0;
		$k = 0;
        $success = false;
        foreach($tasks_id as $task_id)
        {
			if(!($i % 12))
				$k = 0;
            $person_task = $this->context->personTaskAssignmentRepository->findPersonTaskYearAssignment($task_id, $this->year, $this->person_id);
            foreach($person_task as $month)
            {
                if ($k+1 >= $actualMonth && $this->year == Date('Y'))
                    $month->setHours($assigned[$i]);
				elseif($this->year > Date('Y'))
					$month->setHours($assigned[$i]);
				$k++;
                $i++;
            }
            $j = 1;
            foreach($person_task as $month)
            {
                if ($j >= $actualMonth && $this->year == Date('Y'))
                {
                    if ($this->context->personTaskAssignmentRepository->save($month))
                    {
                        $success = true;
                    }
                    else
                    {
                        $success = false;
                    }
                    if(!$success)
                        break 2;
                }
				elseif($this->year > Date('Y'))
                {
                    if ($this->context->personTaskAssignmentRepository->save($month))
                    {
                        $success = true;
                    }
                    else
                    {
                        $success = false;
                    }
                    if(!$success)
                        break 2;
                }
                $j++;
            }
        }

        if($success)
        {
            $this->flashMessage("Worker resource plan updated.", 'success');
        }
        else
        {
            $this->flashMessage("Worker resource plan updating failed.", 'error');
        }
            
        $this->redirect('WorkerPlan:default', $this->person_id, $this->year);
    }
    
    /**
     * Go back button redirects back to Person:default.
     * 
     * @param SubmitButton $button Go back button.
     */
    public function backClicked(SubmitButton $button)
    {
        $this->redirect('Person:default');
    }
}