<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package app
 */


// Load Nette framework
require LIBS_DIR . '/Nette/loader.php';
// Load Dibi library because we do not use dibi as service yet
//require LIBS_DIR . '/dibi/dibi.php';

// Configure application
$configurator = new Configurator;
//Server detection
$serverName = $_SERVER['SERVER_NAME'];
if (strpos($serverName, '-dev.') !== FALSE || strpos($serverName, 'cz27vwat13too') !== FALSE || strpos($serverName, '165.195.211.235') !== FALSE)
{
	$configurator->addParameters(array('serverType' => 'Development'));
	$configurator->setDebugMode(true);
}
elseif (strpos($serverName, '-test.') || strpos($serverName, 'cz27vwat12too') !== FALSE || strpos($serverName, '165.195.211.234') !== FALSE)
{
	$configurator->addParameters(array('serverType' => 'Test'));
	$configurator->setDebugMode(true);
}
elseif ('localhost' == $serverName)
{
	$configurator->addParameters(array('serverType' => 'Localhost'));
	$configurator->setDebugMode(true);
}
else
{
	$configurator->addParameters(array('serverType' => 'Production'));
}
// Enable Nette Debugger for error visualisation & logging
// Hard setup of production mode for debugging on localhost
//$configurator->setProductionMode(TRUE);
$configurator->enableDebugger(dirname(__FILE__) . '/../log');

// Enable RobotLoader - this will load all classes automatically
$configurator->setTempDirectory(TEMP_DIR);
$configurator->createRobotLoader()
	->addDirectory(APP_DIR)
	->addDirectory(LIBS_DIR)
	->register();

//creates sessions dir if not exists
if (!file_exists(TEMP_DIR . '/sessions'))
{
	mkdir(TEMP_DIR . '/sessions');
}

// need TOOL_DIR constant as parameter in to process config.neon
$configurator->addParameters(array('toolDir' => str_replace('//', '/', '/' . str_replace('\\', '/', TOOL_DIR) . '/')));
// need SESSION_DIR constant as parameter in to process config.neon
$configurator->addParameters(array('sessionDir' => SESSION_DIR));
// Create Dependency Injection container from config.neon file
$configurator->addConfig(dirname(__FILE__) . '/config.neon');

$container = $configurator->createContainer();

Stopwatch::start("Total request");

// setup database connection for Dibi library
dibi::connect($container->params['database']);

// registration of extension AddDynamic for forms
//Kdyby\Replicator\Container::register();

// Setup router
//$container->router[] = new Route('index.php', 'Dashboard:default', Route::ONE_WAY);
//$container->router[] = new Route('<presenter>/<action>[/<id>]', 'Dashboard:default');
//
// SimpleRouter is defined in config.neon
//$container->router = new SimpleRouter('Dashboard:default');
$container->application->errorPresenter = 'Error';

// extend FormContainer class of method for list of checkboxes
Form::extensionMethod('Form::addCheckboxList', array('CheckboxList', 'addCheckboxList'));
// extend FormContainer class of method for SelectBox2 and MultiSelectBox2
Form::extensionMethod('Form::addSelect2', array('SelectBox2', 'addSelect2'));
Form::extensionMethod('Form::addMultiSelect2', array('SelectBox2', 'addMultiSelect2'));
 
// Configure and run the application!
$container->application->run();
Stopwatch::stop("Total request");
