<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * ActualsCost (single record).
 *
 * @author Jiri Zajic <jiri.zajic@honeywell.com>
 */
class ActualsCost extends Object
{
	/** @const Definition of sources of data */
    const SOURCE_SAP = 0,
		  SOURCE_IASAP = 1,
		  SOURCE_MANUAL = 2;
	
    /** @var string Project SAP ID */
    private $project_sap_id;

    /** @var int Task ID */
    private $task_id;

    /** @var string Activity Name */
    private $activity_name;

    /** @var string Creation date */
    private $creation_date;

    /** @var string Posting date */
    private $posting_date;

    /** @var string Year */
    private $year;
    
    /** @var string Fiscal Month */
    private $fiscal_month;
    
    /** @var string Description */
    private $description;
    
    /** @var string Category */
    private $category;
    
    /** @var string Quantity */
    private $quantity;
    
    /** @var string Value in USD */
    private $value_usd;
    
    /** @var string Value in CZK */
    private $value_czk;
    
    /** @var string Value */
    private $value;
    
    /** @var string Currency */
    private $currency;
	
	/** @var int Data source */
	private $source;

    public function  __construct() 
    {
    }

    /**
     * Gets Project SAP ID.
     * 
     * @return string
     */
    public function getProject_sap_id()
    {
        return $this->project_sap_id;
    }	

    /**
     * Sets Project SAP ID.
     * 
     * @param string $project_sap_id
     */
    public function setProject_sap_id($project_sap_id)
    {
		if (preg_match('#[A-Z]|[a-z]{2}-{1}\d{6}#', $project_sap_id))
			$this->project_sap_id = iconv("CP1252", "UTF-8", $project_sap_id);
		else
			throw new InvalidArgumentException('Project SAP ID must be formatted like this example EC-002579. Wrong param: '.$project_sap_id);
    }

    /**
     * Gets Task_ID.
     * 
     * @return int
     */
    public function getTask_id()
    {
        return $this->task_id;
    }

    /**
     * Sets Task_ID.
     * 
     * @param int $task_id
     */
    public function setTask_id($task_id)
    {
		$this->task_id = $task_id;
    }

    /**
     * Gets ActivityName.
     * 
     * @return string $activity_name
     */
    public function getActivity_name()
    {
        return $this->activity_name;
    }

    /**
     * Sets ActivityName.
     * 
     * @param string $activity_name
     */
    public function setActivity_name($activity_name)
    {
        $this->activity_name = iconv("CP1252", "UTF-8", $activity_name);
    }

    /**
     * Gets CreationDate.
     * 
     * @return string $creation_date
     */
    public function getCreation_date()
    {
        return $this->creation_date;
    }

    /**
     * Sets CreationDate.
     * 
     * @param string $creation_date
     */
    public function setCreation_date($creation_date)
    {
        $this->creation_date = $creation_date;
    }

    /**
     * Gets PostingDate.
     * 
     * @return string $posting_date
     */
    public function getPosting_date()
    {
        return $this->posting_date;
    }

    /**
     * Sets PostingDate.
     * 
     * @param string $posting_date
     */
    public function setPosting_date($posting_date)
    {
        $this->posting_date = $posting_date;
    }

    /**
     * Gets Year.
     * 
     * @return int $year
     */
    public function getYear()
    {
        return $this->year;
    }

    /**
     * Sets Year.
     * 
     * @param int $year
     */
    public function setYear($year)
    {
        $this->year = $year;
    }

    /**
     * Gets FiscalMonth.
     * 
     * @return int $fiscal_month
     */
    public function getFiscal_month()
    {
        return $this->fiscal_month;
    }

    /**
     * Sets FiscalMonth.
     * 
     * @param int $fiscal_month
     */
    public function setFiscal_month($fiscal_month)
    {
        $this->fiscal_month = $fiscal_month;
    }

    /**
     * Gets Description.
     * 
     * @return string $description
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * Sets Description.
     * 
     * @param string $description
     */
    public function setDescription($description)
    {
        $this->description = iconv("CP1252", "UTF-8", $description);
    }

    /**
     * Gets Category.
     * 
     * @return string $category
     */
    public function getCategory()
    {
        return $this->category;
    }

    /**
     * Sets Category.
     * 
     * @param string $category
     */
    public function setCategory($category)
    {
        $this->category = iconv("CP1252", "UTF-8", $category);
    }

    /**
     * Gets Quantity.
     * 
     * @return int $quantity
     */
    public function getQuantity()
    {
        return $this->quantity;
    }

    /**
     * Sets Quantity.
     * 
     * @param int $quantity
     */
    public function setQuantity($quantity)
    {
        $this->quantity = $quantity;
    }

    /**
     * Gets Value USD.
     * 
     * @return float $value_usd
     */
    public function getValue_usd()
    {
        return $this->value_usd;
    }

    /**
     * Sets Value USD.
     * 
     * @param float $value_usd
     */
    public function setValue_usd($value_usd)
    {
        $this->value_usd = $value_usd;
    }

    /**
     * Gets Value CZK.
     * 
     * @return float $value_czk
     */
    public function getValue_czk()
    {
        return $this->value_czk;
    }

    /**
     * Sets Value CZK.
     * 
     * @param float $value_czk
     */
    public function setValue_czk($value_czk)
    {
        $this->value_czk = $value_czk;
    }

    /**
     * Gets Value.
     * 
     * @return float $value
     */
    public function getValue()
    {
        return $this->value;
    }	

    /**
     * Sets Value.
     * 
     * @param float $value
     */
    public function setValue($value)
    {
        $this->value = $value;
    }

    /**
     * Gets Currency.
     * 
     * @return string $currency
     */
    public function getCurrency()
    {
        return $this->currency;
    }	

    /**
     * Sets Currency.
     * 
     * @param string $currency
     */
    public function setCurrency($currency)
    {
        $this->currency = iconv("CP1252", "UTF-8", $currency);
    }	
	
	/**
	 * Sets source
	 * 
	 * @param int $source Should be in range of Actuals::SOURCE_*
	 * @throws InvalidArgumentException
	 */
	public function setSource($source)
	{
		if (in_array($source, self::getSources()))
		    $this->source = $source;
		else
			throw new InvalidArgumentException('Source must be in range of Actuals::SOURCE_*. Wrong param: ' . $source);
	}
	
	/**
	 * Gets source
	 * 
	 * @return int
	 */
	public function getSource()
	{
		return $this->source;
	}
	
	/**
	 * Gets all possible source values
	 * 
	 * @return array List of sources (Actuals::SOURCE_*)
	 */
	public static function getSources()
	{
		return array(
			self::SOURCE_SAP,
			self::SOURCE_IASAP,
			self::SOURCE_MANUAL
		);
	}

}
