<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Actuals Cost repository access class.
 *
 * @author Jiri Zajic <jiri.zajic@honeywell.com>
 */
class ActualsCostRepository extends Object
{
    /** @var IActualsCostMapper */
    private $mapper;

    public function __construct(IActualsCostMapper $mapper)
    {
        $this->mapper = $mapper;
    }	

    /**
     * Gets all actuals costs.
     * 
     * @return array array of ActualsCost
     */
    public function findAll()
    {
        return $this->mapper->findAll();
    }
    
    /**
     * Returns all actuals costs for single task
     * 
     * @return array Array of arrays (years) of arrays (fiscal months)
     * of Actuals Cost records for single task.
     */
    public function findAllForTask($id)
    {
        return $this->mapper->findAllForTask($id);
    }
	
    /**
     * Returns agregated actuals costs for single task
     * 
	 * @param int $id ID of a Task object
	 * @param string $currency Currency code (only USD or CZK accepted)
     * @return array Array of arrays (years) of arrays (fiscal months)
     * of Actuals Cost(just numbers)
     */
    public function findForTask($id, $currency)
    {
        return $this->mapper->findForTask($id, $currency);
    }
	
    /**
     * Gets single actuals cost record.
     * --- MOSTLY FOR DEBUGGING ---
     * 
     * @return ActualsCost ActualsCost single record
     */
    public function findSingle()
    {
        return $this->mapper->findSingle();
    }
    
    /**
	 * Returns all task years by task id.
	 * 
	 * @param int $id Task ID.
	 * @return array Task duration according to actuals.
	 */
	public function findTaskYears($id)
    {
        return $this->mapper->findTaskYears($id);
    }
	
	/**
	 * Function drops data from selected year.
	 * 
	 * @param int $year
	 * @return boolean Returns true if success, otherwise returns false.
	 */
	public function deleteDataFromYear($year, $source)
	{
		return $this->mapper->deleteDataFromYear($year, $source);
	}
	
	/**
	 * Deletes records associated with given charge numbers in given range.
	 * 
	 * @param DateTime $startDate Delete from.
	 * @param DateTime $endDate Delete to.
	 * @param array $taskIDs List of Task IDs.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function deleteIASAPImport(DateTime $startDate, DateTime $endDate, array $taskIDs)
	{
		return $this->mapper->deleteIASAPImport($startDate, $endDate, $taskIDs);
	}
    
	/**
	 * Function imports data from array where items are ActualsCost objects.
	 * 
	 * @param array $actualCostsArray array of ActualsCost
	 */
	public function importActualsCost($actualCostsArray)
	{
		return $this->mapper->importActualsCost($actualCostsArray);
	}
	
	/**
	 * Saves actuals cost
	 * 
	 * @param ActualsCost $actualsCost
	 * @return boolean Returns true if cost center is successfully saved, otherwise false.
	 */
	public function save(ActualsCost $actualsCost)
	{
		return $this->mapper->save($actualsCost);
	}
	
	/**
	 * Deletes records associated with given task ID.
	 * 
	 * @param int $taskID Task ID.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function deleteIASAPActualsByTaskID($taskID)
	{
		return $this->mapper->deleteIASAPActualsByTaskID($taskID);
	}
	
	/**
	 * Function drops data from selected year and for given task.
	 * 
	 * @param int $year
	 * @return boolean Returns true if success, otherwise returns false.
	 */
	public function deleteDataFromTaskYear($taskID, $year, $source)
	{
		return $this->mapper->deleteDataFromTaskYear($taskID, $year, $source);
	}
	
	/**
	 * Function drops data from selected year and for given task.
	 * 
	 * @param int $taskID
	 * @return boolean Returns true if success, otherwise returns false.
	 */
	public function deleteDataForTask($taskID)
	{
		return $this->mapper->deleteDataForTask($taskID);
	}
	
	/**
	 * Function drops data for given task.
	 * 
	 * @param int $taskID
	 * @param int $source Data source (e.g. SAP, IASAP, manual)
	 * @return boolean Returns true if success, otherwise returns false.
	 */
	public function deleteDataForTaskAndSource($taskID, $source)
	{
		return $this->mapper->deleteDataForTaskAndSource($taskID, $source);
	}
}
