<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2016 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing actuals sap temp from database using dibi.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class ActualsSAPTempDibiMapper extends DibiMapper implements IActualsSAPTempMapper
{

    /**
     * Creates Actuals instance from DibiRow.
     * 
     * @param DibiRow $row
     * @return	Actuals 
     * @todo rewrite using annotations
     */
    private function load(DibiRow $row)
    {
        $actuals = new ActualsSAPTemp();
        $actuals->Eid             = $row->EID;
        $actuals->Charge_number   = $row->ChargeNumber;
        $actuals->Year            = $row->Year;
        $actuals->Fiscal_month    = $row->FiscalMonth;
        $actuals->Hours           = $row->Hours;
        $actuals->Date            = $row->Date;
        $actuals->Project_sap_id  = $row->Project;
        $actuals->Source          = $row->Source;
        return $actuals;
    }	
	
	/**
	 * Deletes records associated with given task SAP ID.
	 * 
	 * @param String $sapID Task SAP ID.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function deleteActualsByTaskSAPID($sapID)
	{
		try
		{
			$db_result = $this->conn->delete('ActualsSapTemp')
					->where('[ChargeNumber] = %s', $sapID)
					->execute();
			
			if ($db_result)
			    Debugger::log('Deleting actuals hour for task with SAP ID: ' . $sapID, Debugger::INFO);
			elseif ($db_result !== 0)
				Debugger::log('Deleting actuals hour for task with SAP ID: ' . $sapID . ' failed', Debugger::ERROR);
			return ($db_result || ($db_result === 0)) ? TRUE : FALSE;
		}
		catch (DibiException $e)
		{
			Debugger::log('Deleting actuals hour for task with SAP ID: ' . $sapID . ' failed' . $e->getMessage(), Debugger::ERROR);
			return FALSE;
		}
	}
	
	/**
     * Moves actuals into ActualsSapTemp.
     * 
	 * @param String $sapID Task SAP ID.
     * @return boolean Returns true if move successfull, otherwiste returns false.
     */
    public function moveIntoActualsSapTemp($sapID)
    {
		try
		{
			$result = $this->conn->query('INSERT INTO [ActualsSapTemp]
												(EID, ChargeNumber, Year, FiscalMonth, Hours, Date, Project, Source) 
												SELECT * FROM [Actuals] WHERE [ChargeNumber] = %s', $sapID);
			Debugger::log("Result moving temp actuals: " . $result, Debugger::ERROR);
			if ($result >= 0)
			{
				Debugger::log("ActualsTemp was moved.", Debugger::INFO);
				$result = true;
			}
			else
				Debugger::log("Moving of ActualsTemp failed.", Debugger::ERROR);
			return $result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Moving of ActualsTemp failed: ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
    }
	
	/**
     * Moves actuals into Actuals.
     * 
	 * @param String $sapID Task SAP ID.
     * @return boolean Returns true if move successfull, otherwiste returns false.
     */
    public function moveIntoActuals($sapID)
    {
		try
		{
			$result = $this->conn->query('INSERT INTO [Actuals]
												(EID, ChargeNumber, Year, FiscalMonth, Hours, Date, Project, Source) 
												SELECT * FROM [ActualsSapTemp] WHERE [ChargeNumber] = %s', $sapID);
			Debugger::log("Result moving temp actuals: " . $result, Debugger::ERROR);
			if ($result >= 0)
			{
				Debugger::log("ActualsTemp was moved.", Debugger::INFO);
				$result = true;
			}
			else
				Debugger::log("Moving of ActualsTemp failed.", Debugger::ERROR);
			return $result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Moving of ActualsTemp failed: ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
    }
}