<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2015 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * ActualsTemp (single record).
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class ActualsTemp extends Object
{
	/** @const Definition of sources of data */
    const SOURCE_ECHARGE = 0,
		  SOURCE_IASAP = 1,
		  SOURCE_MANUAL = 2,
		  SOURCE_IASAP_TIMEADMIN = 3,
		  SOURCE_IASAP_YPSDETAIL = 4;
	
    /** @var string EID */
    private $eid;
    /** @var string ChargeNumber */
    private $charge_number;
    /** @var int Year */
    private $year;
    /** @var int FiscalMonth */
    private $fiscal_month;
    /** @var float Hours */
    private $hours;
    /** @var string DateTime */
    private $date;
    /** @var string Project */
    private $project_sap_id;	
	/** @var int Data source */
	private $source;	

    public function  __construct() 
    {
    }

    /**
     * Gets EID.
     * 
     * @return string
     */
    public function getEid()
    {
        return $this->eid;
    }	

    /**
     * Sets EID.
     * 
     * @param string $eid
     */
    public function setEid($eid)
    {
        $this->eid = $eid;
    }	

    /**
     * Gets ChargeNumber.
     * 
     * @return string
     */
    public function getCharge_number()
    {
        return $this->charge_number;
    }	

    /**
     * Sets ChargeNumber.
     * 
     * @param string $charge_number
     */
    public function setCharge_number($charge_number)
    {
        $this->charge_number = $charge_number;
    }	

    /**
     * Gets Year.
     * 
     * @return int
     */
    public function getYear()
    {
        return $this->year;
    }	

    /**
     * Sets Year.
     * 
     * @param int $year
     */
    public function setYear($year)
    {
        $this->year = $year;
    }	

    /**
     * Gets FiscalMonth.
     * 
     * @return int
     */
    public function getFiscal_month()
    {
        return $this->fiscal_month;
    }	

    /**
     * Sets FiscalMonth.
     * 
     * @param int $fiscal_month
     */
    public function setFiscal_month($fiscal_month)
    {
        $this->fiscal_month = $fiscal_month;
    }	

    /**
     * Gets Hours.
     * 
     * @return string
     */
    public function getHours()
    {
        return $this->hours;
    }	

    /**
     * Sets Hours.
     * 
     * @param float $hours
     */
    public function setHours($hours)
    {
        $this->hours = $hours;
    }	

    /**
     * Gets Date.
     * 
     * @return DateTime
     */
    public function getDate()
    {
        return date($this->date);
    }	

    /**
     * Sets Date.
     * 
     * @param DateTime $date
     */
    public function setDate(DateTime $date)
    {
        $this->date = $date;
    }	

    /**
     * Gets Project SAP ID.
     * 
     * @return string
     */
    public function getProject_sap_id()
    {
        return $this->project_sap_id;
    }	

    /**
     * Sets Project SAP ID.
     * 
     * @param string $project_sap_id
     */
    public function setProject_sap_id($project_sap_id)
    {
        $this->project_sap_id = $project_sap_id;
	}
	
	/**
	 * Sets source
	 * 
	 * @param int $source Should be in range of Actuals::SOURCE_*
	 * @throws InvalidArgumentException
	 */
	public function setSource($source)
	{
		if (in_array($source, self::getSources()))
		    $this->source = $source;
		else
			throw new InvalidArgumentException('Source must be in range of Actuals::SOURCE_*. Wrong param: ' . $source);
	}
	
	/**
	 * Gets source
	 * 
	 * @return int
	 */
	public function getSource()
	{
		return $this->source;
	}
	
	/**
	 * Gets all possible source values
	 * 
	 * @return array List of sources (Actuals::SOURCE_*)
	 */
	public static function getSources()
	{
		return array(
			self::SOURCE_ECHARGE,
			self::SOURCE_IASAP,
			self::SOURCE_MANUAL,
			self::SOURCE_IASAP_TIMEADMIN,
			self::SOURCE_IASAP_YPSDETAIL
		);
	}

}