<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2015 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing actualsTemp for iASA/ from database using dibi.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class ActualsTempForIASAPDibiMapper extends DibiMapper implements IActualsTempForIASAPMapper
{

    /**
     * Creates ActualsTempForIASAP instance from DibiRow.
     * 
     * @param DibiRow $row
     * @return	ActualsTempForIASAP 
     * @todo rewrite using annotations
     */
    private function load(DibiRow $row)
    {
        $actuals = new ActualsTempForIASAP();
        $actuals->Eid             = $row->EID;
        $actuals->Charge_number   = $row->ChargeNumber;
        $actuals->Year            = $row->Year;
        $actuals->Fiscal_month    = $row->FiscalMonth;
        $actuals->Hours           = $row->Hours;
        $actuals->Date            = $row->Date;
		$actuals->ChargeDay       = $row->ChargeDay;
		$actuals->ModifiedAt      = $row->ModifiedAt;
        $actuals->Comment         = $row->Comment;
        return $actuals;
    }	

    /**
     * Returns all actualsTempForIASAP.
     * 
     * @return array Associative array of ActualsTempForIASAP.
     */
    public function findAll()
    {
        $actuals_db_result = $this->conn->query('SELECT * FROM [ActualsTempForIASAP]')->fetchAll();
        $actuals = array();
        foreach ($actuals_db_result as $row)
        {
                $actual = $this->load($row);
                $actuals[] = $actual;
        }
        return $actuals;
    }
	
    /**
     * Returns number all actuals for iASAP.
     * 
     * @param DateTime $startDate
     * @param DateTime $endDate
     * @return int Number of stored actuals.
     */
    public function countActualsByRangeAndType(DateTime $startDate, DateTime $endDate)
    {
		if (((int) $startDate->format('U')) > ((int) $endDate->format('U'))) // PHP 5.2.0 compliant test
			return;
        $start_year = $startDate->format('Y');
        $end_year = $endDate->format('Y');
		$start_month = FiscalCalendar::getFiscalMonthByTimestamp((int) $startDate->format('U'))->getIndex();
		$end_month = FiscalCalendar::getFiscalMonthByTimestamp((int) $endDate->format('U'))->getIndex();
		$start_month_idx = ($start_year - 1) * 12 + $start_month + 1;
		$end_month_idx = ($end_year - 1) * 12 + $end_month + 1;
		
        $actuals_db_result = $this->conn->query('SELECT * FROM [ActualsTempForIASAP] WHERE
												(([Year] - 1) * 12 + [FiscalMonth] + 1) >= %i 
												AND (([Year] - 1) * 12 + [FiscalMonth] + 1) <= %i', 
												$start_month_idx,
												$end_month_idx)->fetchAll();
		$count = 0;
        foreach ($actuals_db_result as $row)
        {
            $count++;
        }
        return $count;
    }
	
	/**
     * Moves actualsTemp record for iASAP.
     * 
     * @return boolean Returns true if move successfull, otherwiste returns false.
     */
    public function moveAll()
    {
		try
		{
			$result = $this->conn->query('INSERT INTO [ActualsForIASAP]
												(EID, ChargeNumber, Year, FiscalMonth, Hours, Date, Comment, ChargeDay, ModifiedAt) 
												SELECT * FROM [ActualsTempForIASAP]');
			Debugger::log("Result moving temp actuals: " . $result, Debugger::ERROR);
			if ($result)
				Debugger::log("ActualsTemp was moved.", Debugger::INFO);
			else
				Debugger::log("Moving of ActualsTemp failed.", Debugger::ERROR);
			return $result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Moving of ActualsTemp failed: ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
    }
	
    /**
     * Deletes all actualsTemp fro iASAP.
     * 
	 * @return boolean
     */
    public function deleteAll()
    {
		try
		{
		    $result = $this->conn->delete('ActualsTempForIASAP')->execute();
			if ($result >= 0)
			{
				Debugger::log("ActualsTemp for iASAP was deleted.", Debugger::INFO);
				$result = true;
			}
			else
				Debugger::log("Deleting of ActualsTemp for iASAP failed.", Debugger::ERROR);
			return $result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting of ActualsTemp for iASAP failed: ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
    }

    /**
     * Saves new ActualsTempForIASAP record.
     * 
     * @param ActualsTemp $actual
     * @return boolean Returns true if save successfull, otherwiste returns false.
     */
    public function save(ActualsTempForIASAP $actual)
    {
        try
        {
            $array = array();
            $array['EID'] = $actual->Eid;
            $array['ChargeNumber'] = $actual->Charge_number;
            $array['Year'] = $actual->Year;
            $array['FiscalMonth'] = $actual->Fiscal_month;
            $array['Hours'] = $actual->Hours;
			$array['ChargeDay'] = $actual->ChargeDay;
			$array['ModifiedAt'] = $actual->ModifiedAt;
            $array['Comment'] = $actual->Comment;
            $db_result = $this->conn->insert('ActualsTempForIASAP', $array)->execute();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Saving actualsTemp record failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
    }
}
