<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009 - 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Repository for accessing AOPs.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class AOPRepository extends Object
{
	/** @var IAOPMapper */
    private $mapper;

    public function __construct(IAOPMapper $mapper)
    {
        $this->mapper = $mapper;
    }
    
	/**
	 * Returns AOP for selected year.
	 * 
	 * @param int $year year
	 * @return	AnnualOperationPlan of NULL if not found.
	 */
	public function find($year)
	{
		return $this->mapper->find($year);
	}

	/**
	 * Returns AOP for selected year for selected user.
	 * 
	 * Returns whole budget of hours which doesn't mean it have to be vacant
	 * (depends on assignments of hours to worker).
	 * 
	 * @param int $year year
	 * @param int $person_id ID of worker/employee.
	 * @return	AnnualOperationPlan of NULL if not found.
	 */
	public function findForPerson($year, $person_id)
	{
		return $this->mapper->findForPerson($year, $person_id);
	}
	
    public function getYears()
    {
        return $this->mapper->getYears();
    }
}
