<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2014 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Repository of task assignments
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class AssignmentsCostRepository extends Object
{
	/** @var IAssignmentsCostMapper */
    private $mapper;
	
	/** @var MultiLogger */
	private $multilog;

    public function __construct(IAssignmentsCostMapper $mapper)
    {
        $this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'AssignmentsCost-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "Assignments Cost"));
    }

	/**
	 * Saves costs task assignment.
	 * 
	 * @param AssignmentsCost $assignment
	 * @return	boolean Returns TRUE if success, otherwise FALSE.
	 */
	public function save(AssignmentsCost $assignment)
	{
		if ($this->mapper->save($assignment))
		{
			$this->multilog->log('Assignments Cost with ID: ' . $assignment->ID . ' saved.', ILogger::INFO);
			return TRUE;
		}
		
		return FALSE;
	}
	
	/**
	 * Finds month assignment by id of assignment.
	 * 
	 * @param int $id task assignment ID.
	 * @return	AssignmentsCost 
	 */
	public function find($id)
	{
		return $this->mapper->find($id);
	}

	/**
	 * Deletes assignment for task in given year.
	 * 
	 * @param int $task_id Task ID.
	 * @param int $year Year.
	 * @return bool TRUE on success, FALSE otherwise.
	 */
	public function deleteByYear($task_id, $year)
	{
		if ($this->mapper->deleteByYear($task_id, $year))
		{
			$this->multilog->log('Assignments Cost with Task ID: ' . $task_id . ' for Year: ' . $year . ' deleted.', ILogger::INFO);
			return TRUE;
		}
		
		return FALSE;
	}
	
	/**
	 * Returns array of arrays of twelve task assigments 
	 * that is assigned to given task ID in given year.
	 * 
	 * Each assignment (inner array) corresponds to a fiscal month in a year.
	 * Month indexes are 1 (January) - 12 (December). 
	 * If month assignment is missing only empty object is available on index.
	 * 
	 * @param int $task_id Task ID.
	 * @param int $year year.
	 * @return	array or 12 AssignmentsCost 
	 *					where keys are month indexes from 0 to 11.
	 */
	public function findTaskYearAssignments($task_id, $year)
	{
		return $this->mapper->findTaskYearAssignments($task_id, $year);
	}
	
    /**
     * Returns all assignments costs for single task
     * 
     * @return array Array of arrays (years) of arrays (fiscal months)
     * of Assignments Cost records for single task.
     */
    public function findAllForTask($taskID)
    {
		return $this->mapper->findAllForTask($taskID);
    }
}
