<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * CostCenterYearLabourRate represents hourly rates (prices) of labour
 * in cost center in a year.
 * Labour rate can be diffrent for every month in a year.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class CostCenterLabourRates extends Object
{
	/** @var string */
	private $id;
	
	/** @var int */
	private $year;
	
	/** @var array Rates of labour work per hour */
	private $rates;
	
	public function __construct()
	{
		for ($i = 1; $i <= 12; $i++)
			$this->rates[$i] = 0;
	}

    /**
     * Sets cost center ID.
     * 
     * @param $id string
     */
	public function setID($id)
	{
		$this->id = $id;
	}

    /**
     * Gets cost center ID.
     * 
     * @return string
     */
	public function getID()
	{
		return $this->id;
	}	
	
	/**
     * Set year.
     * 
     * @param int $year	Must be positive integer or 0.
     */
	public function setYear($year)
	{
		if (is_numeric($year) && ($year >= 0))
			$this->year = 0 + $year;
		else
			throw new InvalidArgumentException('Year must be integer greater or equal to 0. Wrong param: '.$year);
	}

    /**
     * Gets year.
     * 
     * @return int
     */
	public function getYear()
	{
		return $this->year;
	}

	/**
	 * Gets price of labour per hour in specified month.
	 * 
	 * @param int $month
	 * @return float
	 */
	public function getMonthlyHourRate($month)
	{
		if (is_int($month) && ($month >= 1) && ($month <= 12))
			return $this->rates[$month];
		else
			throw new InvalidArgumentException('Month must be spedified as a number in range <1,12>');
	}

	/**
	 * Sets price of labour per hour in specified month.
	 * 
	 * @param int $month
	 * @param float $rate
	 * @return float
	 */
	public function setMonthlyHourRate($month, $rate)
	{
		if (is_int($month) && ($month >= 1) && ($month <= 12) && (is_numeric($rate)))
			$this->rates[$month] = 0.0 + $rate;
		else
			throw new InvalidArgumentException('Month must be spedified as a number in range <1,12>. Rate must be numeric value.');
	}
	
	
	
}