<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Interface for finding exchange rates from one currency to another.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
interface IExchangeRateMapper
{

	/**
	 * Find exchange rate between currencies in defined month in a year.
	 * 
	 * @param string $source_curr	Source currency (3-letter ISO code)
	 * @param string $target_curr	Target currency (3-letter ISO code)
	 * @param int $month
	 * @param int $year
	 * @return CurrencyExchangeRate or NULL in unknown.
	 */
	public function find($source_curr, $target_curr, $month, $year);

	/**
	 * Finds exchange rates between currencies in defined year.
	 * Objects with undefined rates may be returned if rate is unknown
	 * of undefined for a month.
	 * 
	 * @param string $source_curr	Source currency (3-letter ISO code)
	 * @param string $target_curr	Target currency (3-letter ISO code)
	 * @param int $year
	 * @return MonthArray of CurrencyExchangeRate
	 */
	public function findYearRates($source_curr, $target_curr, $year);
	
}


