<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Interface for accessing organization unit.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
interface IOrganizationUnitMapper 
{
	/**
	 * Saves Organization unit.
	 * 
	 * @param OrganizationUnit $orgUnit
	 */
	public function save(OrganizationUnit $orgUnit);
    
	/**
	 * Updates Organization unit.
	 * 
	 * @param OrganizationUnit $orgUnit
	 */
	public function update(OrganizationUnit $orgUnit);
	
	/**
	 * Updates parent id of set of organization units.
	 * Used for move of children to another parent after delete of original parent.
	 * 
	 * @param array $children Of organization unit ids.
	 * @param int $parent_id New parent organization unit id.
	 */
	public function updateParentInBatch(array $children, $parent_id);
    
	/**
	 * Deletes Organization unit.
	 * 
	 * @param int $id organization unit ID.
	 */
	public function delete($id);
	
	/**
	 * Finds Organization unit by id.
	 * 
	 * @param int $id organization unit ID.
	 * @return	Organizaiton unit or NULL if not found.
	 */
	public function find($id);
        
        /**
	 * Gets all Organization units by type.
	 * 
     * @param string Type of org unit
	 * @return array Array of OrganizationUnit where Organization unit ID is key.
	 */
	public function findAllOrgUnitByType($type);
	
}
