<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Interface for mapper accessing ProjectCharterBudgetHistory.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
interface IProjectCharterBudgetHistoryMapper
{
    /**
	 * Saves new project charter budget  history record.
	 * 
	 * @param ProjectCharterBudgetHistory $projectCharterBudgetHistory
	 * @return boolean
	 */
	public function save(ProjectCharterBudgetHistory $projectCharterBudgetHistory);

	/**
	 * Updates existing project charter budget  history record.
	 * Updates only attributes that are updatable.
	 * 
	 * @param ProjectCharterBudgetHistory $projectCharterBudgetHistory
	 * @return boolean
	 */
	public function update(ProjectCharterBudgetHistory $projectCharterBudgetHistory);
	
	/**
	 * Deletes existing project charter budget  history record.
	 * 
	 * @param int $projectCharterID
	 * @param int $version
	 * @return boolean
	 */
	public function delete($projectCharterID, $version);

	/**
	 * Gets project charter budget  history record by ids.
	 * 
	 * @param int $projectCharterID
	 * @param int $version
	 * @return Project
	 */
	public function find($projectCharterID, $version);
	
	/**
	 * Gets project charter budget history record, which is the last one
	 * submitted by certain person (indentified by EID). It's possible this record
	 * won't be the latest one from all histories for this charter,
	 * as someone else (CL - line manager) could have created a new budget
	 * version and attain the latest budget status.
	 * 
	 * But for the EID supplied, it will.
	 * 
	 * @param int $projectCharterID
	 * @param int $EID
	 * @return ProjectCharterBudgetHistory
	 */
	public function findLatestVersionByEID($projectCharterID, $EID);
	
    /**
	 * Gets project charter budget history record with latest version.
	 * 
	 * @param int $projectCharterID
	 * @return ProjectCharterBudgetHistory
	 */
	public function findLatestVersion($projectCharterID);
        
	/**
	 * Initializes flag of latest version
	 * 
	 * @return boolean
	 */
	public function initLatestVersion($projectCharterID);
}
