<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Interface for mapper accessing ProjectCharterFileHistory.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
interface IProjectCharterFileHistoryMapper
{
    /**
	 * Saves new project charter file history record.
	 * 
	 * @param ProjectCharterFileHistory $projectCharterFileHistory
	 * @return boolean
	 */
	public function save(ProjectCharterFileHistory $projectCharterFileHistory);

	/**
	 * Updates existing project charter file history record.
	 * 
	 * Updates only attributes that are updatable.
	 * 
	 * @param ProjectCharterFileHistory $projectCharterFileHistory
	 * @return boolean
	 */
	public function update(ProjectCharterFileHistory $projectCharterFileHistory);
	
	/**
	 * Deletes existing project charter file history record.
	 * 
	 * @param int $projectCharterID
	 * @param int $version
	 * @return boolean
	 */
	public function delete($projectCharterID, $version);

	/**
	 * Gets project charter file history record by ids.
	 * 
	 * @param int $projectCharterID
	 * @param int $version
	 * @return Project
	 */
	public function find($projectCharterID, $version);
        
    /**
	 * Gets project charter file history record with latest version.
	 * 
	 * @param int $projectCharterID
	 * @return ProjectCharterFileHistory
	 */
	public function findLatestVersion($projectCharterID);
        
	/**
	 * Initializes flag LatestVersion
	 * @return boolean
	 */
	public function initLatestVersion($projectCharterID);
}
