<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Interface for accessing project charter.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
interface IProjectCharterMapper 
{
	/**
	 * Saves Project charter.
	 * 
	 * @param ProjectCharter $projectCharter
	 */
	public function save(ProjectCharter $projectCharter);
    
	/**
	 * Updates Project charter.
	 * 
	 * @param ProjectCharter $projectCharter
	 */
	public function update(ProjectCharter $projectCharter);
    
	/**
	 * Deletes Project charter.
	 * 
	 * @param int $id Project charter ID.
	 */
	public function delete($id);
	
	/**
	 * Finds Project charter by id.
	 * 
	 * @param int $id Project charter ID.
	 * @return	Project charter or NULL if not found.
	 */
	public function find($id);
	
	/**
	 * Finds planning year of charter specified by its ID.
	 * 
	 * @param int $projectCharterID
	 * @return mixed ProjectPlanningCharter or NULL
	 */
	public function findYearByProjectCharterID($projectCharterID);
        
	/**
	 * Gets ProjectCharter by ProjectCharterFilter. Returns every project charters that matches all
	 * set conditions.
	 *
	 * @param ProjectCharterFilter $charterFilter
	 * @return array of ProjectCharter
	 */
	public function getChartersByFilter(ProjectCharterFilter $charterFilter);
}
