<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2014 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Interface for accessing SettingDisplayedContent.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
interface ISettingDisplayedContentMapper
{
	/**
	 * Saves setting.
	 * @param SettingDisplayedContent $setting
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function save(SettingDisplayedContent $setting);
    
	/**
	 * Updates setting.
	 * @param SettingDisplayedContent $setting
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function update(SettingDisplayedContent $setting);
    
	/**
	 * Deletes setting.
	 * @param string $eid User EID.
	 * @param int $orgUnitID Organization unit ID.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function delete($eid, $orgUnitID);
	
	/**
	 * Finds setting.
	 * @param string $eid User EID.
	 * @param int $orgUnitID Organization unit ID.
	 * @return	SettingDisplayedContent or NULL if not found.
	 */
	public function find($eid, $orgUnitID);
	
	/**
	 * Finds setting by EID.
	 * @param string $eid User EID.
	 * @return array Array of SettingDisplayedContent or empty array.
	 */
	public function findByEID($eid);
	
}