<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Interface for accessing work packages.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
interface IWorkPackageMapper
{
	/**
	 * Saves new WorkPackage.
	 * 
	 * @param WorkPackage $wp New Workpackage to add.
	 * @return	boolean 
	 */
	public function save(WorkPackage $wp);

	/**
	 * Updates existing WorkPackage.
	 * 
	 * @param WorkPackage $wp Existing WorkPackage.
	 * @return	boolean 
	 */
	public function update(WorkPackage $wp);

	/**
	 * Finds work package by id.
	 * 
	 * @param int $id work package ID.
	 * @return	WorkPackage or NULL if not found.
	 */
	public function find($id);

	/**
	 * Finds work package by SAP id.
	 * 
	 * @param string $id work package SAP ID.
	 * @return	WorkPackage or NULL if not found.
	 */
	public function findBySAPID($workpackage_id);

	/**
	 * Returns array of all WorkPackages that are in Project WBS.
	 * 
	 * @param int $project_id Project ID.
	 * @return	array of WorkPackage 
	 */
	public function getProjectWorkPackages($project_id);

	/**
	 * Returns array of WorkPackages on the path
	 * in the WorkPackage tree from root to the task
	 * specified by $task_id.
	 * 
	 * @param int $task_id Task ID.
	 * @return	array of WorkPackage 
	 * @throws InvalidArgumentException if task ID is invalid.
	 */
	public function getWPTreePathByTaskID($task_id);
	
	/**
	 * Deletes existing WorkPackage.
	 * 
	 * @param int $workpackage_ID
	 * @return boolean
	 */
	public function delete($workpackage_ID);
}

