<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Class provides specific methods for sending e-mails especially for project charters.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class ProjectCharterMailer extends Mailer
{
	/** @var array Options for project charter mailer */
	private $options;
	
	public function __construct($commonOptions, $options)
    {
		parent::__construct($commonOptions);
		$this->options = $options;
	}
	
	
	public function sendApprovalRequest($charter, $emailContent, $link, $copy)
	{
		$user = $this->context->personRepository->find(Environment::getUser()->getId());
		$lineManager = $this->context->personRepository->find($charter->LineManager);
		$recipients = array();
		
		$approverHistory = $this->context->projectCharterApproverHistoryRepository->findLatestVersion($charter->getID());
		if ($approverHistory)
		{
			$approvers = $this->context->projectCharterApproverRepository->findAllApprover($charter->getID(), $approverHistory->getVersion());
			if ($approvers)
			{
				
				if ($this->DevelopmentMode)
				{
					foreach($this->TesterList as $tester)
						$recipients[] = $tester;
				}
				else
				{
					foreach ($approvers as $approver)
						$recipients[] = $approver->getEmail();
					
					if ($copy)
						array_push($recipients, $user->getEmail());
				}
			}
		}
		
		$arrCharter = array();
		$arrCharter['Name'] = $charter->getName();
		$arrCharter['Owner'] = $user->getFullName();
		$arrCharter['Owner_Full'] = $user->getReversedFullNameWithEID();
		$arrCharter['Year'] = $charter->getYear();
		$arrCharter['FinalApprovalDate'] = $approverHistory->getFinalApprovalDateFormatted();
		
		$template = $this->getTemplate(__DIR__ . $this->options['template_storage_dir_from_models'] . $this->options['template_approval_request']);
		$template->showWholeTemplate = TRUE;
		$template->charter = $arrCharter;
		if ($emailContent)
		    $template->emailContent = $emailContent;
		$template->link = $link;
		
		// compose mail
		$mail = $this->getMail();
		$mail->setSubject($this->options['subject_approval_request']);
		$mail->setHtmlBody($template);

		foreach ($recipients as $recipient)
			$mail->addTo(strtolower($recipient));

		if (!$this->DevelopmentMode)
			$mail->addCc($lineManager->Email);
		
		$mail->send();
	}
	
	
	public function sendNoticeStatusChange($charter, $link)
	{
		$user = $this->context->personRepository->find($charter->getOwner());
		
		$arrCharter = array();
		$arrCharter['Name'] = $charter->getName();
		
		$template = $this->getTemplate(__DIR__ . $this->options['template_storage_dir_from_models'] . $this->options['template_notice_status_change']);
		$template->charter = $arrCharter;
		$template->link = $link;
		
		// compose mail
		$mail = $this->getMail();
		$mail->setSubject($this->options['subject_notice_status_change']);
		$mail->setHtmlBody($template);
		
		if ($this->DevelopmentMode)
		{
			foreach($this->TesterList as $tester)
				$mail->addTo($tester);
		}
		else
		{
		    $mail->addTo($user->getEmail());
		}

		$mail->send();
	}
	
	
}
