<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Monthly plan of work in budget and available hours.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class MonthPlan extends Object
{
	/** @var int Count of workable hours. */
	private $hours = 0;
	
	/** @var int Available budget. */
	private $budget = 0;
	
    /**
     * Creates new month plan.
     * 
     * @param int $hours	Count of available hours. Non-negative integer.
     * @param int $budget	Available budget. Non-negative integer.
     */
	public function __construct($hours = 0, $budget = 0)
	{
		$this->setHours($hours);
		$this->setBudget($budget);
	}

    /**
     * Gets available hours in month.
     * 
     * @return int
     */
    public function getHours()
    {
        return $this->hours;
    }
	
    /**
     * Sets available hours in month.
	 * Rounds to integer.
     * 
     * @param int $hours
     */
    public function setHours($hours)
    {
		if (is_numeric($hours))
			$this->hours = $hours;
		else
			throw new InvalidArgumentException('Hours must be integer number.');
    }
	
    /**
     * Gets available hours in month.
     * 
     * @return int
     */
    public function getBudget()
    {
        return $this->budget;
    }
	
    /**
     * Sets available hours in month.
	 * Rounds to integer.
     * 
     * @param int $hours
     */
    public function setBudget($budget)
    {
		if (is_numeric($budget))
			$this->budget = 0 + $budget; // to int
		else
			throw new InvalidArgumentException('Budget must be integer number.');
    }
}



