<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2014 Honeywell International spol. s r.o.
 *
 * For the full copyright and licence information, please view
 * the file licence.txt that was distributed with this source code.
 * @package models
 */

/**
 * Nationality repository access class.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class NationalityRepository
{
	/** @var INationalityMapper */
 	private $mapper;
	
	/** @var MultiLogger */
	private $multilog;
	
	public function __construct(INationalityMapper $mapper)
    {
        $this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'ExportControl-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "ExportControl"));
    }
	
	/**
	 * Saves Nationality.
	 * @param Nationality $nationality
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function save(Nationality $nationality)
	{
		if ($this->mapper->save($nationality))
		{
			$this->multilog->log('Licence with ID: ' . $nationality->NationalityID . ' saved.', ILogger::INFO);
			return TRUE;
		}
		
		return FALSE;
	}
	
	/**
	 * Updates Nationality.
	 * @param Nationality $nationality
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function update(Nationality $nationality)
	{
		$oldObject = $this->find($nationality->NationalityID);
		if ($this->mapper->update($nationality))
		{
			$this->multilog->log('Licence with ID: ' . $nationality->NationalityID . ' updated. ' . $this->multilog->differences($oldObject, $nationality), ILogger::INFO);
			return TRUE;
		}
		
		return FALSE;
	}
	
	/**
	 * Deletes Nationality.
	 * @param int $id Nationality id.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function delete($id)
	{
		if ($this->mapper->delete($id))
		{
			$this->multilog->log('Licence with ID: ' . $id . ' deleted.', ILogger::INFO);
			return TRUE;
		}
		
		return FALSE;
	}
	
	/**
	 * Finds Nationality by id.
	 * @param int $id Nationality id.
	 * @return Nationality or NULL if not found.
	 */
	public function find($id)
	{
		return $this->mapper->find($id);
	}
	
	/**
	 * Finds all Nationalities.
	 * @return array Array of Nationality or empty array.
	 */
	public function findAll()
	{
		return $this->mapper->findAll();
	}
	
	/**
	 * Finds Nationality by name.
	 * @param string $name
	 * @return Nationality or NULL if not found.
	 */
	public function findByName($name)
	{
		return $this->mapper->findByName($name);
	}
}