<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * News is a message for users of tool.
 * HTML Text with annotations of the text.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class News extends Object
{
   
    /** @var int News identification number */
    private $id;

    /** @var string Date of news publishing */
    private $publishDate;

    /** @var string News title */
    private $title;

    /** @var string Short non HTML annotation of text */
    private $annotation;

    /** @var string HTML text of news */
    private $text;

    public function  __construct() 
    {
    }
	
    /**
     * Sets news ID.
     * 
     * @param $id int
     */
	public function setID($id)
	{
		$this->id = 0 + $id;
	}

    /**
     * Gets news ID.
     * 
     * @return int
     */
	public function getID()
	{
		return $this->id;
	}	
	
    /**
     * Sets news title. Max 100 characters (less in UTF-8).
     * 
     * @param string $title
     */
	public function setTitle($title)
	{
		if (strlen($title) > 250)
			throw new InvalidArgumentException('Maximum string lenght of annotation is 100 chars/bytes.');
		else
			$this->title = $title;
	}

    /**
     * Gets news title.
     * 
     * @return string
     */
	public function getTitle()
	{
		return $this->title;
	}	

    /**
     * Sets annotation of news text. Max 250 characters (less in UTF-8).
     * 
     * @param string $annotation
     */
	public function setAnnotation($annotation)
	{
		if (strlen($annotation) > 250)
			throw new InvalidArgumentException('Maximum string lenght of annotation is 250 chars/bytes.');
		else
			$this->annotation = $annotation;
	}

    /**
     * Gets annotation of news text.
     * 
     * @return string
     */
	public function getAnnotation()
	{
		return $this->annotation;
	}	

    /**
     * Sets HTML text of news.
     * 
     * @param string $text
     */
	public function setText($text)
	{
		$this->text = $text;
	}

    /**
     * Gets HTML text of news.
     * 
     * @return string
     */
	public function getText()
	{
		return $this->text;
	}	

    /**
     * Sets publish date in format: YYYY-MM-DD HH:MM:SS
     * 
     * @param string $date date in format: YYYY-MM-DD HH:MM:SS
     */
	public function setPublishDate($date)
	{
		if (FiscalCalendar::isValidDateTime($date))
			$this->publishDate = $date;
		else
			throw new InvalidArgumentException('Invalid publish date value: ' . $date);
	}

    /**
     * Gets publish date in format: YYYY-MM-DD HH:MM:SS
     * 
     * @return string
     */
	public function getPublishDate()
	{
		return $this->publishDate;
	}	

    /**
     * Sets publish date as UNIX timestamp.
     * 
     * @param int $ts 
     */
	public function setPublishDateTS($ts)
	{
		$this->publishDate = date('Y-m-d H:i:s', $ts);
	}

    /**
     * Gets publish date as UNIX timestamp.
     * 
     * @return int
     */
	public function getPublishDateTS()
	{
		return FiscalCalendar::datetimeToTimestamp($this->publishDate);
	}	
	
}

