<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Class for authentication of a user via LDAP.
 */
class PersonEIDchange
{
	/** server parameters for LDAP authentication */
    private $server = '';
    private $port = '';
    private $basedn = '';
    private $basepass = '';
    private $searchdn = '';
    private $filter = '';

	/**
	 * Options injection from application configuration.
	 */
	public function setOptions($options)
	{
		$this->server = (isset($options['server'])) ? $options['server'] : '';
		$this->port  = (isset($options['port'])) ? $options['port'] : '';
		$this->basedn = (isset($options['basedn'])) ? $options['basedn'] : '';
		$this->basepass = (isset($options['basepass'])) ? $options['basepass'] : '';
		$this->searchdn = (isset($options['searchdn'])) ? $options['searchdn'] : '';
		$this->filter = (isset($options['filter'])) ? $options['filter'] : '';
	}
	
    /**
     * Performs authentication against LDAP.
     *
     * @param  $eid Employee eid.
     * @return boolean Returns true if EID found in LDAP, otherwise false.
     */
    public function authenticate($eid)
    {
        $username = $eid;		
		
		$connect = ldap_connect($this->server, $this->port);
		if (!$connect)
			throw new AuthenticationException("LDAP connect failed.", self::FAILURE);
		$bind = ldap_bind($connect, $this->basedn, $this->basepass);
		if (!$bind)
			throw new AuthenticationException("LDAP bind failed.".$this->basedn.";".$this->basepass, self::FAILURE);

		$search = ldap_search($connect, $this->searchdn, $this->filter.$username);
		
        $info = ldap_get_entries($connect, $search);
		if ($info['count'] == 0)
			return false;
        else
            return true;
    }

}
