<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * PersonFilter defines structure to filter Persons.
 * Has fluent interface.
 *
 * @author Jiri Zajic <jiri.zajic@honeywell.com>
 */
class PersonFilter extends Object
{
   
    /** @var string Employee identification code */
    private $id;

    /** @var string Employee lastname */
    private $surname;

    /** @var string Employee firstname */
    private $name;

    /** @var string Code of cost center assigned to the employee */
    private $costCenterID;

    /** @var int Id of work group/team */
    private $organizationUnitID;

    /** @var boolean Flag showing whether employee is a team leader */
    private $groupLeader;

    /** @var boolean Flag showing whether employee is a department manager */
    private $departmentManager;
	
	/** @var array Person activity status */
	private $status;

    /**
     * Sets person ID.
     * 
     * @param $id string
     */
	public function setID($id)
	{
		$this->id = $id;
		return $this;
	}

    /**
     * Gets person ID.
     * 
     * @return string
     */
	public function getID()
	{
		return $this->id;
	}	
	
    /**
     * Sets firstname.
     * 
     * @param string $name
     */
	public function setFirstname($name)
	{
		$this->name = $name;
		return $this;
	}

    /**
     * Gets firstname.
     * 
     * @return string
     */
	public function getFirstname()
	{
		return $this->name;
	}	

    /**
     * Sets surname.
     * 
     * @param string $name
     */
	public function setSurname($name)
	{
		$this->surname = $name;
		return $this;
	}

    /**
     * Gets surname.
     * 
     * @return string
     */
	public function getSurname()
	{
		return $this->surname;
	}	

    /**
     * Sets cost center ID.
     * 
     * @param string $id
     */
	public function setCostCenterID($id)
	{
		$this->costCenterID = $id;
		return $this;
	}

    /**
     * Gets cost center.
     * 
     * @return string
     */
	public function getCostCenterID()
	{
		return $this->costCenterID;
	}	

    /**
     * Sets group/team ID.
     * 
     * @param int $id 
     */
	public function setOrganizationUnitID($id)
	{
		$this->organizationUnitID = $id;
		return $this;
	}

    /**
     * Gets group/team ID.
     * 
     * @return int
     */
	public function getOrganizationUnitID()
	{
		return $this->organizationUnitID;
	}	
    /**
     * @param boolean $enabled
     */
    public function setGroupLeaderFlag($enabled)
    {
		$this->groupLeader = ($enabled) ? TRUE : FALSE;
		return $this;
    }

    /**
     * @return boolean
     */
    public function getGroupLeaderFlag()
    {
        return $this->groupLeader;
    }	


    /**
     * @param boolean $enabled
     */
    public function setDepartmentManagerFlag($enabled)
    {
		$this->departmentManager = ($enabled) ? TRUE : FALSE;
		return $this;
    }

    /**
     * @return boolean
     */
    public function getDepartmentManagerFlag()
    {
        return $this->departmentManager;
    }
	
    /**
	 * Sets person activity flag.
     * @param array $status
     */
    public function setStatus($status)
    {
        $this->status = $status;
		return $this;
    }

    /**
	 * Gets person activity flag.
     * @return boolean
     */
    public function getStatus()
    {
        return $this->status;
    }
}
