<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * ProjectCharterBudgetHistory.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class ProjectCharterBudgetHistory extends Object
{
    /** @var int ID */
    private $projectCharterID;
    /** @var int Version number */
    private $version;
    /** @var string */
    private $creationDateTime;
    /** @var string EID of responsible person*/
    private $responsible;
    /** @var string */
    private $comment;
    /** @var int Flag of latest version */
    private $latestVersion;
    
    /**
     * Set ID of project charter.
     * @param int $id Must be positive integer.
     * @throws InvalidArgumentException if parameter is not integer greater than 0.
     */
        public function setProjectCharterID($id)
        {
            if ($id > 0)
                $this->projectCharterID = $id;
            else
                throw new InvalidArgumentException('Project charter ID must be integer greater than 0. Wrong param: '.$id);
        }
        
    /**
     * Gets ID of project charter.
     * @return int
     */
        public function getProjectCharterID()
        {
            return $this->projectCharterID;
        }
        
    /**
     * Set version of project charter budget.
     * @param int $version Must be positive integer.
     * @throws InvalidArgumentException if parameter is not integer greater than 0.
     */
        public function setVersion($version)
        {
            if ($version > 0)
                $this->version = $version;
            else
                throw new InvalidArgumentException('Version of project charter budget must be integer greater than 0. Wrong param: '.$version);
        }
        
    /**
     * Gets version of project charter budget.
     * @return int
     */
        public function getVersion()
        {
            return $this->version;
        }
        
    /**
     * Set creation date and time of project charter budget.
     * @param string $creationDateTime Date in YYYY-MM-DD HH:MM:SS format.
         * @todo Check format of $creationDateTime.
     */
        public function setCreationDateTime($creationDateTime)
        {
            $this->creationDateTime = $creationDateTime;
        }
        
    /**
     * Gets creation date and time of project charter budget.
     * @return string
     */
        public function getCreationDateTime()
        {
            return $this->creationDateTime;
        }
        
    /**
     * Set EID of responsible person for project charter budget.
     * @param string $eid
     */
        public function setResponsible($eid)
        {
            $this->responsible = Person::processEID($eid);
        }
        
    /**
     * Gets EID of responsible person for project charter budget.
     * @return string
     */        
        public function getResponsible()
        {
            return $this->responsible;
        }
        
    /**
     * Set comment of current version.
     * @param string $comment
     */
        public function setComment($comment)
        {
            $this->comment = $comment;
        }
        
    /**
     * Gets comment of current version.
     * @return string
     */
        public function getComment()
        {
            return $this->comment;
        }
        
    /**
     * Set latest version.
     * @param int $latestVersion
     */
        public function setLatestVersion($latestVersion)
        {
            $this->latestVersion = $latestVersion;
        }
        
    /**
     * Gets latest version.
     * @return int
     */
        public function getLatestVersion()
        {
            return $this->latestVersion;
        }

}
