<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2016 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Interface for accessing project planning charter item budget.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
interface IProjectPlanningCharterItemBudgetMapper
{
	/**
	 * Saves project planning charter item budget.
	 * 
     * @param ProjectPlanningCharterItemBudget $projectPlanningCharterItemBudget Project planning charter item budget
	 * @return boolean
	 */
	public function save(ProjectPlanningCharterItemBudget $projectPlanningCharterItemBudget);
    
	/**
	 * Updates project planning charter item budget.
	 * 
	 * @param ProjectPlanningCharterItemBudget $projectPlanningCharterItemBudget Project planning charter item
	 * @return boolean
	 */
	public function update(ProjectPlanningCharterItemBudget $projectPlanningCharterItemBudget);
	
	/**
	 * Deletes existing project planning charter item budget.
	 * 
	 * @param int $projectPlanningCharterItemBudget_id
	 * @return boolean
	 */
	public function delete($projectPlanningCharterItemBudget_id);
    
	/**
	 * Returns project planning charter item budget by its id.
	 * 
	 * @param int $id ProjectPlanningCharterItemBudget ID.
	 * @return ProjectPlanningCharterItemBudget
	 */
	public function find($id);
	
	/**
	 * Returns project planning charter item budget by project planning charter item id.
	 * 
	 * @param int $projectPlanningCharterItemID ProjectPlanningCharterItem ID.
	 * @return array of ProjectPlanningCharterItemBudget or empty array if not found
	 */
	public function findByProjectPlanningCharterItemID($projectPlanningCharterItemID);

	/**
	 * Returns all project planning charter item budgets.
	 * 
	 * @return array of ProjectPlanningCharterItemBudget or empty array if not found
	 */
	public function findAll();

}