<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2016 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Interface for accessing project planning charter.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
interface IProjectPlanningCharterMapper
{
	/**
	 * Saves project planning charter.
	 * 
     * @param ProjectPlanningCharter $projectPlanningCharter Project planning charter
	 * @return boolean
	 */
	public function save(ProjectPlanningCharter $projectPlanningCharter);
    
	/**
	 * Updates project planning charter.
	 * 
	 * @param ProjectPlanningCharter $projectPlanningCharter Project planning charter
	 * @return boolean
	 */
	public function update(ProjectPlanningCharter $projectPlanningCharter);
	
	/**
	 * Deletes existing project planning charter.
	 * 
	 * @param int $projectPlanningCharter_id
	 * @return boolean
	 */
	public function delete($projectPlanningCharter_id);
	
	/**
	 * Deletes existing project planning charters by project id and by planning year.
	 * 
	 * @param int $projectID Project ID.
	 * @param int $planningYear Year.
	 * @return boolean
	 */
	public function deleteByProjectIDAndYear($projectID, $planningYear);
    
	/**
	 * Returns project planning charter by its id.
	 * 
	 * @param int $id ProjectPlanningCharter ID.
	 * @return ProjectPlanningCharter
	 */
	public function find($id);
	
	/**
	 * Returns project planning charter by project id.
	 * 
	 * @param int $projectID Project ID.
	 * @return array of ProjectPlanningCharter or empty array if not found
	 */
	public function findByProjectID($projectID);

	/**
	 * Returns project planning charter by project id and by planning year.
	 * 
	 * @param int $projectID Project ID.
	 * @param int $planningYear Year.
	 * @return array of ProjectPlanningCharter or empty array if not found
	 */
	public function findByProjectIDAndYear($projectID, $planningYear);
	
	/**
	 * Returns rows related to Project ID with specified Type
	 * 
	 * @param int $projectID Project ID.
	 * @param int $year The year of planning.
	 * @param string $type Type of the records to find, e.g. 'HRS', 'SUB', 'TRV'.
	 * @return 
	 */
	public function findByProjectIDAndYearAndType($projectID, $year, $type);
	
	/**
	 * Returns rows related to Project ID with specified Type
	 * 
	 * @param int $projectID Project ID.
	 * @param int $year The year of planning.
	 * @param string $type Type of the records to find, e.g. 'HRS', 'SUB', 'TRV'.
	 * @param string $name Name column. Used for labor tables.
	 * @return 
	 */
	public function findByProjectIDAndYearAndTypeAndName($projectID, $year, $type, $name);
	
	/*
	 * Returns years, which are planned for project identified by $projectID
	 * 
	 * @param int $projectID Project ID.
	 * @return array
	 */
	public function findYearsByProjectId($projectID);
	
	/**
	 * Returns all project planning charters.
	 * 
	 * @return array of ProjectPlanningCharter or empty array if not found
	 */
	public function findAll();

}