<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing project planning charter from database using dibi.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
  */
class ProjectPlanningCharterItemBudgetDibiMapper extends DibiMapper implements IProjectPlanningCharterItemBudgetMapper
{
	/**
	 * Creates ProjectPlanningCharterItemBudget instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return	ProjectPlanningCharterItemBudget
	 */
	private function load(DibiRow $row)
	{
		$projectPlanningCharterItemBudget = new ProjectPlanningCharterItemBudget();
		$projectPlanningCharterItemBudget->ID = $row->ProjectPlanningCharterItemBudget_ID;
		$projectPlanningCharterItemBudget->ProjectPlanningCharterItemID = $row->ProjectPlanningCharterItem_ID;
		$projectPlanningCharterItemBudget->Month = $row->Month;
		$projectPlanningCharterItemBudget->Budget = $row->Budget;
		$projectPlanningCharterItemBudget->BudgetType = $row->BudgetType;
		return $projectPlanningCharterItemBudget;
	}

	/**
	 * Saves project planning charter.
	 * 
     * @param ProjectPlanningCharterItemBudget $projectPlanningCharterItemBudget Project planning charter
	 * @return boolean
	 */
	public function save(ProjectPlanningCharterItemBudget $projectPlanningCharterItemBudget)
	{
		try
		{
			$array = array();
			$array['ProjectPlanningCharterItem_ID'] = $projectPlanningCharterItemBudget->ProjectPlanningCharterItemID;
			$array['Month'] = $projectPlanningCharterItemBudget->Month;
			$array['Budget'] = $projectPlanningCharterItemBudget->Budget;
			$array['BudgetType'] = $projectPlanningCharterItemBudget->BudgetType;
			$budget_db_result = $this->conn->insert('ProjectPlanningCharterItemBudget', $array)->execute();
			$projectPlanningCharterItemBudget->ID = $this->conn->insertId();
			return true;
		}
		catch (DibiException $e)
		{
			Debugger::log("Saving ProjectPlanningCharterItemBudget failed. " . $e->getMessage(), Debugger::ERROR);
			return false;
		}
	}

	/**
	 * Updates project planning charter.
	 * 
	 * @param ProjectPlanningCharterItemBudget $projectPlanningCharterItemBudget Project planning charter
	 * @return boolean
	 */
	public function update(ProjectPlanningCharterItemBudget $projectPlanningCharterItemBudget)
	{
		try
		{
			$array = array();
			$array['Month'] = $projectPlanningCharterItemBudget->Month;
			$array['Budget'] = $projectPlanningCharterItemBudget->Budget;
			$array['BudgetType'] = $projectPlanningCharterItemBudget->BudgetType;
			$budget_db_result = $this->conn->update('ProjectPlanningCharterItemBudget', $array)->where('[ProjectPlanningCharterItemBudget_ID] = %i', $projectPlanningCharterItemBudget->getID())->execute();
			return true;
		}
		catch (DibiException $e)
		{
			Debugger::log("ProjectPlanningCharterItemBudget update with ID = " . $projectPlanningCharterItemBudget->getID() . " failed. ".$e->getMessage());
			return false;
		}
	}
	
	/**
	 * Deletes existing project planning charter.
	 * 
	 * @param int $projectPlanningCharterItemBudget_id
	 * @return boolean
	 */
	public function delete($projectPlanningCharterItemBudget_id)
	{
		try
		{
			$result = $this->conn->delete('ProjectPlanningCharterItemBudget')->where('[ProjectPlanningCharterItemBudget_ID] = %i', $projectPlanningCharterItemBudget_id)->execute();
			if ($result)
				Debugger::log("Project planning charter item budget with ID = " . $projectPlanningCharterItemBudget_id . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of project planning charter item budget with ID = " . $projectPlanningCharterItemBudget_id . " failed.", Debugger::ERROR);
			return $result;
		}
		catch (DibiException $e)
		{
			Debugger::log("Deleting of project planning charter item budget with ID = " . $projectPlanningCharterItemBudget_id . " failed: ".$e->getMessage(), Debugger::ERROR);
			return false;
		}
	}

	/**
	 * Returns project planning charter by its id.
	 * 
	 * @param int $id ProjectPlanningCharterItemBudget ID.
	 * @return ProjectPlanningCharterItemBudget
	 */
	public function find($id)
	{
		$projectPlanningCharterItemBudget_db_result = $this->conn->select('*')->from('ProjectPlanningCharterItemBudget')->where('[ProjectPlanningCharterItemBudget_ID] = %i', $id)->fetch();
		if ($projectPlanningCharterItemBudget_db_result !== FALSE)
			return $this->load($projectPlanningCharterItemBudget_db_result);
		else
			return NULL;
	}

	/**
	 * Returns project planning charter by project id.
	 * The indexes in returned array are month numbers e.g. 1-12.
	 * 
	 * @param int $projectPlanningCharterItemID ProjectPlanningCharterItem ID.
	 * @return array of ProjectPlanningCharterItemBudget or empty array if not found
	 */
	public function findByProjectPlanningCharterItemID($projectPlanningCharterItemID)
	{
		$projectPlanningCharterItemBudget_array = array();
		$projectPlanningCharterItemBudget_db_result = $this->conn->select('*')->from('ProjectPlanningCharterItemBudget')->where('[ProjectPlanningCharterItem_ID] = %i', $projectPlanningCharterItemID)->fetchAll();
		foreach ($projectPlanningCharterItemBudget_db_result as $projectPlanningCharterItemBudget)
		{
			$projectPlanningCharterItemBudget_array[$projectPlanningCharterItemBudget->Month] = $this->load($projectPlanningCharterItemBudget);
		}
		return $projectPlanningCharterItemBudget_array;
	}
	
	/**
	 * Returns all project planning charters.
	 * The indexes in returned array are month numbers e.g. 1-12.
	 * 
	 * @return array of ProjectPlanningCharterItemBudget or empty array if not found
	 */
	public function findAll()
	{
		$projectPlanningCharterItemBudget_array = array();
		$projectPlanningCharterItemBudget_db_result = $this->conn->select('*')->from('ProjectPlanningCharterItemBudget')->fetchAll();
		foreach ($projectPlanningCharterItemBudget_db_result as $projectPlanningCharterItemBudget)
		{
			$projectPlanningCharterItemBudget_array[$projectPlanningCharterItemBudget->Month] = $this->load($projectPlanningCharterItemBudget);
		}
		return $projectPlanningCharterItemBudget_array;
	}
}
