<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * ProjectCharterSummaryFileHistory repository access class.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class ProjectCharterSummaryFileHistoryRepository extends Object
{
    /** @var IProjectCharterSummaryFileHistoryMapper */
 	private $mapper;
	
	/** @var MultiLogger */
	private $multilog;

    public function __construct(IProjectCharterSummaryFileHistoryMapper $mapper)
    {
        $this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'ProjectCharterSummaryFileHistory-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "Project Charter Summary File History"));
    }
    
	/**
	 * Creates new project charter file history record.
	 * 
	 * @param ProjectCharterSummaryFileHistory $projectCharterSummaryFileHistory
	 * @return boolean
	 */
	public function save(ProjectCharterSummaryFileHistory $projectCharterSummaryFileHistory)
	{
		if($this->mapper->save($projectCharterSummaryFileHistory))
		{
			$this->multilog->log('Project Charter Summary File History for charter with ID: ' . $projectCharterSummaryFileHistory->ProjectCharterID . ' and version: ' . $projectCharterSummaryFileHistory->Version . ' and responsible EID: ' . $projectCharterSummaryFileHistory->Responsible . ' saved.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
        
	/**
	 * Updates existing project charter file history record.
	 * 
	 * Updates only attributes that are updatable.
	 * 
	 * @param ProjectCharterSummaryFileHistory $projectCharterSummaryFileHistory
	 * @return boolean
	 */
	public function update(ProjectCharterSummaryFileHistory $projectCharterSummaryFileHistory)
	{
		$oldObject = $this->find($projectCharterSummaryFileHistory->ProjectCharterID, $projectCharterSummaryFileHistory->Version);
		if($this->mapper->update($projectCharterSummaryFileHistory))
		{
			$this->multilog->log('Project Charter Summary File History for charter with ID: ' . $projectCharterSummaryFileHistory->ProjectCharterID . ' and version: ' . $projectCharterSummaryFileHistory->Version . ' and responsible EID: ' . $projectCharterSummaryFileHistory->Responsible . ' updated. ' . $this->multilog->differences($oldObject, $projectCharterSummaryFileHistory), ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Deletes existing project charter file history record.
	 * 
	 * @param int $projectCharterID
	 * @param int $version
	 * @return boolean
	 */
	public function delete($projectCharterID, $version)
	{
		if($this->mapper->delete($projectCharterID, $version))
		{
			$this->multilog->log('Project Charter Summary File History for charter with ID: ' . $projectCharterID . ' and version: ' . $version . ' deleted.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Gets project charter file history record by ids.
	 * 
	 * @param int $projectCharterID
	 * @param int $version
	 * @return ProjectCharterSummaryFileHistory
	 */
	public function find($projectCharterID, $version)
	{
		return $this->mapper->find($projectCharterID, $version);
	}
	
	/**
	 * Gets all project charter history records by project charter id.
	 * 
	 * @param int $projectCharterID
	 * @return array Array of ProjectCharterSummaryFileHistory or empty array if error occured.
	 */
	public function findByProjectCharterID($projectCharterID)
	{
		return $this->mapper->findByProjectCharterID($projectCharterID);
	}
        
	/**
	 * Gets project charter file history record with latest version.
	 * 
	 * @param int $projectCharterID
	 * @return ProjectCharterSummaryFileHistory
	 */
	public function findLatestVersion($projectCharterID)
	{
		return $this->mapper->findLatestVersion($projectCharterID);
	}
        
	/**
	 * Initializes flag LatestVersion
	 * 
	 * @return boolean
	 */
	public function initLatestVersion($projectCharterID)
	{
		return $this->mapper->initLatestVersion($projectCharterID);
	}
}
