<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2014 Honeywell International spol. s r.o.
 *
 * For the full copyright and licence information, please view
 * the file licence.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing Project - ECLicenceName relation from database using dibi.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class ProjectECLicenceNameDibiMapper extends DibiMapper implements IProjectECLicenceNameMapper
{
	/**
	 * Creates ProjectECLicenceName instance from DibiRow.
	 * @param DibiRow $row
	 * @return ProjectECLicenceName
	 */
    private function load(DibiRow $row)
    {
		$relation = new ProjectECLicenceName();
		$relation->ProjectID = $row->Project_ID;
		$relation->ECLicenceNameID = $row->ECLicenceName_ID;
		return $relation;
    }
	
	/**
	 * Saves Project - ECLicenceName relation
	 * @param ProjectECLicenceName $relation
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function save(ProjectECLicenceName $relation)
	{
		try
        {
            $array = array();
            $array['Project_ID'] = $relation->ProjectID;
            $array['ECLicenceName_ID'] = $relation->ECLicenceNameID;
            $db_result = $this->conn->insert('Project_ECLicenceName', $array)->execute();
            return TRUE;
        }
        catch (DibiException $e)
        {
            Debugger::log("Saving ProjectECLicenceName relation: " . $relation->ProjectID . "-" . $relation->ECLicenceNameID .  " failed. " . $e->getMessage(), Debugger::ERROR);
            return FALSE;
        }
	}
	
	/**
	 * Updates Project - ECLicenceName relation
	 * @param ProjectECLicenceName $relation
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function update(ProjectECLicenceName $relation)
	{
		try
        {
            $array = array();
            $array['Project_ID'] = $relation->ProjectID;
            $array['ECLicenceName_ID'] = $relation->ECLicenceNameID;
            $db_result = $this->conn->update('Project_ECLicenceName', $array)->where('[Project_ID] = %i AND [ECLicenceName_ID] = %i', $relation->ProjectID, $relation->ECLicenceNameID)->execute();
            return TRUE;
        }
        catch (DibiException $e)
        {
            Debugger::log("ProjectECLicenceName relation: " . $relation->ProjectID . "-" . $relation->ECLicenceNameID . " update failed. " . $e->getMessage(), Debugger::ERROR);
            return FALSE;
        }
	}
	
	/**
	 * Deletes Project - ECLicenceName relation
	 * @param int $project_id Project id.
	 * @param int $licence_name_id ECLicenceName id.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function delete($project_id, $licence_name_id)
	{
		try
		{
		    $db_result = $this->conn->delete('[Project_ECLicenceName]')->where('[Project_ID] = %i AND [ECLicenceName_ID]', $project_id, $licence_name_id)->execute();
			if ($db_result)
				Debugger::log("ProjectECLicenceName relation: " . $project_id ."-" . $licence_name_id . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of ProjectECLicenceName relation: " . $project_id ."-" . $licence_name_id . " failed.", Debugger::ERROR);
			return ($db_result) ? TRUE : FALSE;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting of ProjectECLicenceName relation: " . $project_id ."-" . $licence_name_id . " failed: " . $e->getMessage(), Debugger::ERROR);
            return FALSE;
        }
	}
	
	/**
	 * Deletes Project - ECLicenceName relations for given project. 
	 * @param int $project_id Should be project id.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function deleteByProjectID($project_id)
	{
		try
		{
		    $db_result = $this->conn->delete('Project_ECLicenceName')->where('[Project_ID] = %i', $project_id)->execute();
			if ($db_result)
				Debugger::log("ProjectECLicenceName relations for project ID = " . $project_id . " were deleted.", Debugger::INFO);
			elseif ($db_result !== 0)
				Debugger::log("Deleting of ProjectECLicenceName relations for project ID = " . $project_id . " failed.", Debugger::ERROR);
			return ($db_result || ($db_result === 0)) ? TRUE : FALSE;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting of ProjectECLicenceName relations for project ID = " . $project_id . " failed: " . $e->getMessage(), Debugger::ERROR);
            return FALSE;
        }
	}
	
	/**
	 * Finds Project - ECLicenceName relation by project id and ECLicenceName id.
	 * @param int $project_id Project id.
	 * @param int $licence_name_id ECLicenceName id.
	 * @return ProjectECLicenceName or NULL if not found.
	 */
	public function find($project_id, $licence_name_id)
	{
		$db_result = $this->conn->select('*')->from('[Project_ECLicenceName]')->where('[Project_ID] = %i AND [ECLicenceName_ID] = %i', $project_id, $licence_name_id)->fetch();
		if ($db_result !== FALSE)
			return $this->load($db_result);
		else
			return NULL;
	}
	
	/**
	 * Finds all Project - ECLicenceName relations
	 * @return array Array of ProjectECLicenceName or empty array.
	 */
	public function findAll()
	{
		$db_result = $this->conn->select('*')->from('[Project_ECLicenceName]')->fetchAll();     
		$relation_array = array();
        if ($db_result !== FALSE)
        {
            foreach ($db_result as $row)
                $relation_array[] = $this->load($row);
        }		
		return $relation_array;
	}
	
	/**
	 * Finds all Project - ECLicenceName relations by project id
	 * @param int $project_id Project id.
	 * @return array Array of ProjectECLicenceName or empty array.
	 */
	public function findByProjectID($project_id)
	{
		$db_result = $this->conn->select('*')->from('[Project_ECLicenceName]')->where('[Project_ID] = %i', $project_id)->fetchAll();     
		$relation_array = array();
        if ($db_result !== FALSE)
        {
            foreach ($db_result as $row)
                $relation_array[] = $this->load($row);
        }		
		return $relation_array;
	}
	
	/**
	 * Finds all Project - ECLicenceName relations by ECLicenceName id
	 * @param int $licence_name_id ECLicenceName id.
	 * @return array Array of ProjectECLicenceName or empty array.
	 */
	public function findByECLicenceNameID($licence_name_id)
	{
		$db_result = $this->conn->select('*')->from('[Project_ECLicenceName]')->where('[ECLicenceName_ID] = %i', $licence_name_id)->fetchAll();     
		$relation_array = array();
        if ($db_result !== FALSE)
        {
            foreach ($db_result as $row)
                $relation_array[] = $this->load($row);
        }		
		return $relation_array;
	}
}
