<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing project states from database using dibi.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class ProjectStateDibiMapper extends DibiMapper implements IProjectStateMapper
{

	/**
	 * Creates ProjectState instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return	ProjectState 
	 */
    private function load(DibiRow $row)
    {
		return new ProjectState($row->Number, $row->Name);
	}	

	/**
	 * Gets project state by number.
	 * 
	 * @param int $number Project state number.
	 * @return ProjectState or NULL if not found
	 */
	public function find($number)
	{
		$state_db_result = $this->conn->select('*')->from('State')->where('[Number] = %u', $number)->fetch();
		if ($state_db_result !== FALSE)
			return $this->load($state_db_result);
		else
			return NULL;
	}

	/**
	 * Gets all Project states.
	 * 
	 * @return array Array of Projects states where state number is key.
	 */
	public function findAll()
	{
        $states_array = array();
		$states_db_result = $this->conn->select('*')->from('State')->fetchAll();
		foreach ($states_db_result as $state_row)
			$states_array[$state_row->Number] = $this->load($state_row);
        return $states_array;
	}
}

