<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2016 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models\Import
 */

/**
 * Service for project eac report.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
abstract class ProjectEACReportService extends object
{
	/* possible report row types */
	const PROJECT_REPORT_PLAN = 'Plan';
	const PROJECT_REPORT_ASSIGNED = 'Assigned';
	const PROJECT_REPORT_ACTUALS = 'Actuals';
	const PROJECT_REPORT_ACTUALS_EAC = 'EAC (Actuals&Assigned)';
	const PROJECT_REPORT_DELTA_PLAN_ACTUALS_EAC = 'Delta (Plan - EAC)';
	const PROJECT_REPORT_DELTA_ASSIGNED_ACTUALS_EAC = 'Delta (Assigned - Actuals)';
	
	/** @var DIContainer DI context with application repositories access */
	protected $context;

	/** @var ILogger Logging facility */
	protected $logger;
	
	/** @var array Year to generation */
	protected $year;

	/** @var String Value for freeze user interface during report generation. */
	protected $uiFreezeToken;
	
	/** @var string Report Name */
	protected $reportType;
	
	/**
	 * 
	 * @param DIContainer $context
	 * @param ILogger $logger
	 * @param array $year
	 * @param int $uiFreezeToken
	 * 
	 */
	public function __construct(DIContainer $context, ILogger $logger, $year, $reportType, $uiFreezeToken = null)
	{
		$this->context = $context;
		$this->logger = $logger;
		$this->year = $year;
		$this->reportType = $reportType;
		$this->uiFreezeToken = $uiFreezeToken;
	}
	
	/**
	 * 
	 * Generates project charter row data report as a XLSX (MS-EXCEL file).
	 * File can be downloaded through browser and also file is saved to the public/ProjectCharterRowDataReport folder.
	 * 
	 */
	public function runProjectEACReportGeneration($tasks, $oneYearReport, $objectName, $objectSapID)
	{
//        $memory_limit = "512M";
//        if (!ini_set("memory_limit", $memory_limit)) 
//		{
//			$this->flashMessage('Setup of memory limit failed. Contact administrator.', 'error');
//			trigger_error('Unable to setup memory limit to ' . $memory_limit . '.', E_USER_ERROR);
//		}
//
        $exec_time = 600;
	    if (!ini_set("max_execution_time", $exec_time)) 
		{
			$this->flashMessage('Setup of maximum execution time failed. Contact administrator.', 'error');
			trigger_error('Unable to setup maximum execution time to '.$exec_time.'.', E_USER_ERROR);
		}
//
//		$lock = $this->context->createLock('ProjectReport', $exec_time);
//		if (!$lock->lock())
//		{
//			$this->logger->log('Project Report generation failed - active lock detected: ' . date('m-d-Y H:i:s'), ILogger::INFO);
//			throw new ProjectEACReportServiceException("Generation of Project Report is already running.");
//		}

		$monthsShort = array(
			'Jan',
			'Feb',
			'Mar',
			'Apr',
			'May',
			'Jun',
			'Jul',
			'Aug',
			'Sep',
			'Oct',
			'Nov',
			'Dec'
		);

		$unit = 'hrs';
		$currency = 'USD';
		
		// FORMAT SETTINGS
		$columnAWidth = 40;
		$columnBWidth = 30;
		$columnTotalYear = 12;
		$columnTotalOverall = 21;
		$columntCostShare = 11;
		$columntSAPID = 18;
        $borderTask = array(
        	'borders' => array(
                'top' => array(
                    'style' => PHPExcel_Style_Border::BORDER_MEDIUM,
                    'color' => array('rgb' => '000000')
                ),
            ),
        );
        $borderPerson = array(
        	'borders' => array(
                'top' => array(
                    'style' => PHPExcel_Style_Border::BORDER_THIN,
                    'color' => array('rgb' => '000000')
                ),
            ),
        );
        $borderColumnR = array(
        	'borders' => array(
                'right' => array(
                    'style' => PHPExcel_Style_Border::BORDER_MEDIUM,
                    'color' => array('rgb' => '000000')
                ),
            ),
        );
        $borderColumnL = array(
        	'borders' => array(
                'left' => array(
                    'style' => PHPExcel_Style_Border::BORDER_MEDIUM,
                    'color' => array('rgb' => '000000')
                ),
            ),
        );

		$worker_rows = $task_effort_rows = $task_cost_rows = $effort_rows = $cost_rows = $cost_share_column = $labour_tasks = $costQ_rows = $honInvestment_rows = $honInvestmentQ_rows = array();
		$actualMonth = FiscalCalendar::getFiscalMonthByTimestamp(time())->index + 1;

		// ---REPORT GENERATION STARTS---
		// Generate general info
		$objPHPExcel = new PHPExcel();
		$sheet = $objPHPExcel->getActiveSheet();
		$sheet->setTitle($this->reportType . " Report");
		$sheet->freezePane('C6');
		$rowIterator = $sheet->getRowIterator();
		$cellIterator = $rowIterator->current()->getCellIterator();
		$cellIterator->current()->setValue($this->reportType . ' Name:');
		$cellIterator->next();
		$cellIterator->current()->setValue($objectName);
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		$cellIterator->current()->setValue('SAP ' . $this->reportType . ' ID:');
		$cellIterator->next();
		$cellIterator->current()->setValue($objectSapID);
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		$cellIterator->current()->setValue('SAP Schedule export date:');
		$cellIterator->next();
		$cellIterator->current()->setValue(date('m-d-Y H:i:s'));
		$rowIterator->next();

		// ---PROJECT PLAN SECTION STARTS---
		// Generate Year header
		$cellIterator = $rowIterator->current()->getCellIterator();
		$cellIterator->next();
		$cellIterator->next();
		$row = $cellIterator->current()->getRow();
		foreach ($this->year as $year)
		{
			$col = $cellIterator->current()->getColumn();
			$cellIterator->current()->setValue($year);
			$sheet->mergeCells($col . $row . ':' . PHPExcel_Cell::stringFromColumnIndex(PHPExcel_Cell::columnIndexFromString($col) + 11) . $row);
			for ($i = 1; $i < 14; $i++)
			{
				if($year = Date('Y'))
				{
					if($i == fiscalCalendar::currentFiscalMonth())
					{
						$fiscalColumn = $cellIterator->current()->getColumn();
					}
				}
				$cellIterator->next();
			}
		}
		$rowIterator->next();

		// Generate columns header
		$cellIterator = $rowIterator->current()->getCellIterator();
		$cellIterator->current()->setValue('--- ' . $this->reportType . ' PLAN ---');
		$sheet->getColumnDimension($cellIterator->current()->getColumn())->setWidth($columnAWidth);
		$cellIterator->next();
		$sheet->getColumnDimension($cellIterator->current()->getColumn())->setWidth($columnBWidth);
		$cellIterator->next();
		foreach ($this->year as $year)
		{
			foreach ($monthsShort as $month)
			{
				$cellIterator->current()->setValue($month);
				$col = $cellIterator->current()->getColumn();
				$sheet->getColumnDimension($col)->setOutlineLevel(1);
				if(!$oneYearReport)
				{
					$sheet->getColumnDimension($col)->setVisible(false);
				}
				$cellIterator->next();
			}
			$cellIterator->current()->setValue('TOTAL ' . $year);
			$sheet->getColumnDimension($cellIterator->current()->getColumn())->setWidth($columnTotalYear);
			$cellIterator->next();
		}
		if(!$oneYearReport)
		{
			$cellIterator->current()->setValue('TOTAL Overall Project');
			$sheet->getColumnDimension($cellIterator->current()->getColumn())->setWidth($columnTotalOverall);
			$cellIterator->next();
		}
		$cellIterator->current()->setValue('Unit');
		$cellIterator->next();
		$cellIterator->current()->setValue('CostShare');
		$sheet->getColumnDimension($cellIterator->current()->getColumn())->setWidth($columntCostShare);
		$cellIterator->next();
		$cellIterator->current()->setValue('SAP ID');
		$sheet->getColumnDimension($cellIterator->current()->getColumn())->setWidth($columntSAPID);
		$sheet->getStyle('A4:' . $cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
		$sheet->getStyle('A1:' . $cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setBold(true);
		$cellIterator->next();
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();

		// Generate rows
		$changeTask = 0;
		foreach ($tasks as $task)
		{
			$taskPlans = $this->context->taskPlanRepository->findTaskPlans($task->ID);
			if ($task->isLabourTask())
			{
				//Get reported persons for labour tasks
				if(!$oneYearReport)
				{
					$assignedWorkers = $this->context->personRepository->findAllPersonsAssignedToTask($task->ID);
					$chargedWorkersEID = $this->context->personRepository->findAllPersonsChargedOnTask($task->SAPID);
				}
				else
				{
					$assignedWorkers = $this->context->personRepository->findAllPersonsAssignedToTaskInYear($task->ID, $year);
					$chargedWorkersEID = $this->context->personRepository->findAllPersonsChargedOnTaskInYear($task->SAPID, $year);
				}
				
				$chargedWorkers = array();
				foreach($chargedWorkersEID as $eid)
				{
					$person = $this->context->personRepository->find($eid);
					if($person)
					{
						$chargedWorkers[$eid] = $person;
					}
					else
					{
						$newPerson = new Person();
						$newPerson->ID = $eid;
						$newPerson->Firstname = 'WORKER';
						$newPerson->Surname = 'UNKNOWN';
						$chargedWorkers[$eid] = $newPerson;
					}
				}
				$persons = array();
				$persons = array_merge($assignedWorkers, $chargedWorkers);
			}
			else
			{
				$costAssignments = $this->context->assignmentsCostRepository->findAllForTask($task->ID);
				$costActuals = $this->context->actualsCostRepository->findForTask($task->ID, $currency);
			}			
			foreach (ProjectEACReportService::getProjectRowTypes() as $exportRow)
			{
				$row = $cellIterator->current()->getRow();
				$firstColorCol = $cellIterator->current()->getColumn();
				$cellIterator->current()->setValue($task->Name);
				$cellIterator->next();
				$cellIterator->current()->setValue($exportRow);
				$col = $cellIterator->current()->getColumn();
				$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
				$cellIterator->next();
				foreach ($this->year as $year)
				{
					for ($i = 1; $i < 13; $i++)
					{
						$col = $cellIterator->current()->getColumn();
						if($year == date('Y'))
						{
							if($fiscalColumn == $col)
								{
									$sheet->getStyle($fiscalColumn . $row)->applyFromArray($borderColumnR);
									$sheet->getStyle($fiscalColumn . $row)->applyFromArray($borderColumnL);
								} 
						}
						if ($task->isLabourTask())
						{
							if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
							{
								$task_effort_rows[$task->CostCenterID][$task->ID] = $row;
								$labour_tasks[$task->CostCenterID][$task->ID] = $row;
							}
							switch ($exportRow)
							{
								case ProjectEACReportService::PROJECT_REPORT_PLAN:
									if (isset($taskPlans[$year]))
											$cellIterator->current()->setValue($taskPlans[$year]->getMonthPlan($i)->Hours);
									else $cellIterator->current()->setValue('0');
									break;
								case ProjectEACReportService::PROJECT_REPORT_ASSIGNED:
									$cellIterator->current()->setValue($this->getPersonAssignedActualsAndEACFormula($col, $row, $persons));
									break;
								case ProjectEACReportService::PROJECT_REPORT_ACTUALS:
									$cellIterator->current()->setValue($this->getPersonAssignedActualsAndEACFormula($col, $row, $persons));
									break;
								case ProjectEACReportService::PROJECT_REPORT_ACTUALS_EAC:
									$cellIterator->current()->setValue($this->getPersonAssignedActualsAndEACFormula($col, $row, $persons));
									break;
								case ProjectEACReportService::PROJECT_REPORT_DELTA_PLAN_ACTUALS_EAC:
									$cellIterator->current()->setValue($this->getTaskPlanDeltaFormula($col, $row));
									break;
								case ProjectEACReportService::PROJECT_REPORT_DELTA_ASSIGNED_ACTUALS_EAC:
									$cellIterator->current()->setValue($this->getTaskAssignedDeltaFormula($col, $row));
									break;
							}
						}
						else
						{
							if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
							{
								$task_cost_rows[substr($task->Name, 0, 3)][$task->ID] = $row;
							}
							switch ($exportRow)
							{
								case ProjectEACReportService::PROJECT_REPORT_PLAN:
									if (isset($taskPlans[$year]))
											$cellIterator->current()->setValue($taskPlans[$year]->getMonthPlan($i)->Budget);
									else 
										$cellIterator->current()->setValue('0');
									$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
									break;
								case ProjectEACReportService::PROJECT_REPORT_ASSIGNED:
									if(isset($taskPlans[$year]))
									{
										$cellIterator->current()->setValue($this->getAssignedCost($costAssignments, $taskPlans[$year], $year, $i));
									}
									else
									{
										$cellIterator->current()->setValue($this->getAssignedCost($costAssignments, null, $year, $i));
									}
									$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
									break;
								case ProjectEACReportService::PROJECT_REPORT_ACTUALS:
									$cellIterator->current()->setValue($this->getActualsCost($costActuals, $year, $i));
									$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
									break;
								case ProjectEACReportService::PROJECT_REPORT_ACTUALS_EAC:
									if ($year == Date('Y'))
									{
										if ($i >= $actualMonth)
										{
											if(isset($taskPlans[$year]))
											{
												$cellIterator->current()->setValue($this->getAssignedCost($costAssignments, $taskPlans[$year], $year, $i));
											}
											else
											{
												$cellIterator->current()->setValue($this->getAssignedCost($costAssignments, null, $year, $i));
											}
										}
										else
										{
											$cellIterator->current()->setValue($this->getActualsCost($costActuals, $year, $i));
										}
									}
									elseif ($year > Date('Y'))
									{
										if(isset($taskPlans[$year]))
										{
											$cellIterator->current()->setValue($this->getAssignedCost($costAssignments, $taskPlans[$year], $year, $i));
										}
										else
										{
											$cellIterator->current()->setValue($this->getAssignedCost($costAssignments, null, $year, $i));
										}
									}
									else
									{
										$cellIterator->current()->setValue($this->getActualsCost($costActuals, $year, $i));
									}
									$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
									break;
								case ProjectEACReportService::PROJECT_REPORT_DELTA_PLAN_ACTUALS_EAC:
									$cellIterator->current()->setValue($this->getTaskPlanDeltaFormula($col, $row));
									$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
									break;
								case ProjectEACReportService::PROJECT_REPORT_DELTA_ASSIGNED_ACTUALS_EAC:
									$cellIterator->current()->setValue($this->getTaskAssignedDeltaFormula($col, $row));
									$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
									break;
							}
						}
						$cellIterator->next();
					}
					$col = $cellIterator->current()->getColumn();
					$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
					$sheet->getStyle($col . $row)->applyFromArray($borderColumnL);
					$cellIterator->current()->setValue($this->getYearSumFormula($col, $row));
					if ($task->isLabourTask())
						;
					else
						$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
					$cellIterator->next();
				}
				if(!$oneYearReport)
				{
					$col = $cellIterator->current()->getColumn();
					$cellIterator->current()->setValue($this->getTaskSumFormula($col, $row, count($this->year)));
					if ($task->isLabourTask())
						;
					else
						$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
					$cellIterator->next();
				}
				if ($task->isLabourTask()) 
					$cellIterator->current()->setValue($unit);
				else 
					$cellIterator->current()->setValue($currency);
				$cellIterator->next();
				if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
					$cellIterator->current()->setValue($task->CostShare * 100);
				$col = $cellIterator->current()->getColumn();
				$cost_share_column[$task->ID] = $col;
				$cellIterator->next();
				if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
					$cellIterator->current()->setValue($task->SAPID);
				$lastColorCol = $cellIterator->current()->getColumn();
				$this->setColor($row, $firstColorCol, $lastColorCol, $exportRow, $changeTask, $sheet);
				if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
				{
					$sheet->getStyle($firstColorCol . $row . ':' . $lastColorCol . $row)->applyFromArray($borderTask);
				}
				$cellIterator->next();
				$rowIterator->next();
				$cellIterator = $rowIterator->current()->getCellIterator();
			}
			// Generates workers for labour task
			if ($task->isLabourTask())
			{
				foreach ($persons as $person)
				{
					foreach (ProjectEACReportService::getProjectRowTypesForPerson() as $exportRow)
					{
						$row = $cellIterator->current()->getRow();
						$col = $firstColorCol = $cellIterator->current()->getColumn();
						$sheet->getRowDimension($row)->setOutlineLevel(1);
						$sheet->getRowDimension($row)->setVisible(false);
						if(ProjectEACReportService::PROJECT_REPORT_ASSIGNED == $exportRow)
							$worker_rows[$person->getReversedShortName() . ' (' . $person->ID . ')'][] = $row;
						$cellIterator->current()->setValue($person->getReversedShortName() . ' (' . $person->ID . ')');
						$firstBorderCol = $cellIterator->current()->getColumn();
						$cellIterator->next();
						$col = $cellIterator->current()->getColumn();
						$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
						$cellIterator->current()->setValue($exportRow);
						$cellIterator->next();
						foreach ($this->year as $year)
						{
							$assignment = $this->context->personTaskAssignmentRepository->findPersonTaskYearAssignment($task->ID, $year, $person->ID);
							$actuals = $this->context->workRepository->findPersonTaskYearWork($person->ID, $task->ID, $year);
							for ($i = 1; $i < 13; $i++)
							{
								$col = $cellIterator->current()->getColumn();
								if($year == date('Y'))
								{
									if($fiscalColumn == $col)
										{
											$sheet->getStyle($fiscalColumn . $row)->applyFromArray($borderColumnR);
											$sheet->getStyle($fiscalColumn . $row)->applyFromArray($borderColumnL);
										} 
								}
								switch ($exportRow)
								{
									case ProjectEACReportService::PROJECT_REPORT_ASSIGNED:
										$cellIterator->current()->setValue($assignment[$i]->Hours);
										break;
									case ProjectEACReportService::PROJECT_REPORT_ACTUALS:
										$cellIterator->current()->setValue($actuals[$i]);
										break;
									case ProjectEACReportService::PROJECT_REPORT_ACTUALS_EAC:
										if ($year == Date('Y'))
										{
											if ($i >= $actualMonth)
													$cellIterator->current()->setValue($assignment[$i]->Hours);
											else $cellIterator->current()->setValue($actuals[$i]);
										}
										elseif ($year > Date('Y'))
										{
											$cellIterator->current()->setValue($assignment[$i]->Hours);
										}
										else
										{
											$cellIterator->current()->setValue($actuals[$i]);
										}
										break;
									case ProjectEACReportService::PROJECT_REPORT_DELTA_ASSIGNED_ACTUALS_EAC:
										$cellIterator->current()->setValue($this->getPersonAssignedDeltaFormula($col, $row));
										break;
								}
								$cellIterator->next();
							}
							$col = $cellIterator->current()->getColumn();
							$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
							$sheet->getStyle($col . $row)->applyFromArray($borderColumnL);
							$cellIterator->current()->setValue($this->getYearSumFormula($col, $row));
							$cellIterator->next();
						}
						if(!$oneYearReport)
						{
							$col = $cellIterator->current()->getColumn();
							$cellIterator->current()->setValue($this->getTaskSumFormula($col, $row, count($this->year)));
							$cellIterator->next();
						}
						$cellIterator->current()->setValue($unit);
						$cellIterator->next();
						$cellIterator->next();
						$lastBorderCol = $cellIterator->current()->getColumn();
						if(ProjectEACReportService::PROJECT_REPORT_ASSIGNED == $exportRow)
						{
							$sheet->getStyle($firstBorderCol . $row . ':' . $lastBorderCol . $row)->applyFromArray($borderPerson);
						}
						$rowIterator->next();
						$cellIterator = $rowIterator->current()->getCellIterator();
					}
				}
			}
			$changeTask++;
		}
		$rowIteratorProjectReportSheet = $rowIterator;
		// ---PROJECT PLAN SECTION ENDS---
		// ---COST CENTERS / LABOUR RATES SECTION STARTS---
		$sheet2 = $objPHPExcel->createSheet(1);
		$sheet2->setTitle("Labour rates");
		$sheet2->freezePane('C3');
		$rowIterator = $sheet2->getRowIterator();
		$cellIterator = $rowIterator->current()->getCellIterator();
		// Generate Year header
		$cellIterator->next();
		$cellIterator->next();
		$row = $cellIterator->current()->getRow();
		foreach ($this->year as $year)
		{
			$col2 = $cellIterator->current()->getColumn();
			$cellIterator->current()->setValue($year);
			$sheet2->mergeCells($col2 . $row . ':' . PHPExcel_Cell::stringFromColumnIndex(PHPExcel_Cell::columnIndexFromString($col2) + 10) . $row);
			for ($i = 1; $i < 14; $i++)
			{
				$cellIterator->next();
			}
		}
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		//Generate columns header
		$cellIterator->current()->setValue('--- LABOUR RATES ---');
		$sheet2->getColumnDimension($cellIterator->current()->getColumn())->setWidth($columnAWidth);
		$cellIterator->next();
		$cellIterator->current()->setValue('Work Center ID');
		$sheet2->getColumnDimension($cellIterator->current()->getColumn())->setWidth($columnBWidth);
		$cellIterator->next();
		foreach($this->year as $year)
		{
			foreach ($monthsShort as $month)
			{
				$cellIterator->current()->setValue($month);
				$cellIterator->next();
			}
			$sheet2->getColumnDimension($cellIterator->current()->getColumn())->setWidth(1);
			$cellIterator->next();
		}
		$cellIterator->current()->setValue('Unit');
		$sheet2->getStyle('A1:' . $cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
		$sheet2->getStyle('A1:' . $cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setBold(true);
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();

		$labour_rates_rows = array();
		$labour_centers = $this->context->costCenterRepository->findAllCostCenters();
		foreach ($labour_centers as $center)
		{
			$row = $cellIterator->current()->getRow();
			$labour_rates_rows[$center->ID] = $row;
			$cellIterator->current()->setValue($center->Name);
			$cellIterator->next();
			$cellIterator->current()->setValue($center->ID);
			$cellIterator->next();
			foreach($this->year as $year)
			{
				$rate = $center->getYearLabourRate($year);
				if ($rate != NULL)
				{
					for ($i = 1; $i < 13; $i++)
					{
						$cellIterator->current()->setValue($rate->getMonthlyHourRate($i));
						$cellIterator->next();
					}
					$cellIterator->next();
					$cellIterator->current()->setValue($currency . '/' . $unit);
				}
			}
			$rowIterator->next();
			$cellIterator = $rowIterator->current()->getCellIterator();
		}
		// ---COST CENTERS / LABOUR RATES SECTION ENDS---
		// ---EFFORT SECTION STARTS---
		// Generate Year header
		$rowIterator = $rowIteratorProjectReportSheet;
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		$colBold = $cellIterator->current()->getColumn();
		$rowBold = $cellIterator->current()->getRow();
		$cellIterator->next();
		$cellIterator->next();
		$row = $cellIterator->current()->getRow();
		foreach ($this->year as $year)
		{
			$col = $cellIterator->current()->getColumn();
			$cellIterator->current()->setValue($year);
			$sheet->mergeCells($col . $row . ':' . PHPExcel_Cell::stringFromColumnIndex(PHPExcel_Cell::columnIndexFromString($col) + 11) . $row);
			for ($i = 1; $i < 14; $i++)
			{
				$cellIterator->next();
			}
		}
		$rowIterator->next();
		// Generate columns header
		$cellIterator = $rowIterator->current()->getCellIterator();
		$cellIterator->current()->setValue('--- EFFORT ---');
		$cellIterator->next();
		$cellIterator->next();
		foreach ($this->year as $year)
		{
			foreach ($monthsShort as $month)
			{
				$cellIterator->current()->setValue($month);
				$cellIterator->next();
			}
			$cellIterator->current()->setValue('TOTAL ' . $year);
			$cellIterator->next();
		}
		if(!$oneYearReport)
		{
			$cellIterator->current()->setValue('TOTAL Overall Project');
			$cellIterator->next();
		}
		$cellIterator->current()->setValue('Unit');
		$sheet->getStyle($colBold . $rowBold . ':' . $cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
		$sheet->getStyle($colBold . $rowBold . ':' . $cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setBold(true);
		$cellIterator->next();
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		// Generate rows
		foreach ($labour_centers as $center)
		{
			if (isset($task_effort_rows[$center->ID]))
			{
				foreach (ProjectEACReportService::getProjectRowTypes() as $exportRow)
				{
					$row = $cellIterator->current()->getRow();
					$firstColorCol = $cellIterator->current()->getColumn();
					$sheet->getRowDimension($row)->setOutlineLevel(1);
					$sheet->getRowDimension($row)->setVisible(false);
					if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
					{
						$effort_rows[] = $row;
						$cellIterator->current()->setValue($center->Name);
					}
					$cellIterator->next();
					$col = $cellIterator->current()->getColumn();
					$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
					$cellIterator->current()->setValue($exportRow);
					$cellIterator->next();
					foreach ($this->year as $year)
					{
						for ($i = 1; $i < 13; $i++)
						{
							$col = $cellIterator->current()->getColumn();
							if($year == date('Y'))
							{
								if($fiscalColumn == $col)
									{
										$sheet->getStyle($fiscalColumn . $row)->applyFromArray($borderColumnR);
										$sheet->getStyle($fiscalColumn . $row)->applyFromArray($borderColumnL);
									} 
							}
							$cellIterator->current()->setValue($this->getEffortFormula($task_effort_rows[$center->ID], $exportRow, $col));
							$cellIterator->next();
						}
						$col = $cellIterator->current()->getColumn();
						$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
						$sheet->getStyle($col . $row)->applyFromArray($borderColumnL);
						$cellIterator->current()->setValue($this->getYearSumFormula($col, $row));
						$cellIterator->next();
					}
					if(!$oneYearReport)
					{
						$col = $cellIterator->current()->getColumn();
						$cellIterator->current()->setValue($this->getTaskSumFormula($col, $row, count($this->year)));
						$cellIterator->next();
					}
					$cellIterator->current()->setValue($unit);
					$lastColorCol = $cellIterator->current()->getColumn();
					if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
					{
						$sheet->getStyle($firstColorCol . $row . ':' . $lastColorCol . $row)->applyFromArray($borderTask);
					}
					$rowIterator->next();
					$cellIterator = $rowIterator->current()->getCellIterator();
				}
			}
		}
		// Generate Total
		foreach (ProjectEACReportService::getProjectRowTypes() as $exportRow)
		{
			$row = $cellIterator->current()->getRow();
			$firstColorCol = $cellIterator->current()->getColumn();
			$sheet->getRowDimension($row)->setOutlineLevel(1);
			$sheet->getRowDimension($row)->setVisible(false);
			if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
				$cellIterator->current()->setValue('>>TOTAL<<');
			$cellIterator->next();
			$col = $cellIterator->current()->getColumn();
			$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
			$cellIterator->current()->setValue($exportRow);
			$cellIterator->next();
			foreach ($this->year as $year)
			{
				for ($i = 1; $i < 13; $i++)
				{
					$col = $cellIterator->current()->getColumn();
					if($year == date('Y'))
					{
						if($fiscalColumn == $col)
							{
								$sheet->getStyle($fiscalColumn . $row)->applyFromArray($borderColumnR);
								$sheet->getStyle($fiscalColumn . $row)->applyFromArray($borderColumnL);
							} 
					}
					$cellIterator->current()->setValue($this->getTotalFormula($effort_rows, $exportRow, $col));
					$cellIterator->next();
				}
				$col = $cellIterator->current()->getColumn();
				$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
				$sheet->getStyle($col . $row)->applyFromArray($borderColumnL);
				$cellIterator->current()->setValue($this->getYearSumFormula($col, $row));
				$cellIterator->next();
			}
			if(!$oneYearReport)
			{
				$col = $cellIterator->current()->getColumn();
				$cellIterator->current()->setValue($this->getTaskSumFormula($col, $row, count($this->year)));
				$cellIterator->next();
			}
			$cellIterator->current()->setValue($unit);
			$lastColorCol = $cellIterator->current()->getColumn();
			if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
			{
				$sheet->getStyle($firstColorCol . $row . ':' . $lastColorCol . $row)->applyFromArray($borderTask);
			}
			$rowIterator->next();
			$cellIterator = $rowIterator->current()->getCellIterator();
		}
		// ---EFFORT SECTION ENDS---
		// ---COSTS SECTION STARTS---
		// Generate Year header
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		$colBold = $cellIterator->current()->getColumn();
		$rowBold = $cellIterator->current()->getRow();
		$cellIterator->next();
		$cellIterator->next();
		$row = $cellIterator->current()->getRow();
		foreach ($this->year as $year)
		{
			$col = $cellIterator->current()->getColumn();
			$cellIterator->current()->setValue($year);
			$sheet->mergeCells($col . $row . ':' . PHPExcel_Cell::stringFromColumnIndex(PHPExcel_Cell::columnIndexFromString($col) + 11) . $row);
			for ($i = 1; $i < 14; $i++)
			{
				$cellIterator->next();
			}
		}
		$rowIterator->next();
		// Generate columns header
		$cellIterator = $rowIterator->current()->getCellIterator();
		$cellIterator->current()->setValue('--- COSTS ---');
		$cellIterator->next();
		$cellIterator->next();
		foreach ($this->year as $year)
		{
			foreach ($monthsShort as $month)
			{
				$cellIterator->current()->setValue($month);
				$cellIterator->next();
			}
			$cellIterator->current()->setValue('TOTAL ' . $year);
			$cellIterator->next();
		}
		if(!$oneYearReport)
		{
			$cellIterator->current()->setValue('TOTAL Overall Project');
			$cellIterator->next();
		}
		$cellIterator->current()->setValue('Unit');
		$sheet->getStyle($colBold . $rowBold . ':' . $cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
		$sheet->getStyle($colBold . $rowBold . ':' . $cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setBold(true);
		$cellIterator->next();
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		// Generate rows
		// Generate labour tasks
		foreach ($labour_centers as $center)
		{
			if (isset($labour_tasks[$center->ID]))
			{
				foreach (ProjectEACReportService::getProjectRowTypes() as $exportRow)
				{
					$row = $cellIterator->current()->getRow();
					$firstColorCol = $cellIterator->current()->getColumn();
					$sheet->getRowDimension($row)->setOutlineLevel(1);
					$sheet->getRowDimension($row)->setVisible(false);
					if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
					{
						$cellIterator->current()->setValue($center->Name);
						$cost_rows[$center->ID] = $row;
					}
					$cellIterator->next();
					$col = $cellIterator->current()->getColumn();
					$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
					$cellIterator->current()->setValue($exportRow);
					$cellIterator->next();
					foreach ($this->year as $year)
					{
						for ($i = 1; $i < 13; $i++)
						{
							$col = $cellIterator->current()->getColumn();
							if($year == date('Y'))
							{
								if($fiscalColumn == $col)
									{
										$sheet->getStyle($fiscalColumn . $row)->applyFromArray($borderColumnR);
										$sheet->getStyle($fiscalColumn . $row)->applyFromArray($borderColumnL);
									} 
							}
							$cellIterator->current()->setValue($this->getCostFormula($labour_tasks[$center->ID], $exportRow, $col, $labour_rates_rows, $center->ID, true));
							$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
							$cellIterator->next();
						}
						$col = $cellIterator->current()->getColumn();
						$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
						$sheet->getStyle($col . $row)->applyFromArray($borderColumnL);
						$cellIterator->current()->setValue($this->getYearSumFormula($col, $row));
						$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
						$cellIterator->next();
					}
					if(!$oneYearReport)
					{
						$col = $cellIterator->current()->getColumn();
						$cellIterator->current()->setValue($this->getTaskSumFormula($col, $row, count($this->year)));
						$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
						$cellIterator->next();
					}
					$cellIterator->current()->setValue($currency);
					$lastColorCol = $cellIterator->current()->getColumn();
					if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
					{
						$sheet->getStyle($firstColorCol . $row . ':' . $lastColorCol . $row)->applyFromArray($borderTask);
					}
					$rowIterator->next();
					$cellIterator = $rowIterator->current()->getCellIterator();
				}
			}
		}
		// Generate cost tasks
		foreach ($task_cost_rows as $task_prefix => $cost_tasks)
		{
			foreach (ProjectEACReportService::getProjectRowTypes() as $exportRow)
			{
				$row = $cellIterator->current()->getRow();
				$firstColorCol = $cellIterator->current()->getColumn();
				$sheet->getRowDimension($row)->setOutlineLevel(1);
				$sheet->getRowDimension($row)->setVisible(false);
				if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
				{
					$cellIterator->current()->setValue($task_prefix);
					$cost_rows[$task_prefix] = $row;
				}
				$cellIterator->next();
				$col = $cellIterator->current()->getColumn();
				$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
				$cellIterator->current()->setValue($exportRow);
				$cellIterator->next();
				foreach ($this->year as $year)
				{
					for ($i = 1; $i < 13; $i++)
					{
						$col = $cellIterator->current()->getColumn();
						if($year == date('Y'))
						{
							if($fiscalColumn == $col)
								{
									$sheet->getStyle($fiscalColumn . $row)->applyFromArray($borderColumnR);
									$sheet->getStyle($fiscalColumn . $row)->applyFromArray($borderColumnL);
								} 
						}
						$cellIterator->current()->setValue($this->getCostFormula($cost_tasks, $exportRow, $col));
						$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
						$cellIterator->next();
					}
					$col = $cellIterator->current()->getColumn();
					$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
					$sheet->getStyle($col . $row)->applyFromArray($borderColumnL);
					$cellIterator->current()->setValue($this->getYearSumFormula($col, $row));
					$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
					$cellIterator->next();
				}
				if(!$oneYearReport)
				{
					$col = $cellIterator->current()->getColumn();
					$cellIterator->current()->setValue($this->getTaskSumFormula($col, $row, count($this->year)));
					$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
					$cellIterator->next();
				}
				$cellIterator->current()->setValue($currency);
				$lastColorCol = $cellIterator->current()->getColumn();
				if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
				{
					$sheet->getStyle($firstColorCol . $row . ':' . $lastColorCol . $row)->applyFromArray($borderTask);
				}
				$rowIterator->next();
				$cellIterator = $rowIterator->current()->getCellIterator();
			}
		}
		// Generate Total
		foreach (ProjectEACReportService::getProjectRowTypes() as $exportRow)
		{
			$row = $cellIterator->current()->getRow();
			$firstColorCol = $cellIterator->current()->getColumn();
			$sheet->getRowDimension($row)->setOutlineLevel(1);
			$sheet->getRowDimension($row)->setVisible(false);
			if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
				$cellIterator->current()->setValue('>>TOTAL<<');
			$cellIterator->next();
			$col = $cellIterator->current()->getColumn();
			$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
			$cellIterator->current()->setValue($exportRow);
			$cellIterator->next();
			foreach ($this->year as $year)
			{
				for ($i = 1; $i < 13; $i++)
				{
					$col = $cellIterator->current()->getColumn();
					if($year == date('Y'))
					{
						if($fiscalColumn == $col)
							{
								$sheet->getStyle($fiscalColumn . $row)->applyFromArray($borderColumnR);
								$sheet->getStyle($fiscalColumn . $row)->applyFromArray($borderColumnL);
							} 
					}
					$cellIterator->current()->setValue($this->getTotalFormula($cost_rows, $exportRow, $col));
					$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
					$cellIterator->next();
				}
				$col = $cellIterator->current()->getColumn();
				$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
				$sheet->getStyle($col . $row)->applyFromArray($borderColumnL);
				$cellIterator->current()->setValue($this->getYearSumFormula($col, $row));
				$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
				$cellIterator->next();
			}
			if(!$oneYearReport)
			{
				$col = $cellIterator->current()->getColumn();
				$cellIterator->current()->setValue($this->getTaskSumFormula($col, $row, count($this->year)));
				$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
				$cellIterator->next();
			}
			$cellIterator->current()->setValue($currency);
			$lastColorCol = $cellIterator->current()->getColumn();
			if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
			{
				$sheet->getStyle($firstColorCol . $row . ':' . $lastColorCol . $row)->applyFromArray($borderTask);
			}
			$rowIterator->next();
			$cellIterator = $rowIterator->current()->getCellIterator();
		}
		// ---COSTS SECTION ENDS---
		// ---HON INVESTMENT SECTION STARTS---
		// Generate Year header
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		$colBold = $cellIterator->current()->getColumn();
		$rowBold = $cellIterator->current()->getRow();
		$cellIterator->next();
		$cellIterator->next();
		$row = $cellIterator->current()->getRow();
		foreach ($this->year as $year)
		{
			$col = $cellIterator->current()->getColumn();
			$cellIterator->current()->setValue($year);
			$sheet->mergeCells($col . $row . ':' . PHPExcel_Cell::stringFromColumnIndex(PHPExcel_Cell::columnIndexFromString($col) + 11) . $row);
			for ($i = 1; $i < 14; $i++)
			{
				$cellIterator->next();
			}
		}
		$rowIterator->next();
		// Generate columns header
		$cellIterator = $rowIterator->current()->getCellIterator();
		$cellIterator->current()->setValue('--- HON INVESTMENT ---');
		$cellIterator->next();
		$cellIterator->next();
		foreach ($this->year as $year)
		{
			foreach ($monthsShort as $month)
			{
				$cellIterator->current()->setValue($month);
				$cellIterator->next();
			}
			$cellIterator->current()->setValue('TOTAL ' . $year);
			$cellIterator->next();
		}
		if(!$oneYearReport)
		{
			$cellIterator->current()->setValue('TOTAL Overall Project');
			$cellIterator->next();
		}
		$cellIterator->current()->setValue('Unit');
		$sheet->getStyle($colBold . $rowBold . ':' . $cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
		$sheet->getStyle($colBold . $rowBold . ':' . $cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setBold(true);
		$cellIterator->next();
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		// Generate rows
		// Generate labour tasks
		foreach ($labour_centers as $center)
		{
			if (isset($labour_tasks[$center->ID]))
			{
				foreach (ProjectEACReportService::getProjectRowTypes() as $exportRow)
				{
					$row = $cellIterator->current()->getRow();
					$firstColorCol = $cellIterator->current()->getColumn();
					$sheet->getRowDimension($row)->setOutlineLevel(1);
					$sheet->getRowDimension($row)->setVisible(false);
					if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
					{
						$cellIterator->current()->setValue($center->Name);
						$honInvestment_rows[$center->ID] = $row;
					}
					$cellIterator->next();
					$col = $cellIterator->current()->getColumn();
					$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
					$cellIterator->current()->setValue($exportRow);
					$cellIterator->next();
					foreach ($this->year as $year)
					{
						for ($i = 1; $i < 13; $i++)
						{
							$col = $cellIterator->current()->getColumn();
							if($year == date('Y'))
							{
								if($fiscalColumn == $col)
									{
										$sheet->getStyle($fiscalColumn . $row)->applyFromArray($borderColumnR);
										$sheet->getStyle($fiscalColumn . $row)->applyFromArray($borderColumnL);
									} 
							}
							$cellIterator->current()->setValue($this->getHonInvestmentFormula($labour_tasks[$center->ID], $exportRow, $col, $cost_share_column, $labour_rates_rows, $center->ID, true));
							$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
							$cellIterator->next();
						}
						$col = $cellIterator->current()->getColumn();
						$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
						$sheet->getStyle($col . $row)->applyFromArray($borderColumnL);
						$cellIterator->current()->setValue($this->getYearSumFormula($col, $row));
						$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
						$cellIterator->next();
					}
					if(!$oneYearReport)
					{
						$col = $cellIterator->current()->getColumn();
						$cellIterator->current()->setValue($this->getTaskSumFormula($col, $row, count($this->year)));
						$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
						$cellIterator->next();
					}
					$cellIterator->current()->setValue($currency);
					$lastColorCol = $cellIterator->current()->getColumn();
					if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
					{
						$sheet->getStyle($firstColorCol . $row . ':' . $lastColorCol . $row)->applyFromArray($borderTask);
					}
					$rowIterator->next();
					$cellIterator = $rowIterator->current()->getCellIterator();
				}
			}
		}
		// Generate cost tasks
		foreach ($task_cost_rows as $task_prefix => $cost_tasks)
		{
			foreach (ProjectEACReportService::getProjectRowTypes() as $exportRow)
			{
				$row = $cellIterator->current()->getRow();
				$firstColorCol = $cellIterator->current()->getColumn();
				$sheet->getRowDimension($row)->setOutlineLevel(1);
				$sheet->getRowDimension($row)->setVisible(false);
				if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
				{
					$cellIterator->current()->setValue($task_prefix);
					$honInvestment_rows[$task_prefix] = $row;
				}
				$cellIterator->next();
				$col = $cellIterator->current()->getColumn();
				$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
				$cellIterator->current()->setValue($exportRow);
				$cellIterator->next();
				foreach ($this->year as $year)
				{
					for ($i = 1; $i < 13; $i++)
					{
						$col = $cellIterator->current()->getColumn();
						if($year == date('Y'))
						{
							if($fiscalColumn == $col)
								{
									$sheet->getStyle($fiscalColumn . $row)->applyFromArray($borderColumnR);
									$sheet->getStyle($fiscalColumn . $row)->applyFromArray($borderColumnL);
								} 
						}
						$cellIterator->current()->setValue($this->getHonInvestmentFormula($cost_tasks, $exportRow, $col, $cost_share_column));
						$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
						$cellIterator->next();
					}
					$col = $cellIterator->current()->getColumn();
					$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
					$sheet->getStyle($col . $row)->applyFromArray($borderColumnL);
					$cellIterator->current()->setValue($this->getYearSumFormula($col, $row));
					$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
					$cellIterator->next();
				}
				if(!$oneYearReport)
				{
					$col = $cellIterator->current()->getColumn();
					$cellIterator->current()->setValue($this->getTaskSumFormula($col, $row, count($this->year)));
					$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
					$cellIterator->next();
				}
				$cellIterator->current()->setValue($currency);
				$lastColorCol = $cellIterator->current()->getColumn();
				if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
				{
					$sheet->getStyle($firstColorCol . $row . ':' . $lastColorCol . $row)->applyFromArray($borderTask);
				}
				$rowIterator->next();
				$cellIterator = $rowIterator->current()->getCellIterator();
			}
		}
		// Generate Total
		foreach (ProjectEACReportService::getProjectRowTypes() as $exportRow)
		{
			$row = $cellIterator->current()->getRow();
			$firstColorCol = $cellIterator->current()->getColumn();
			$sheet->getRowDimension($row)->setOutlineLevel(1);
			$sheet->getRowDimension($row)->setVisible(false);
			if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
				$cellIterator->current()->setValue('>>TOTAL<<');
			$cellIterator->next();
			$col = $cellIterator->current()->getColumn();
			$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
			$cellIterator->current()->setValue($exportRow);
			$cellIterator->next();
			foreach ($this->year as $year)
			{
				for ($i = 1; $i < 13; $i++)
				{
					$col = $cellIterator->current()->getColumn();
					if($year == date('Y'))
					{
						if($fiscalColumn == $col)
							{
								$sheet->getStyle($fiscalColumn . $row)->applyFromArray($borderColumnR);
								$sheet->getStyle($fiscalColumn . $row)->applyFromArray($borderColumnL);
							} 
					}
					$cellIterator->current()->setValue($this->getTotalFormula($honInvestment_rows, $exportRow, $col));
					$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
					$cellIterator->next();
				}
				$col = $cellIterator->current()->getColumn();
				$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
				$sheet->getStyle($col . $row)->applyFromArray($borderColumnL);
				$cellIterator->current()->setValue($this->getYearSumFormula($col, $row));
				$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
				$cellIterator->next();
			}
			if(!$oneYearReport)
			{
				$col = $cellIterator->current()->getColumn();
				$cellIterator->current()->setValue($this->getTaskSumFormula($col, $row, count($this->year)));
				$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
				$cellIterator->next();
			}
			$cellIterator->current()->setValue($currency);
			$lastColorCol = $cellIterator->current()->getColumn();
			if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
			{
				$sheet->getStyle($firstColorCol . $row . ':' . $lastColorCol . $row)->applyFromArray($borderTask);
			}
			$rowIterator->next();
			$cellIterator = $rowIterator->current()->getCellIterator();
		}
		// ---HON INVESTMENT SECTION ENDS---
		// ---QUARTERLY SPEND PROFILE STARTS---
		// Generate Year header
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		$colBold = $cellIterator->current()->getColumn();
		$rowBold = $cellIterator->current()->getRow();
		$cellIterator->next();
		$cellIterator->next();
		$row = $cellIterator->current()->getRow();
		foreach ($this->year as $year)
		{
			$col = $cellIterator->current()->getColumn();
			$cellIterator->current()->setValue($year);
			$sheet->mergeCells($col . $row . ':' . PHPExcel_Cell::stringFromColumnIndex(PHPExcel_Cell::columnIndexFromString($col) + 11) . $row);
			for ($i = 1; $i < 14; $i++)
			{
				$cellIterator->next();
			}
		}
		$rowIterator->next();
		// Generate columns header
		$cellIterator = $rowIterator->current()->getCellIterator();
		$cellIterator->current()->setValue('--- QUARTERLY SPEND PROFILE ---');
		$cellIterator->next();
		$cellIterator->next();
		$row = $cellIterator->current()->getRow();
		foreach ($this->year as $year)
		{
			for($i = 1; $i < 5; $i++)
			{
				$cellIterator->current()->setValue('Q' . $i);
				$col = $cellIterator->current()->getColumn();
				$sheet->mergeCells($col . $row . ':' . PHPExcel_Cell::stringFromColumnIndex(PHPExcel_Cell::columnIndexFromString($col) + 1) . $row);
				$cellIterator->next();
				$cellIterator->next();
				$cellIterator->next();
				
			}
			$cellIterator->current()->setValue('TOTAL ' . $year);
			$cellIterator->next();
		}
		if(!$oneYearReport)
		{
			$cellIterator->current()->setValue('TOTAL Overall Project');
			$cellIterator->next();
		}
		$cellIterator->current()->setValue('Unit');
		$sheet->getStyle($colBold . $rowBold . ':' . $cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
		$sheet->getStyle($colBold . $rowBold . ':' . $cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setBold(true);
		$cellIterator->next();
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		// Generate rows
		// Generate labour tasks
		foreach ($labour_centers as $center)
		{
			if (isset($labour_tasks[$center->ID]))
			{
				foreach (ProjectEACReportService::getProjectRowTypes() as $exportRow)
				{
					$row = $cellIterator->current()->getRow();
					$firstColorCol = $cellIterator->current()->getColumn();
					$sheet->getRowDimension($row)->setOutlineLevel(1);
					$sheet->getRowDimension($row)->setVisible(false);
					if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
					{
						$cellIterator->current()->setValue($center->Name);
						$costQ_rows[] = $row;
					}
					$cellIterator->next();
					$col = $cellIterator->current()->getColumn();
					$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
					$cellIterator->current()->setValue($exportRow);
					$cellIterator->next();
					foreach ($this->year as $year)
					{
						for ($i = 0; $i < 4; $i++)
						{
							$col = $cellIterator->current()->getColumn();
							$cellIterator->current()->setValue($this->getQFormula($cost_rows[$center->ID], $col, $exportRow));
							$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
							$sheet->mergeCells($col . $row . ':' . PHPExcel_Cell::stringFromColumnIndex(PHPExcel_Cell::columnIndexFromString($col) + 1) . $row);
							$cellIterator->next();
							$cellIterator->next();
							$cellIterator->next();
						}
						$col = $cellIterator->current()->getColumn();
						$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
						$sheet->getStyle($col . $row)->applyFromArray($borderColumnL);
						$cellIterator->current()->setValue($this->getYearSumFormula($col, $row));
						$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
						$cellIterator->next();
					}
					if(!$oneYearReport)
					{
						$col = $cellIterator->current()->getColumn();
						$cellIterator->current()->setValue($this->getTaskSumFormula($col, $row, count($this->year)));
						$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
						$cellIterator->next();
					}
					$cellIterator->current()->setValue($currency);
					$lastColorCol = $cellIterator->current()->getColumn();
					if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
					{
						$sheet->getStyle($firstColorCol . $row . ':' . $lastColorCol . $row)->applyFromArray($borderTask);
					}
					$rowIterator->next();
					$cellIterator = $rowIterator->current()->getCellIterator();
				}
			}
		}
		// Generate cost tasks
		foreach ($task_cost_rows as $task_prefix => $cost_tasks)
		{
			foreach (ProjectEACReportService::getProjectRowTypes() as $exportRow)
			{
				$row = $cellIterator->current()->getRow();
				$firstColorCol = $cellIterator->current()->getColumn();
				$sheet->getRowDimension($row)->setOutlineLevel(1);
				$sheet->getRowDimension($row)->setVisible(false);
				if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
				{
					$cellIterator->current()->setValue($task_prefix);
					$costQ_rows[] = $row;
				}
				$cellIterator->next();
				$col = $cellIterator->current()->getColumn();
				$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
				$cellIterator->current()->setValue($exportRow);
				$cellIterator->next();
				foreach ($this->year as $year)
				{
					for ($i = 0; $i < 4; $i++)
					{
						$col = $cellIterator->current()->getColumn();
						$cellIterator->current()->setValue($this->getQFormula($cost_rows[$task_prefix], $col, $exportRow));
						$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
						$sheet->mergeCells($col . $row . ':' . PHPExcel_Cell::stringFromColumnIndex(PHPExcel_Cell::columnIndexFromString($col) + 1) . $row);
						$cellIterator->next();
						$cellIterator->next();
						$cellIterator->next();
					}
					$col = $cellIterator->current()->getColumn();
					$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
					$sheet->getStyle($col . $row)->applyFromArray($borderColumnL);
					$cellIterator->current()->setValue($this->getYearSumFormula($col, $row));
					$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
					$cellIterator->next();
				}
				if(!$oneYearReport)
				{
					$col = $cellIterator->current()->getColumn();
					$cellIterator->current()->setValue($this->getTaskSumFormula($col, $row, count($this->year)));
					$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
					$cellIterator->next();
				}
				$cellIterator->current()->setValue($currency);
				$lastColorCol = $cellIterator->current()->getColumn();
				if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
				{
					$sheet->getStyle($firstColorCol . $row . ':' . $lastColorCol . $row)->applyFromArray($borderTask);
				}
				$rowIterator->next();
				$cellIterator = $rowIterator->current()->getCellIterator();
			}
		}
		// Generate Total
		foreach (ProjectEACReportService::getProjectRowTypes() as $exportRow)
		{
			$row = $cellIterator->current()->getRow();
			$firstColorCol = $cellIterator->current()->getColumn();
			$sheet->getRowDimension($row)->setOutlineLevel(1);
			$sheet->getRowDimension($row)->setVisible(false);
			if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
				$cellIterator->current()->setValue('>>TOTAL<<');
			$cellIterator->next();
			$col = $cellIterator->current()->getColumn();
			$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
			$cellIterator->current()->setValue($exportRow);
			$cellIterator->next();
			foreach ($this->year as $year)
			{
				for ($i = 1; $i < 5; $i++)
				{
					$col = $cellIterator->current()->getColumn();
					$cellIterator->current()->setValue($this->getTotalFormula($costQ_rows, $exportRow, $col));
					$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
					$sheet->mergeCells($col . $row . ':' . PHPExcel_Cell::stringFromColumnIndex(PHPExcel_Cell::columnIndexFromString($col) + 1) . $row);
					$cellIterator->next();
					$cellIterator->next();
					$cellIterator->next();
				}
				$col = $cellIterator->current()->getColumn();
				$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
				$sheet->getStyle($col . $row)->applyFromArray($borderColumnL);
				$cellIterator->current()->setValue($this->getYearSumFormula($col, $row));
				$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
				$cellIterator->next();
			}
			if(!$oneYearReport)
			{
				$col = $cellIterator->current()->getColumn();
				$cellIterator->current()->setValue($this->getTaskSumFormula($col, $row, count($this->year)));
				$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
				$cellIterator->next();
			}
			$cellIterator->current()->setValue($currency);
			$lastColorCol = $cellIterator->current()->getColumn();
			if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
			{
				$sheet->getStyle($firstColorCol . $row . ':' . $lastColorCol . $row)->applyFromArray($borderTask);
			}
			$rowIterator->next();
			$cellIterator = $rowIterator->current()->getCellIterator();
		}
		// ---QUARTERLY SPEND PROFILE  SECTION ENDS---
		// ---QUARTERLY HON INVEST SECTION STARTS---
		// Generate Year header
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		$colBold = $cellIterator->current()->getColumn();
		$rowBold = $cellIterator->current()->getRow();
		$cellIterator->next();
		$cellIterator->next();
		$row = $cellIterator->current()->getRow();
		foreach ($this->year as $year)
		{
			$col = $cellIterator->current()->getColumn();
			$cellIterator->current()->setValue($year);
			$sheet->mergeCells($col . $row . ':' . PHPExcel_Cell::stringFromColumnIndex(PHPExcel_Cell::columnIndexFromString($col) + 11) . $row);
			for ($i = 1; $i < 14; $i++)
			{
				$cellIterator->next();
			}
		}
		$rowIterator->next();
		// Generate columns header
		$cellIterator = $rowIterator->current()->getCellIterator();
		$cellIterator->current()->setValue('--- QUARTERLY HON INVEST ---');
		$cellIterator->next();
		$cellIterator->next();
		$row = $cellIterator->current()->getRow();
		foreach ($this->year as $year)
		{
			for($i = 1; $i < 5; $i++)
			{
				$cellIterator->current()->setValue('Q' . $i);
				$col = $cellIterator->current()->getColumn();
				$sheet->mergeCells($col . $row . ':' . PHPExcel_Cell::stringFromColumnIndex(PHPExcel_Cell::columnIndexFromString($col) + 1) . $row);
				$cellIterator->next();
				$cellIterator->next();
				$cellIterator->next();
				
			}
			$cellIterator->current()->setValue('TOTAL ' . $year);
			$cellIterator->next();
		}
		if(!$oneYearReport)
		{
			$cellIterator->current()->setValue('TOTAL Overall Project');
			$cellIterator->next();
		}
		$cellIterator->current()->setValue('Unit');
		$sheet->getStyle($colBold . $rowBold . ':' . $cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
		$sheet->getStyle($colBold . $rowBold . ':' . $cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setBold(true);
		$cellIterator->next();
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		// Generate rows
		// Generate labour tasks
		foreach ($labour_centers as $center)
		{
			if (isset($labour_tasks[$center->ID]))
			{
				foreach (ProjectEACReportService::getProjectRowTypes() as $exportRow)
				{
					$row = $cellIterator->current()->getRow();
					$firstColorCol = $cellIterator->current()->getColumn();
					$sheet->getRowDimension($row)->setOutlineLevel(1);
					$sheet->getRowDimension($row)->setVisible(false);
					if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
					{
						$cellIterator->current()->setValue($center->Name);
						$honInvestmentQ_rows[] = $row;
					}
					$cellIterator->next();
					$col = $cellIterator->current()->getColumn();
					$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
					$cellIterator->current()->setValue($exportRow);
					$cellIterator->next();
					foreach ($this->year as $year)
					{
						for ($i = 0; $i < 4; $i++)
						{
							$col = $cellIterator->current()->getColumn();
							$cellIterator->current()->setValue($this->getQFormula($honInvestment_rows[$center->ID], $col, $exportRow));
							$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
							$sheet->mergeCells($col . $row . ':' . PHPExcel_Cell::stringFromColumnIndex(PHPExcel_Cell::columnIndexFromString($col) + 1) . $row);
							$cellIterator->next();
							$cellIterator->next();
							$cellIterator->next();
						}
						$col = $cellIterator->current()->getColumn();
						$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
						$sheet->getStyle($col . $row)->applyFromArray($borderColumnL);
						$cellIterator->current()->setValue($this->getYearSumFormula($col, $row));
						$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
						$cellIterator->next();
					}
					if(!$oneYearReport)
					{
						$col = $cellIterator->current()->getColumn();
						$cellIterator->current()->setValue($this->getTaskSumFormula($col, $row, count($this->year)));
						$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
						$cellIterator->next();
					}
					$cellIterator->current()->setValue($currency);
					$lastColorCol = $cellIterator->current()->getColumn();
					if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
					{
						$sheet->getStyle($firstColorCol . $row . ':' . $lastColorCol . $row)->applyFromArray($borderTask);
					}
					$rowIterator->next();
					$cellIterator = $rowIterator->current()->getCellIterator();
				}
			}
		}
		// Generate cost tasks
		foreach ($task_cost_rows as $task_prefix => $cost_tasks)
		{
			foreach (ProjectEACReportService::getProjectRowTypes() as $exportRow)
			{
				$row = $cellIterator->current()->getRow();
				$firstColorCol = $cellIterator->current()->getColumn();
				$sheet->getRowDimension($row)->setOutlineLevel(1);
				$sheet->getRowDimension($row)->setVisible(false);
				if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
				{
					$cellIterator->current()->setValue($task_prefix);
					$honInvestmentQ_rows[] = $row;
				}
				$cellIterator->next();
				$col = $cellIterator->current()->getColumn();
				$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
				$cellIterator->current()->setValue($exportRow);
				$cellIterator->next();
				foreach ($this->year as $year)
				{
					for ($i = 0; $i < 4; $i++)
					{
						$col = $cellIterator->current()->getColumn();
						$cellIterator->current()->setValue($this->getQFormula($honInvestment_rows[$task_prefix], $col, $exportRow));
						$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
						$sheet->mergeCells($col . $row . ':' . PHPExcel_Cell::stringFromColumnIndex(PHPExcel_Cell::columnIndexFromString($col) + 1) . $row);
						$cellIterator->next();
						$cellIterator->next();
						$cellIterator->next();
					}
					$col = $cellIterator->current()->getColumn();
					$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
					$sheet->getStyle($col . $row)->applyFromArray($borderColumnL);
					$cellIterator->current()->setValue($this->getYearSumFormula($col, $row));
					$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
					$cellIterator->next();
				}
				if(!$oneYearReport)
				{
					$col = $cellIterator->current()->getColumn();
					$cellIterator->current()->setValue($this->getTaskSumFormula($col, $row, count($this->year)));
					$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
					$cellIterator->next();
				}
				$cellIterator->current()->setValue($currency);
				$lastColorCol = $cellIterator->current()->getColumn();
				if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
				{
					$sheet->getStyle($firstColorCol . $row . ':' . $lastColorCol . $row)->applyFromArray($borderTask);
				}
				$rowIterator->next();
				$cellIterator = $rowIterator->current()->getCellIterator();
			}
		}
		// Generate Total
		foreach (ProjectEACReportService::getProjectRowTypes() as $exportRow)
		{
			$row = $cellIterator->current()->getRow();
			$firstColorCol = $cellIterator->current()->getColumn();
			$sheet->getRowDimension($row)->setOutlineLevel(1);
			$sheet->getRowDimension($row)->setVisible(false);
			if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
				$cellIterator->current()->setValue('>>TOTAL<<');
			$cellIterator->next();
			$col = $cellIterator->current()->getColumn();
			$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
			$cellIterator->current()->setValue($exportRow);
			$cellIterator->next();
			foreach ($this->year as $year)
			{
				for ($i = 1; $i < 5; $i++)
				{
					$col = $cellIterator->current()->getColumn();
					$cellIterator->current()->setValue($this->getTotalFormula($honInvestmentQ_rows, $exportRow, $col));
					$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
					$sheet->mergeCells($col . $row . ':' . PHPExcel_Cell::stringFromColumnIndex(PHPExcel_Cell::columnIndexFromString($col) + 1) . $row);
					$cellIterator->next();
					$cellIterator->next();
					$cellIterator->next();
				}
				$col = $cellIterator->current()->getColumn();
				$sheet->getStyle($col . $row)->applyFromArray($borderColumnR);
				$sheet->getStyle($col . $row)->applyFromArray($borderColumnL);
				$cellIterator->current()->setValue($this->getYearSumFormula($col, $row));
				$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
				$cellIterator->next();
			}
			if(!$oneYearReport)
			{
				$col = $cellIterator->current()->getColumn();
				$cellIterator->current()->setValue($this->getTaskSumFormula($col, $row, count($this->year)));
				$sheet->getStyle($col . $row)->getNumberFormat()->setFormatCode('### ### ##0');
				$cellIterator->next();
			}
			$cellIterator->current()->setValue($currency);
			$lastColorCol = $cellIterator->current()->getColumn();
			if(ProjectEACReportService::PROJECT_REPORT_PLAN == $exportRow)
			{
				$sheet->getStyle($firstColorCol . $row . ':' . $lastColorCol . $row)->applyFromArray($borderTask);
			}
			$rowIterator->next();
			$cellIterator = $rowIterator->current()->getCellIterator();
		}
		//---QUARTERLY HON INVEST SECTION ENDS---
		//---WORKER AGREGATION SECTION STARTS---
		$sheet3 = $objPHPExcel->createSheet(2);
		$sheet3->setTitle("Workers Aggregation");
		$sheet3->freezePane('C3');
		$rowIterator = $sheet3->getRowIterator();
		$cellIterator = $rowIterator->current()->getCellIterator();
		// Generate Year header
		$cellIterator = $rowIterator->current()->getCellIterator();
		$cellIterator->next();
		$cellIterator->next();
		$row = $cellIterator->current()->getRow();
		foreach ($this->year as $year)
		{
			$col = $cellIterator->current()->getColumn();
			$cellIterator->current()->setValue($year);
			$sheet3->mergeCells($col . $row . ':' . PHPExcel_Cell::stringFromColumnIndex(PHPExcel_Cell::columnIndexFromString($col) + 11) . $row);
			for ($i = 1; $i < 14; $i++)
			{
				$cellIterator->next();
			}
		}
		$rowIterator->next();

		// Generate columns header
		$cellIterator = $rowIterator->current()->getCellIterator();
		$cellIterator->current()->setValue('--- WORKER ---');
		$sheet3->getColumnDimension($cellIterator->current()->getColumn())->setWidth($columnAWidth);
		$cellIterator->next();
		$sheet3->getColumnDimension($cellIterator->current()->getColumn())->setWidth($columnBWidth);
		$cellIterator->next();
		foreach ($this->year as $year)
		{
			foreach ($monthsShort as $month)
			{
				$cellIterator->current()->setValue($month);
				$col = $cellIterator->current()->getColumn();
				$sheet3->getColumnDimension($col)->setOutlineLevel(1);
				if(!$oneYearReport)
				{
					$sheet3->getColumnDimension($col)->setVisible(false);
				}
				$cellIterator->next();
			}
			$cellIterator->current()->setValue('TOTAL ' . $year);
			$sheet3->getColumnDimension($cellIterator->current()->getColumn())->setWidth($columnTotalYear);
			$cellIterator->next();
		}
		if(!$oneYearReport)
		{
			$cellIterator->current()->setValue('TOTAL Overall');
			$sheet3->getColumnDimension($cellIterator->current()->getColumn())->setWidth($columnTotalOverall);
			$cellIterator->next();
		}
		$cellIterator->current()->setValue('Unit');
		$sheet3->getStyle('A1:' . $cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);
		$sheet3->getStyle('A1:' . $cellIterator->current()->getColumn() . $cellIterator->current()->getRow())->getFont()->setBold(true);
		$cellIterator->next();
		$rowIterator->next();
		$cellIterator = $rowIterator->current()->getCellIterator();
		// generate rows
		$changeWorker = 0;
		foreach ($worker_rows as $name => $rows)
		{
			foreach (ProjectEACReportService::getProjectRowTypesForWorkerAggregation() as $exportRow)
			{
				$row = $cellIterator->current()->getRow();
				$cellIterator->current()->setValue($name);
				$firstColorCol = $cellIterator->current()->getColumn();
				$cellIterator->next();
				$cellIterator->current()->setValue($exportRow);
				$col = $cellIterator->current()->getColumn();
				$sheet3->getStyle($col . $row)->applyFromArray($borderColumnR);
				$cellIterator->next();
				foreach ($this->year as $year)
				{
					for ($i = 1; $i < 13; $i++)
					{
						$col = $cellIterator->current()->getColumn();
						if($year == date('Y'))
						{
							if($fiscalColumn == $col)
								{
									$sheet3->getStyle($fiscalColumn . $row)->applyFromArray($borderColumnR);
									$sheet3->getStyle($fiscalColumn . $row)->applyFromArray($borderColumnL);
								} 
						}
						$cellIterator->current()->setValue($this->getWorkerAgregationFormula($rows, $exportRow, $col));
						$cellIterator->next();
					}
					$col = $cellIterator->current()->getColumn();
					$sheet3->getStyle($col . $row)->applyFromArray($borderColumnR);
					$sheet3->getStyle($col . $row)->applyFromArray($borderColumnL);
					$cellIterator->current()->setValue($this->getYearSumFormula($col, $row));
					$cellIterator->next();
				}
				if(!$oneYearReport)
				{
					$col = $cellIterator->current()->getColumn();
					$cellIterator->current()->setValue($this->getTaskSumFormula($col, $row, count($this->year)));
					$cellIterator->next();
				}
				$cellIterator->current()->setValue($unit);
				$lastColorCol = $cellIterator->current()->getColumn();
				$this->setColor($row, $firstColorCol, $lastColorCol, $exportRow, $changeWorker, $sheet3);
				if(ProjectEACReportService::PROJECT_REPORT_ASSIGNED == $exportRow)
				{
					$sheet3->getStyle($firstColorCol . $row . ':' . $lastColorCol . $row)->applyFromArray($borderTask);
				}
				$rowIterator->next();
				$cellIterator = $rowIterator->current()->getCellIterator();
			}
			$changeWorker++;
		}
		// ---WORKER AGREGATION SECTION ENDS---
		// This set active sheet
		$objPHPExcel->setActiveSheetIndex(0);
		
		if (!is_null($this->uiFreezeToken))
				setcookie('fileDownloadToken', $this->uiFreezeToken);
		
		$reportName = '';
		if(!$oneYearReport)
		{
			$reportName = $objectName . ' - Overall ' . $this->reportType . ' Report.xlsx';
		}
		else
		{
			$reportName = $objectName . ' - ' . $this->year[0] . ' ' . $this->reportType . ' Report.xlsx';
		}
		//dump($reportName); die;
		header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
		header('Content-Disposition: attachment; filename="' . $reportName . '"');
		header('Cache-Control: max-age=0');
		$objWriter = PHPExcel_IOFactory::createWriter($objPHPExcel, 'Excel2007');
		ob_end_clean();
		$objWriter->save("php://output");
		$this->logger->log($this->reportType . ' Report generation end: ' . date('m-d-Y H:i:s'), ILogger::INFO);
		exit;
		//$lock->unlock();
	}

	/**
	 * Returns array of available report rows
	 * @return array arr[string] == string .. arr[PROJECT_REPORT_ACTUALS] == PROJECT_REPORT_ACTUALS
	 */
	private static function getProjectRowTypes()
	{
		$arrRows = array(
			self::PROJECT_REPORT_PLAN => self::PROJECT_REPORT_PLAN,
			self::PROJECT_REPORT_ASSIGNED => self::PROJECT_REPORT_ASSIGNED,
			self::PROJECT_REPORT_ACTUALS => self::PROJECT_REPORT_ACTUALS,
			self::PROJECT_REPORT_ACTUALS_EAC => self::PROJECT_REPORT_ACTUALS_EAC,
			self::PROJECT_REPORT_DELTA_PLAN_ACTUALS_EAC => self::PROJECT_REPORT_DELTA_PLAN_ACTUALS_EAC,
			self::PROJECT_REPORT_DELTA_ASSIGNED_ACTUALS_EAC => self::PROJECT_REPORT_DELTA_ASSIGNED_ACTUALS_EAC
		);
		return $arrRows;
	}

	/**
	 * Returns array of available report rows for person
	 * @return array arr[string] == string .. arr[PROJECT_REPORT_ACTUALS] == PROJECT_REPORT_ACTUALS
	 */
	private static function getProjectRowTypesForPerson()
	{
		$arrRows = array(
			self::PROJECT_REPORT_ASSIGNED => self::PROJECT_REPORT_ASSIGNED,
			self::PROJECT_REPORT_ACTUALS => self::PROJECT_REPORT_ACTUALS,
			self::PROJECT_REPORT_ACTUALS_EAC => self::PROJECT_REPORT_ACTUALS_EAC,
			self::PROJECT_REPORT_DELTA_ASSIGNED_ACTUALS_EAC => self::PROJECT_REPORT_DELTA_ASSIGNED_ACTUALS_EAC
		);
		return $arrRows;
	}
	
	/**
	 * Returns array of available report rows for person
	 * @return array arr[string] == string .. arr[PROJECT_REPORT_ACTUALS] == PROJECT_REPORT_ACTUALS
	 */
	private static function getProjectRowTypesForWorkerAggregation()
	{
		$arrRows = array(
			self::PROJECT_REPORT_ASSIGNED => self::PROJECT_REPORT_ASSIGNED,
			self::PROJECT_REPORT_ACTUALS => self::PROJECT_REPORT_ACTUALS,
			self::PROJECT_REPORT_DELTA_ASSIGNED_ACTUALS_EAC => self::PROJECT_REPORT_DELTA_ASSIGNED_ACTUALS_EAC
		);
		return $arrRows;
	}

	private function getYearSumFormula($col, $row)
	{
		return '=SUM(' . PHPExcel_Cell::stringFromColumnIndex(PHPExcel_Cell::columnIndexFromString($col) - 13) . $row . ':' . PHPExcel_Cell::stringFromColumnIndex(PHPExcel_Cell::columnIndexFromString($col) - 2) . $row . ')';
	}

	private function getTaskSumFormula($col, $row, $yearNumber)
	{
		$formula = array();
		for ($i = 0; $i < $yearNumber; $i++)
		{
			$formula[] = PHPExcel_Cell::stringFromColumnIndex((PHPExcel_Cell::columnIndexFromString($col) - 2) - 13 * $i) . $row;
		}
		return '=' . implode('+', $formula);
	}
	
	private function getYearQSumFormula($col, $row)
	{
		return '=SUM(' . PHPExcel_Cell::stringFromColumnIndex(PHPExcel_Cell::columnIndexFromString($col) - 5) . $row . ':' . PHPExcel_Cell::stringFromColumnIndex(PHPExcel_Cell::columnIndexFromString($col) - 2) . $row . ')';
	}

	private function getTaskQSumFormula($col, $row, $yearNumber)
	{
		$formula = array();
		for ($i = 0; $i < $yearNumber; $i++)
		{
			$formula[] = PHPExcel_Cell::stringFromColumnIndex((PHPExcel_Cell::columnIndexFromString($col) - 2) - 5 * $i) . $row;
		}
		return '=' . implode('+', $formula);
	}

	private function getTaskPlanDeltaFormula($col, $row)
	{
		return '=' . $col . ($row - 4) . '-' . $col . ($row - 1);
	}

	private function getTaskAssignedDeltaFormula($col, $row)
	{
		return '=' . $col . ($row - 4) . '-' . $col . ($row - 3);
	}

	private function getPersonAssignedDeltaFormula($col, $row)
	{
		return '=' . $col . ($row - 3) . '-' . $col . ($row - 2);
	}

	private function getPersonAssignedActualsAndEACFormula($col, $row, $personsNumber)
	{
		if (count($personsNumber) > 0)
		{
			$formula = array();
			for ($i = 1; $i <= count($personsNumber); $i++)
			{
				if ($i == 1)
				{
					$formula[] = $col . ($row + 5);
				}
				else
				{
					$formula[] = $col . ($row + 5 + (4 * ($i - 1)));
				}
			}
			return '=' . implode('+', $formula);
		}
		else
		{
			return 0;
		}
	}

	private function getActualsCost($costActuals, $year, $month)
	{
		if (isset($costActuals[$year]))
		{
			if (isset($costActuals[$year][$month]))
			{
				return $costActuals[$year][$month];
			}
			else
			{
				return 0;
			}
		}
		else
		{
			return 0;
		}
	}

	private function getAssignedCost($costAssignments, $taskPlan, $year, $month)
	{
		if (isset($costAssignments[$year]))
		{
			if (isset($costAssignments[$year][$month]))
			{
					return $costAssignments[$year][$month]->Cost;
			}
			else
			{
				if($taskPlan != null)
					return $taskPlan->getMonthPlan($month)->Budget;
				else
					return 0;
			}
		}
		else
		{
			if($taskPlan != null)
				return $taskPlan->getMonthPlan($month)->Budget;
			else
				return 0;
		}
	}
	
	private function getEffortFormula(array $task_effort_rows, $exportRow, $col)
	{
		$formula = array();
		foreach($task_effort_rows as $row)
		{
			switch ($exportRow)
			{
				case ProjectEACReportService::PROJECT_REPORT_PLAN:
					$formula[] = $col . $row;
					break;
				case ProjectEACReportService::PROJECT_REPORT_ASSIGNED:
					$formula[] = $col . ($row + 1);
					break;
				case ProjectEACReportService::PROJECT_REPORT_ACTUALS:
					$formula[] = $col . ($row + 2);
					break;
				case ProjectEACReportService::PROJECT_REPORT_ACTUALS_EAC:
					$formula[] = $col . ($row + 3);
					break;
				case ProjectEACReportService::PROJECT_REPORT_DELTA_PLAN_ACTUALS_EAC:
					$formula[] = $col . ($row + 4);
					break;
				case ProjectEACReportService::PROJECT_REPORT_DELTA_ASSIGNED_ACTUALS_EAC:
					$formula[] = $col . ($row + 5);
					break;
			}
		}
		return '=' . implode('+', $formula);
	}
	
	private function getCostFormula(array $task_rows, $exportRow, $col, $labour_rates_rows = null, $costCenterID = null, $isLabour = false)
	{
		$formula = array();
		foreach($task_rows as $row)
		{
			switch ($exportRow)
			{
				case ProjectEACReportService::PROJECT_REPORT_PLAN:
					$formula[] = $col . $row;
					break;
				case ProjectEACReportService::PROJECT_REPORT_ASSIGNED:
					$formula[] = $col . ($row + 1);
					break;
				case ProjectEACReportService::PROJECT_REPORT_ACTUALS:
					$formula[] = $col . ($row + 2);
					break;
				case ProjectEACReportService::PROJECT_REPORT_ACTUALS_EAC:
					$formula[] = $col . ($row + 3);
					break;
				case ProjectEACReportService::PROJECT_REPORT_DELTA_PLAN_ACTUALS_EAC:
					$formula[] = $col . ($row + 4);
					break;
				case ProjectEACReportService::PROJECT_REPORT_DELTA_ASSIGNED_ACTUALS_EAC:
					$formula[] = $col . ($row + 5);
					break;
			}
		}
		if($isLabour)
			return '=(' . implode('+', $formula) . ")*'Labour rates'!". $col . $labour_rates_rows[$costCenterID];
		else
			return '=' . implode('+', $formula);
	}
	
	private function getHonInvestmentFormula(array $task_rows, $exportRow, $col, $cost_shares, $labour_rates_rows = null, $costCenterID = null, $isLabour = false)
	{
		$formula = array();
		foreach($task_rows as $taskID => $row)
		{
			switch ($exportRow)
			{
				//H7*(100-Q7)/100
				case ProjectEACReportService::PROJECT_REPORT_PLAN:
					$formula[] = $col . $row . '*(100-' . $cost_shares[$taskID] . $row . ')/100';
					break;
				case ProjectEACReportService::PROJECT_REPORT_ASSIGNED:
					$formula[] = $col . ($row + 1) . '*(100-' . $cost_shares[$taskID] . $row . ')/100';
					break;
				case ProjectEACReportService::PROJECT_REPORT_ACTUALS:
					$formula[] = $col . ($row + 2) . '*(100-' . $cost_shares[$taskID] . $row . ')/100';
					break;
				case ProjectEACReportService::PROJECT_REPORT_ACTUALS_EAC:
					$formula[] = $col . ($row + 3) . '*(100-' . $cost_shares[$taskID] . $row . ')/100';
					break;
				case ProjectEACReportService::PROJECT_REPORT_DELTA_PLAN_ACTUALS_EAC:
					$formula[] = $col . ($row + 4) . '*(100-' . $cost_shares[$taskID] . $row . ')/100';
					break;
				case ProjectEACReportService::PROJECT_REPORT_DELTA_ASSIGNED_ACTUALS_EAC:
					$formula[] = $col . ($row + 5) . '*(100-' . $cost_shares[$taskID] . $row . ')/100';
					break;
			}
		}
		if($isLabour)
			return '=(' . implode('+', $formula) . ")*'Labour rates'!". $col . $labour_rates_rows[$costCenterID];
		else
			return '=' . implode('+', $formula);
	}
	
	private function getQFormula($rows, $col, $exportRow)
	{
		$formula = array();
		for($i = -1; $i < 2; $i++)
		{
			switch ($exportRow)
			{
				case ProjectEACReportService::PROJECT_REPORT_PLAN:
					$formula[] = PHPExcel_Cell::stringFromColumnIndex(PHPExcel_Cell::columnIndexFromString($col) + $i) . $rows;
					break;
				case ProjectEACReportService::PROJECT_REPORT_ASSIGNED:
					$formula[] = PHPExcel_Cell::stringFromColumnIndex(PHPExcel_Cell::columnIndexFromString($col) + $i) . ($rows + 1);
					break;
				case ProjectEACReportService::PROJECT_REPORT_ACTUALS:
					$formula[] = PHPExcel_Cell::stringFromColumnIndex(PHPExcel_Cell::columnIndexFromString($col) + $i) . ($rows + 2);
					break;
				case ProjectEACReportService::PROJECT_REPORT_ACTUALS_EAC:
					$formula[] = PHPExcel_Cell::stringFromColumnIndex(PHPExcel_Cell::columnIndexFromString($col) + $i) . ($rows + 3);
					break;
				case ProjectEACReportService::PROJECT_REPORT_DELTA_PLAN_ACTUALS_EAC:
					$formula[] = PHPExcel_Cell::stringFromColumnIndex(PHPExcel_Cell::columnIndexFromString($col) + $i) . ($rows + 4);
					break;
				case ProjectEACReportService::PROJECT_REPORT_DELTA_ASSIGNED_ACTUALS_EAC:
					$formula[] = PHPExcel_Cell::stringFromColumnIndex(PHPExcel_Cell::columnIndexFromString($col) + $i) . ($rows + 5);
					break;
			}
		}
		return '=' . implode('+', $formula);
	}
	
	private function getWorkerAgregationFormula(array $worker_rows, $exportRow, $col)
	{
		$formula = array();
		foreach($worker_rows as $row)
		{
			switch ($exportRow)
			{
				case ProjectEACReportService::PROJECT_REPORT_ASSIGNED:
					$formula[] = "'" . $this->reportType . " Report'!" . $col . ($row);
					break;
				case ProjectEACReportService::PROJECT_REPORT_ACTUALS:
					$formula[] = "'" . $this->reportType . " Report'!" . $col . ($row + 1);
					break;
				case ProjectEACReportService::PROJECT_REPORT_ACTUALS_EAC:
					$formula[] = "'" . $this->reportType . " Report'!" . $col . ($row + 2);
					break;
				case ProjectEACReportService::PROJECT_REPORT_DELTA_ASSIGNED_ACTUALS_EAC:
					$formula[] = "'" . $this->reportType . " Report'!" . $col . ($row + 3);
					break;
			}
		}
		return '=' . implode('+', $formula);
	}
	
	private function getTotalFormula(array $rows, $exportRow, $col)
	{
		$formula = array();
		foreach($rows as $row)
		{
			switch ($exportRow)
			{
				case ProjectEACReportService::PROJECT_REPORT_PLAN:
					$formula[] = $col . $row;
					break;
				case ProjectEACReportService::PROJECT_REPORT_ASSIGNED:
					$formula[] = $col . ($row + 1);
					break;
				case ProjectEACReportService::PROJECT_REPORT_ACTUALS:
					$formula[] = $col . ($row + 2);
					break;
				case ProjectEACReportService::PROJECT_REPORT_ACTUALS_EAC:
					$formula[] = $col . ($row + 3);
					break;
				case ProjectEACReportService::PROJECT_REPORT_DELTA_PLAN_ACTUALS_EAC:
					$formula[] = $col . ($row + 4);
					break;
				case ProjectEACReportService::PROJECT_REPORT_DELTA_ASSIGNED_ACTUALS_EAC:
					$formula[] = $col . ($row + 5);
					break;
			}
		}
		return '=' . implode('+', $formula);
	}
	
	/**
     * Set background color of cell
     * 
     * @param type $row Row number.
     * @param type $firstCol First column letter.
	 * @param type $lastCol Last column letter.
     * @param type $exportRow What type of row is beeing colored
     * @param type $changeTask Decide what color will be painted for each task.
     * @param type $sheet Active excel sheet.
     */
    private function setColor($row, $firstCol, $lastCol, $exportRow, $changeTask ,$sheet)
    {
        //colors
        $firstNormal = 'DBE5F1';
        $firstDelta = 'B8CCE4';
        $secondNormal = 'F2F2F2';
        $secondDelta = 'D8D8D8';

        
        if ($changeTask % 2 == 0)
        {
            if ($exportRow == ProjectEACReportService::PROJECT_REPORT_ACTUALS
					|| $exportRow == ProjectEACReportService::PROJECT_REPORT_ACTUALS_EAC
					|| $exportRow == ProjectEACReportService::PROJECT_REPORT_ASSIGNED
					|| $exportRow == ProjectEACReportService::PROJECT_REPORT_PLAN)
			{
                $sheet->getStyle($firstCol . $row . ':' . $lastCol . $row)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($firstNormal);
			}
            else
			{
				$sheet->getStyle($firstCol . $row . ':' . $lastCol . $row)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($firstDelta);
			}
        }
        else
        {
            if ($exportRow == ProjectEACReportService::PROJECT_REPORT_ACTUALS
					|| $exportRow == ProjectEACReportService::PROJECT_REPORT_ACTUALS_EAC
					|| $exportRow == ProjectEACReportService::PROJECT_REPORT_ASSIGNED
					|| $exportRow == ProjectEACReportService::PROJECT_REPORT_PLAN)
			{
                $sheet->getStyle($firstCol . $row . ':' . $lastCol . $row)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($secondNormal);
			}
            else
			{
				$sheet->getStyle($firstCol . $row . ':' . $lastCol . $row)->getFill()->setFillType(PHPExcel_Style_Fill::FILL_SOLID)->getStartColor()->setARGB($secondDelta);
			}
        }
    }
}

class ProjectReportEACServiceException extends Exception
{
	
}