<?php

/**
 * This file is part of the ATE Tool - STRAP addon
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * STRAP advisors for strap itemActivity.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class Advisor extends Object
{
    /* possible advisor roles */
    const ADVISOR_MAPM_SUB_PORTFOLIO = 'MaPMSub';
    const ADVISOR_RTC = 'RTC';
    const ADVISOR_MAPM_PORTFOLIO = 'MaPMPort';
	
	/** @var int Advisor ID*/
	private $id;

    /** @var int ID of parent Advisor */
    private $parentID;

	/** @var string Advisor name*/
	private $name;
	
	/** @var string Type of Advisor*/
	private $type;
	
	public function __construct()
	{
	}

    /**
     * Sets item ID.
     * 
     * @param int $id
     */
	public function setID($id)
	{
		if ($id > 0)
			$this->id = $id;
		else
			throw new InvalidArgumentException('Advisor ID must be integer greater than 0. Wrong param: '.$id);
	}

    /**
     * Gets item ID.
     * 
     * @return int
     */
	public function getID()
	{
		return $this->id;
	}	
	
    /**
     * Sets ID of parent node.
     * @param int $id Must be positive integer.
     */
	public function setParent_ID($id)
	{
		if ($id > 0 || is_null($id))
			$this->parentID = $id;
		else
			throw new InvalidArgumentException('Advisor parent ID must be null or integer greater than 0. Wrong param: '.$id);
	}
	
    /**
     * Gets ID of parent node.
     * @return int
     */
	public function getParent_ID()
	{
		return $this->parentID;
	}
	
    /** 
     * Sets name. 
     *  
     * @param string $name  
     */ 
    public function setName($name) 
    { 
            $this->name = $name; 
    } 

    /** 
     * Gets name. 
     *  
     * @return string 
     */ 
    public function getName() 
    { 
            return $this->name; 
    } 
	
    /** 
     * Sets type. 
     *  
     * @param string $type  
     */ 
    public function setType($type) 
    { 
		if (in_array($type, array_keys(Advisor::getTypes())))
			$this->type = $type; 
		else
			throw new InvalidArgumentException('Unknown advisor type.');
    }

    /** 
     * Gets type. 
     *  
     * @return string 
     */ 
    public function getType() 
    { 
        return $this->type; 
    } 
	
    /**
     * Returns array of available types.
     * 
     * @return array
     */
	public static function getTypes()
	{
		return array(	self::ADVISOR_MAPM_SUB_PORTFOLIO => self::ADVISOR_MAPM_SUB_PORTFOLIO,
						self::ADVISOR_MAPM_PORTFOLIO => self::ADVISOR_MAPM_PORTFOLIO,
						self::ADVISOR_RTC => self::ADVISOR_RTC
					);
	}
}