<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Interface for accessing CustomerProgramCall.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
interface ICustomerProgramCallMapper
{
    /**
	 * Saves program call.
	 * @param CustomerProgramCall $programCall
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function save(CustomerProgramCall $programCall);
    
	/**
	 * Updates program call.
	 * @param CustomerProgramCall $programCall
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function update(CustomerProgramCall $programCall);
    
	/**
	 * Deletes program call.
	 * @param int $id programCall ID.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function delete($id);
	
	/**
	 * Finds program call by ID.
	 * @param int $id programCall ID.
	 * @return	CustomerProgramCall or NULL if not found.
	 */
	public function find($id);
	
	/** 
	 * Finds all program calls.
	 * @return array Array of CustomerProgramCall or empty array.	
	 */
	public function findAll();
	
	/** 
	 * Finds program calls by customer program ID.
	 * @param int $id customer program ID.
	 * @return array Array of CustomerProgramCall or empty array.	
	 */
	public function findByProgram($id);
	
	/** 
	 * Finds program calls by customer ID.
	 * @param int $id customer ID.
	 * @return array Array of CustomerProgramCall or empty array.	
	 */
	public function findByCustomer($id);
}