<?php

/**
 * This file is part of the ATE Tool - STRAP addon
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * STRAP Item Activity represents one possible part from STRAP Item.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class ItemActivity extends Object
{
	/* Multiple and not defined customer */
	const ITEM_ACTIVITY_CUSTOMER_NOT_DEFINED = 'Not defined';
	const ITEM_ACTIVITY_CUSTOMER_MULTIPLE = 'Multiple';
	
    /* possible item activity types */
    const ITEM_ACTIVITY_TYPE_PROJECT = 'Project';
    const ITEM_ACTIVITY_TYPE_PURSUIT = 'Pursuit';
    const ITEM_ACTIVITY_TYPE_NEED_TO_PURSUE = 'NeedToPursue';
	
    /* possible item activity types - names*/
    const ITEM_ACTIVITY_TYPE_NAME_PROJECT = 'Project';
    const ITEM_ACTIVITY_TYPE_NAME_PURSUIT = 'Pursuit';
    const ITEM_ACTIVITY_TYPE_NAME_NEED_TO_PURSUE = 'Need To Pursue';
	
    /* possible item activity statuses*/
    const ITEM_ACTIVITY_STATUS_INACTIVE = 'Inactive';
    const ITEM_ACTIVITY_STATUS_PURSUING = 'Pursuing';
    const ITEM_ACTIVITY_STATUS_COMMITTED = 'Committed';
	const ITEM_ACTIVITY_STATUS_REJECTED = 'Rejected';
	const ITEM_ACTIVITY_STATUS_PLANNED = 'Planned';
	const ITEM_ACTIVITY_STATUS_EXECUTING = 'Executing';
	const ITEM_ACTIVITY_STATUS_COMPLETED = 'Completed';
	const ITEM_ACTIVITY_STATUS_STOPPED = 'Stopped';
	const ITEM_ACTIVITY_STATUS_WIN = 'Win';
	const ITEM_ACTIVITY_STATUS_MULTPLE = 'Multiple';
	const ITEM_ACTIVITY_STATUS_NOT_DEFINED = 'Not defined';
	
	/** @var int */
	private $id;
	
	/** @var int Item id */
	private $itemID;

	/** @var string ItemActivity id */
	private $customID;
	
	/** @var int Pursuit id */
	private $pursuitID;
	
	/** @var int Project id */
	private $projectID;
	
	/** @var int NeedToPursuit id */
	private $needToPursueID;
	
	/** @var int TRL*/
	private $targetTRL = 0;
	
	/** @var int TRL Year*/
	private $targetTRLYear = 0;
	
	/** @var string Name*/
	private $name;
	
	/** @var string Remarks*/
	private $remarks;
	
	/** @var string Technology roadmap domain*/
	private $technologyRoadmapDomain;
	
	/** @var string Marketing product manager*/
	private $marketingProductManager;
	
	/** @var string Portfolio manager eid*/
	private $portfolioManager;
	
	/** @var string Product line eid*/
	private $productLine;
	
	/** @var string RTC director eid*/
	private $RTCDirector;
	
	/** @var string Date of creation*/
	private $created;
	
	/** @var string Creator eid */
	private $creator;
	
	/** @var string CostShare*/
	private $costShare;
	
	/** @var string Project budget usage*/
	private $projectBudgetUsage = 1.00;
	
	/** @var string */
	private $potentialCustomer;
	
	/** @var int Active*/
	private $active = 1;
	
	/** @var string Creator eid */
	private $updatedBy;
	
	/** @var string Date of creation*/
	private $lastUpdatedDate;
	
	/** @var int ECLicence id */
	private $ECLicenceID;
	
	public function __construct()
	{
	}

    /**
     * Sets item activity ID.
     * 
     * @param int $id
     */
	public function setID($id)
	{
		if ($id > 0)
			$this->id = $id;
		else
			throw new InvalidArgumentException('STRAP Item Activity ID must be integer greater than 0. Wrong param: '.$id);
	}

    /**
     * Gets item activity ID.
     * 
     * @return int
     */
	public function getID()
	{
		return $this->id;
	}	
	
    /**
     * Sets item ID.
     * 
     * @param int $id
     */
	public function setItemID($id)
	{
		if ($id > 0)
			$this->itemID = $id;
		else
			throw new InvalidArgumentException('STRAP Item ID must be integer greater than 0. Wrong param: '.$id);
	}

    /**
     * Gets item ID.
     * 
     * @return int
     */
	public function getItemID()
	{
		return $this->itemID;
	}		
	
    /**
     * Sets custom ID.
     * 
     * @param string $id 
     */
	public function setCustomID($id)
	{
		$this->customID = $id;
	}

    /**
     * Gets custom ID.
     * 
     * @return string
     */
	public function getCustomID()
	{
		return $this->customID;
	}
	
    /**
     * Sets pursuit ID.
     * 
     * @param int $id
     */
	public function setPursuitID($id)
	{
		if (is_array($id) || is_null($id) || NumericUtils::isNonNegativeInteger($id))
			$this->pursuitID = $id;
		else
			throw new InvalidArgumentException('STRAP Pursuit ID must be integer greater than 0. Wrong param: '.$id);
	}

    /**
     * Gets pursuit ID.
     * 
     * @return int
     */
	public function getPursuitID()
	{
		return $this->pursuitID;
	}
	
    /**
     * Sets project ID.
     * 
     * @param int $id
     */
	public function setProjectID($id)
	{
		if (NumericUtils::isNonNegativeInteger($id) || is_null($id))
			$this->projectID = $id;
		else
			throw new InvalidArgumentException('STRAP project ID must be integer greater than 0. Wrong param: '.$id);
	}

    /**
     * Gets project ID.
     * 
     * @return int
     */
	public function getProjectID()
	{
		return $this->projectID;
	}
	
    /**
     * Sets Need To Pursue ID.
     * 
     * @param int $id
     */
	public function setNeedToPursueID($id)
	{
		if (NumericUtils::isNonNegativeInteger($id) || is_null($id))
			$this->needToPursueID = $id;
		else
			throw new InvalidArgumentException('STRAP Need To Pursue ID must be integer greater than 0. Wrong param: '.$id);
	}

    /**
     * Gets Need To Pursue ID.
     * 
     * @return int
     */
	public function getNeedToPursueID()
	{
		return $this->needToPursueID;
	}

    /**
     * Sets target TRL.
     * 
     * @param int $id
     */
	public function setTargetTRL($targetTRL)
	{
		$this->targetTRL = $targetTRL;
	}

    /**
     * Gets target TRL.
     * 
     * @return int
     */
	public function getTargetTRL()
	{
		return $this->targetTRL;
	}	
	
    /**
     * Sets target TRL year ID.
     * 
     * @param $id int
     */
	public function setTargetTRLYear($targetTRLYear)
	{
		$this->targetTRLYear = $targetTRLYear;
	}

    /**
     * Gets target TRL year ID.
     * 
     * @return int
     */
	public function getTargetTRLYear()
	{
		return $this->targetTRLYear;
	}
	
	/**
	 * Gets target TRL with target TRL year: e.g.  "4 (2019)"
	 * 
	 * @return string
	 */
	public function getTRLYear()
	{
		return $this->targetTRL . ' ' . (($this->targetTRLYear) ? '(' . $this->targetTRLYear . ')' : '');
	}
	
    /** 
     * Sets name. 
     *  
     * @param string $name  
     */ 
    public function setName($name) 
    { 
            $this->name = $name; 
    } 

    /** 
     * Gets name. 
     *  
     * @return string 
     */ 
    public function getName() 
    { 
            return $this->name; 
    } 
	
    /** 
     * Sets remarks. 
     *  
     * @param string $remarks  
     */ 
    public function setRemarks($remarks) 
    { 
            $this->remarks = $remarks; 
    } 

    /** 
     * Gets remakrs. 
     *  
     * @return string 
     */ 
    public function getRemarks() 
    { 
            return $this->remarks; 
    } 
	
    /** 
     * Sets TechnologyRoadmapDomain. 
     *  
     * @param string $technologyRoadmapDomain  
     */ 
    public function setTechnologyRoadmapDomain($technologyRoadmapDomain) 
    { 
            $this->technologyRoadmapDomain = $technologyRoadmapDomain; 
    } 

    /** 
     * Gets TechnologyRoadmapDomain. 
     *  
     * @return string 
     */ 
    public function getTechnologyRoadmapDomain() 
    { 
            return $this->technologyRoadmapDomain;
    } 
	
    /** 
     * Sets MarketingProductManager. 
     *  
     * @param string $marketingProductManager  
     */ 
    public function setMarketingProductManager($marketingProductManager) 
    { 
            $this->marketingProductManager = $marketingProductManager; 
    } 

    /** 
     * Gets MarketingProductManager. 
     *  
     * @return string 
     */ 
    public function getMarketingProductManager() 
    { 
            return $this->marketingProductManager; 
    } 
	
    /** 
     * Sets PortfolioManager. 
     *  
     * @param string $portfolioManager  
     */ 
    public function setPortfolioManager($portfolioManager) 
    { 
            $this->portfolioManager = $portfolioManager; 
    } 

    /** 
     * Gets PortfolioManager. 
     *  
     * @return string 
     */ 
    public function getPortfolioManager() 
    { 
            return $this->portfolioManager; 
    } 
	
    /** 
     * Sets ProductLine. 
     *  
     * @param string $productLine  
     */ 
    public function setProductLine($productLine) 
    { 
            $this->productLine = $productLine; 
    } 

    /** 
     * Gets ProductLine. 
     *  
     * @return string 
     */ 
    public function getProductLine() 
    { 
            return $this->productLine; 
    } 
	
    /** 
     * Sets RTCDirector. 
     *  
     * @param string $RTCDirector  
     */ 
    public function setRTCDirector($RTCDirector) 
    { 
            $this->RTCDirector = $RTCDirector; 
    } 

    /** 
     * Gets description. 
     *  
     * @return string 
     */ 
    public function getRTCDirector() 
    { 
            return $this->RTCDirector; 
    } 
	
	/**
     * Set creation date.
     * 
     * @param string $date Date in YYYY-MM-DD formatD.
	 * @todo Check format of $date.
     */
	public function setCreated($date)
	{
		$this->created = $date;
	}
	
    /**
     * Gets creation date.
     * 
     * @return string	Date in YYYY-MM-DD format
     */
	public function getCreated()
	{
		return $this->created;
	}
	
    /**
     * Sets Creator eid.
     * 
     * @param string $eid 
     */
	public function setCreator($eid)
	{
		$this->creator = Person::processEID($eid);
	}

    /**
     * Gets Creator eid.
     * 
     * @return string
     */
	public function getCreator()
	{
		return $this->creator;
	}
	
    /**
     * Sets costs sharing percentage between customer and supplier as float.
     * 
     * @return float
     */
	public function getCostShare()
	{
		return $this->costShare;
	}

    /**
     * Sets costs sharing percentage between customer and supplier.
	 * Must be float in range <0,1>. Percetage should be set as float.
	 * E.g.: 60% -> 0.6
	 * 
	 * Precision will be rounded to 2 decimal positions.
     * 
     * @param float $percent_as_float
     */
	public function setCostShare($percent_as_float)
	{
		if ((is_numeric($percent_as_float) && ($percent_as_float >= 0) && ($percent_as_float <= 1.0)) || is_null($percent_as_float))
			$this->costShare = $percent_as_float;
		else
			throw new InvalidArgumentException('Cost sharing must be float in range <0,1>.');
	}
	
    /**
     * Gets Project Budget Usage percentage between STRAP item activites.
     * 
     * @return float
     */
	public function getProjectBudgetUsage()
	{
		return $this->projectBudgetUsage;
	}

    /**
     * Sets Project Budget Usage percentage between STRAP item activites.
	 * Must be float in range <0,1>. Percetage should be set as float.
	 * E.g.: 60% -> 0.6
	 * 
	 * Precision will be rounded to 2 decimal positions.
     * 
     * @param float $percent_as_float
     */
	public function setProjectBudgetUsage($percent_as_float)
	{
		if (is_numeric($percent_as_float)
				&& ($percent_as_float >= 0) && ($percent_as_float <= 1.0))
			$this->projectBudgetUsage = $percent_as_float;
		else
			throw new InvalidArgumentException('Project Budget Usage must be float in range <0,1>.');
	}
	
    /** 
     * Sets potentialCustomer. 
     *  
     * @param string $potentialCustomer  
     */ 
    public function setPotentialCustomer($potentialCustomer) 
    { 
		$this->potentialCustomer = $potentialCustomer; 
    } 

    /** 
     * Gets potentialCustomer. 
     *  
     * @return string 
     */ 
    public function getPotentialCustomer() 
    { 
		return $this->potentialCustomer; 
    } 

    /**
     * Sets Active.
     * 
     * @param int $active
     */
	public function setActive($active)
	{
		$this->active = $active;
	}

    /**
     * Gets Active.
     * 
     * @return int
     */
	public function getActive()
	{
		return $this->active;
	}
	
	/**
     * Set last updated date.
     * 
     * @param string $date Date in YYYY-MM-DD formatD.
	 * @todo Check format of $date.
     */
	public function setLastUpdatedDate($date)
	{
		$this->lastUpdatedDate = $date;
	}
	
    /**
     * Gets last updated date .
     * 
     * @return string	Date in YYYY-MM-DD format
     */
	public function getLastUpdatedDate()
	{
		return $this->lastUpdatedDate;
	}
	
    /**
     * Sets UpdatedBy eid.
     * 
     * @param string $eid 
     */
	public function setUpdatedBy($eid)
	{
		$this->updatedBy = Person::processEID($eid);
	}

    /**
     * Gets UpdatedBy eid.
     * 
     * @return string
     */
	public function getUpdatedBy()
	{
		return $this->updatedBy;
	}
	
	/*
	 * Gets type of Item Activity
	 * 
	 * @return string Type of Item Activity or Null if doesnt exists.
	 */
	public function getType()
	{
		if(!is_null($this->projectID))
			return self::ITEM_ACTIVITY_TYPE_PROJECT;
		elseif(!is_null($this->pursuitID))
			return self::ITEM_ACTIVITY_TYPE_PURSUIT;
		elseif(!is_null($this->needToPursueID))
			return self::ITEM_ACTIVITY_TYPE_NEED_TO_PURSUE;
		else
			return 'Not defined';
	}
	
	/**
	 * Gets ItemActivity status.
	 * 
	 * return string Status or null if not found.
	 */
	public function getStatus()
	{
		$context = Environment::getContext();
		//get pursuits
		if(!is_null($this->pursuitID))
		{
			$allPursuits = $context->pursuitRepository->findAllPursuitsByActivityID($this->id);
			if($allPursuits)
			{
				if (count($allPursuits) == 1)
				{
					$key = key($allPursuits);
					$pursuit = $allPursuits[$key];
				}
				else
				{
					$statuses = array();
					foreach($allPursuits as $pursuit)
					{
						$statuses[] = $pursuit->Status;
					}
					$pursuit = null;
					if((count(array_unique($statuses)) == 1) && count($allPursuits) > 0)
					{
						reset($allPursuits);
						$key = key($allPursuits);
						$pursuit = $allPursuits[$key];
					}
					else
						$pursuits = 1;
				}
			}
		}
		if(is_null($this->pursuitID) && is_null($this->projectID))
			return self::ITEM_ACTIVITY_STATUS_INACTIVE;
		elseif(isset($pursuit) && ($pursuit->Status == Pursuit::PURSUIT_STATUS_PLAN || $pursuit->Status == Pursuit::PURSUIT_STATUS_PROPOSED || $pursuit->Status == Pursuit::PURSUIT_STATUS_ACTIVE))
			return self::ITEM_ACTIVITY_STATUS_PURSUING;
		elseif(isset($pursuit) && $pursuit->Status == Pursuit::PURSUIT_STATUS_CLOSED)
		{
			if(Pursuit::PURSUIT_CLOSURE_RESULT_ON_HOLD == $pursuit->closureResult || Pursuit::PURSUIT_CLOSURE_RESULT_STOPPED == $pursuit->closureResult)
				return self::ITEM_ACTIVITY_STATUS_STOPPED;
			elseif(Pursuit::PURSUIT_CLOSURE_RESULT_WIN == $pursuit->closureResult)
				return self::ITEM_ACTIVITY_STATUS_WIN;
			elseif(Pursuit::PURSUIT_CLOSURE_RESULT_LOST == $pursuit->closureResult)
				return self::ITEM_ACTIVITY_STATUS_REJECTED;
		}
		elseif(isset($pursuit) && $pursuit->Status == Pursuit::PURSUIT_STATUS_COMMITTED)
			return self::ITEM_ACTIVITY_STATUS_COMMITTED;
		elseif(isset($pursuits))
		{
			$status = $this->getMultipleStatuses();
			if(count($status) == 1)
				return $status;
			else
				return self::ITEM_ACTIVITY_STATUS_MULTPLE;
		}
		elseif(!is_null($this->projectID) && $context->projectRepository->find($this->projectID)->Phase == Project::PROJECT_PHASE_ACTIVE)
			return self::ITEM_ACTIVITY_STATUS_EXECUTING;
		elseif(!is_null($this->projectID) && $context->projectRepository->find($this->projectID)->Phase == Project::PROJECT_PHASE_PLANNED)
			return self::ITEM_ACTIVITY_STATUS_PLANNED;
		elseif(!is_null($this->projectID) && $context->projectRepository->find($this->projectID)->Phase == Project::PROJECT_PHASE_CLOSED)
			return self::ITEM_ACTIVITY_STATUS_COMPLETED;
		else 
			return self::ITEM_ACTIVITY_STATUS_NOT_DEFINED;
	}
	
	/**
	 * Gets ItemActivity status.
	 * 
	 * @throws InvalidArgumentException if Activity is not covered by more then one pursuit with different statuses.
	 * return string Status or null if not found.
	 */
	public function getMultipleStatuses()
	{
		$context = Environment::getContext();
		//get pursuits
		if(!is_null($this->pursuitID))
		{
			$allPursuits = $context->pursuitRepository->findAllPursuitsByActivityID($this->id);
			if($allPursuits)
			{
				if (count($allPursuits) == 1)
				{
					throw new InvalidArgumentException('Multiple customers are not available for Activity with ID: ' . $this->id);
				}
				else
				{
					$pursuits = array();
					$pursuitStatuses = array();
					foreach($allPursuits as $pursuit)
					{
						$pursuits[] = $pursuit;
						$pursuitStatuses[] = $pursuit->Status;
					}
					if((count(array_unique($pursuitStatuses)) == 1) && count($allPursuits) > 0)
					{
						throw new InvalidArgumentException('Multiple customers are not available for Activity with ID: ' . $this->id);
					}
					else
					{
						$statusArray = array();
						foreach ($pursuits as $pursuit)
						{
							if($pursuit->Status == Pursuit::PURSUIT_STATUS_PLAN || $pursuit->Status == Pursuit::PURSUIT_STATUS_PROPOSED || $pursuit->Status == Pursuit::PURSUIT_STATUS_ACTIVE)
								$statusArray[] = self::ITEM_ACTIVITY_STATUS_PURSUING;
							elseif($pursuit->Status == Pursuit::PURSUIT_STATUS_CLOSED)
							{
								if(Pursuit::PURSUIT_CLOSURE_RESULT_ON_HOLD == $pursuit->closureResult || Pursuit::PURSUIT_CLOSURE_RESULT_STOPPED == $pursuit->closureResult)
									$statusArray[] = self::ITEM_ACTIVITY_STATUS_STOPPED;
								elseif(Pursuit::PURSUIT_CLOSURE_RESULT_WIN == $pursuit->closureResult)
									$statusArray[] = self::ITEM_ACTIVITY_STATUS_WIN;
								elseif(Pursuit::PURSUIT_CLOSURE_RESULT_LOST == $pursuit->closureResult)
									$statusArray[] = self::ITEM_ACTIVITY_STATUS_REJECTED;
							}
						}
						return join(', ', array_unique($statusArray));
					}
				}
			}
			else
			{
				throw new InvalidArgumentException('Multiple customers are not available for Activity with ID: ' . $this->id);
			}
		}
		else
			throw new InvalidArgumentException('Multiple customers are not available for Activity with ID: ' . $this->id);
	}
	
	/**
	 * Gets ItemActivity customer.
	 * 
	 * return string Customer name or null if not found.
	 */
	public function getCustomer()
	{
		$context = Environment::getContext();
		//get pursuits
		if(!is_null($this->pursuitID))
		{
			$allPursuits = $context->pursuitRepository->findAllPursuitsByActivityID($this->id);
			if($allPursuits)
			{
				if (count($allPursuits) == 1)
				{
					$key = key($allPursuits);
					$pursuit = $allPursuits[$key];
				}
				else
				{
					$customers = array();
					foreach($allPursuits as $pursuit)
					{
						$customers[] = $pursuit->CustomerID;
					}
					$pursuit = null;
					if((count(array_unique($customers)) == 1) && count($allPursuits) > 0)
					{
						reset($allPursuits);
						$key = key($allPursuits);
						$pursuit = $allPursuits[$key];
					}
					else
						$pursuits = 1;
				}
			}
		}
		if(!is_null($this->projectID))
		{
			if($context->projectRepository->find($this->projectID)->Customer)
				return $context->customerRepository->find($context->projectRepository->find($this->projectID)->Customer)->Name;
			else
				return self::ITEM_ACTIVITY_CUSTOMER_NOT_DEFINED;
		}
		elseif(isset($pursuit))
			return $context->customerRepository->find($pursuit->CustomerID)->Name;
		elseif(isset($pursuits))
			return self::ITEM_ACTIVITY_CUSTOMER_MULTIPLE;
		elseif(!is_null($this->needToPursueID))
		{
			if($this->potentialCustomer)
				return $this->potentialCustomer;
			else
				return self::ITEM_ACTIVITY_CUSTOMER_NOT_DEFINED;
		}
		else
			return self::ITEM_ACTIVITY_CUSTOMER_NOT_DEFINED;
	}
	
	/**
	 * Gets ItemActivity customers.
	 * 
	 * @throws InvalidArgumentException if Activity is not covered by more then one pursuit with different cutomers.
	 * return string Customer name or null if not found.
	 */
	public function getMultipleCustomers()
	{
		$context = Environment::getContext();
		//get pursuits
		if(!is_null($this->pursuitID))
		{
			$allPursuits = $context->pursuitRepository->findAllPursuitsByActivityID($this->id);
			if($allPursuits)
			{
				if (count($allPursuits) == 1)
				{
					throw new InvalidArgumentException('Multiple customers are not available for Activity with ID: ' . $this->id);
				}
				else
				{
					$customers = array();
					foreach($allPursuits as $pursuit)
					{
						$customers[] = $pursuit->CustomerID;
					}
					if((count(array_unique($customers)) == 1) && count($allPursuits) > 0)
					{
						throw new InvalidArgumentException('Multiple customers are not available for Activity with ID: ' . $this->id);
					}
					else
					{
						$customerArray = array();
						foreach ($customers as $customerID)
						{
							$customerArray[] = $context->customerRepository->find($customerID)->Name;
						}
						return join(', ', array_unique($customerArray));
					}
				}
			}
			else
			{
				throw new InvalidArgumentException('Multiple customers are not available for Activity with ID: ' . $this->id);
			}
		}
		else
			throw new InvalidArgumentException('Multiple customers are not available for Activity with ID: ' . $this->id);
	}
	
    /**
     * Returns array of available types.
     * 
     * @return array
     */
	public static function getTypes()
	{
		return array(	self::ITEM_ACTIVITY_TYPE_NEED_TO_PURSUE => self::ITEM_ACTIVITY_TYPE_NAME_NEED_TO_PURSUE,
						self::ITEM_ACTIVITY_TYPE_PROJECT => self::ITEM_ACTIVITY_TYPE_NAME_PROJECT,
						self::ITEM_ACTIVITY_TYPE_PURSUIT => self::ITEM_ACTIVITY_TYPE_NAME_PURSUIT
					);
	}
	
	/**
     * Sets ECLicence id.
     * 
     * @param int $id
     */
	public function setECLicenceID($id)
	{
		if ($id > 0)
			$this->ECLicenceID = $id;
		else
			throw new InvalidArgumentException('ECLicence ID must be integer greater than 0. Wrong param: ' . $id);
	}

    /**
     * Gets ECLicence id.
     * 
     * @return int
     */
	public function getECLicenceID()
	{
		return $this->ECLicenceID;
	}
}