<?php

/**
 * This file is part of the ATE Tool - STRAP addon
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * STRAP ItemActivityYearlyBudget represents several types of budget for STRAP ItemActivity.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class ItemActivityYearlyBudget extends Object
{
	/** @var int */
	private $id;

	/** @var int Year */
	private $year;

	/** @var int Labour costs */
	private $laborCosts = 0;
	
	/** @var int Labour costs */
	private $subcontractingCosts = 0;
	
	/** @var int Labour costs */
	private $travelCosts = 0;
	
	/** @var int Labour costs */
	private $materialCosts = 0;
	
	public function __construct()
	{
	}

    /**
     * Sets item ID.
     * 
     * @param int $id
     */
	public function setID($id)
	{
		if ($id > 0)
			$this->id = $id;
		else
			throw new InvalidArgumentException('STRAP ItemActivity ID must be integer greater than 0. Wrong param: '.$id);
	}

    /**
     * Gets item ID.
     * 
     * @return int
     */
	public function getID()
	{
		return $this->id;
	}	
	
    /**
     * Sets year.
     * @param string $year Year in YYYY format.
     */
	public function setYear($year)
	{
		$this->year = $year;
	}

    /**
     * Gets year.
     * @return string
     */
	public function getYear()
	{
		return $this->year;
	}
	
    /**
     * Sets Labour Costs.
     * @param float $value Must be positive integer or NULL.
     */
    public function setLaborCosts($value)
    {
		if (NumericUtils::isNonNegativeInteger($value))
			$this->laborCosts = 0.0 + $value; // to float
		else if (!$value && !($value < 0))
			$this->laborCosts = 0;
		else
			throw new InvalidArgumentException('Value must be positive integer number.');
    }
    
    /**
     * Gets Labour Costs.
     * @return float
     */
    public function getLaborCosts()
    {
        return $this->laborCosts;
    }
	
    /**
     * Sets subcontracting costs.
     * @param float $value Must be positive integer or NULL.
     */
    public function setSubcontractingCosts($value)
    {
		if (NumericUtils::isNonNegativeInteger($value))
			$this->subcontractingCosts = 0.0 + $value; // to float
		else if (!$value && !($value < 0))
			$this->subcontractingCosts = 0;
		else
			throw new InvalidArgumentException('Value must be positive integer number.');
    }
    
    /**
     * Gets subcontracting costs.
     * @return float
     */
    public function getSubcontractingCosts()
    {
        return $this->subcontractingCosts;
    }
	
    /**
     * Sets Travel Costs.
     * @param float $value Must be positive integer or NULL.
     */
    public function setTravelCosts($value)
    {
		if (NumericUtils::isNonNegativeInteger($value))
			$this->travelCosts = 0.0 + $value; // to float
		else if (!$value && !($value < 0))
			$this->travelCosts = 0;
		else
			throw new InvalidArgumentException('Value must be positive integer number.');
    }
    
    /**
     * Gets Travel Costs.
     * @return float
     */
    public function getTravelCosts()
    {
        return $this->travelCosts;
    }
	
    /**
     * Sets Material Costs.
     * @param float $value Must be positive integer or NULL.
     */
    public function setMaterialCosts($value)
    {
		if (NumericUtils::isNonNegativeInteger($value))
			$this->materialCosts = 0.0 + $value; // to float
		else if (!$value && !($value < 0))
			$this->materialCosts = 0;
		else
			throw new InvalidArgumentException('Value must be positive integer number.');
    }
    
    /**
     * Gets Material Costs.
     * @return float
     */
    public function getMaterialCosts()
    {
        return $this->materialCosts;
    }
}