<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * PursuitAction object class.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class PursuitAction extends Object
{
    /* possible pursuit action statuses */
    const PURSUIT_ACTION_STATUS_NOT_STARTED = 'Not Started';
    const PURSUIT_ACTION_STATUS_IN_PROGRESS = 'In Progress';
	const PURSUIT_ACTION_STATUS_COMPLETED = 'Completed';
	const PURSUIT_ACTION_STATUS_WAITONG_ON_SOMEONE = 'Waiting on someone';
	const PURSUIT_ACTION_STATUS_DEFERRED = 'Deferred';
	
	/** @var int ID of PursuitAction */
	private $id;
	
	/** @var int ID of pursuit */
	private $pursuitID;
	
	/** @var string Name of pursuit action */
	private $action;
	
	/** @var string EID of pursuit action owner */
	private $owner;
	
	/** @var datetime Deadline date */
	private $deadline;
	
	/** @var string Current status of pursuit action */
	private $status;
	
	/** @var string Remarks of pursuit plan */
	private $remarks;
	
    public function  __construct() 
    {
    }
	
	/**
     * Sets ID of pursuit action.
     * @param int $id Must be positive integer.
	 * @throws InvalidArgumentException
     */
	public function setID($id)
	{
		if ($id > 0)
			$this->id = $id;
		else
			throw new InvalidArgumentException('PursuitAction ID must be integer greater than 0. Wrong param: ' . $id);
	}
	
    /**
     * Gets ID of pursuit action.
     * @return int
     */
	public function getID()
	{
		return $this->id;
	}
	
	/**
     * Sets ID of pursuit.
     * @param int $id Must be positive integer.
	 * @throws InvalidArgumentException
     */
	public function setPursuitID($id)
	{
		if ($id > 0)
			$this->pursuitID = $id;
		else
			throw new InvalidArgumentException('Pursuit ID must be integer greater than 0. Wrong param: ' . $id);
	}
	
    /**
     * Gets ID of pursuit.
     * @return int
     */
	public function getPursuitID()
	{
		return $this->pursuitID;
	}
	
	/**
     * Sets name of pursuit action.
	 * @param string Pursuit action name.
     */
	public function setAction($action)
	{
	    $this->action = $action;
	}
    
    /**
     * Gets name of pursuit action.
     * @return string
     */
	public function getAction()
	{
		return $this->action;
	}
	
	/**
	 * Sets pursuit action owner.
	 * @param string $eid
	 */
	public function setOwner($eid)
	{
		$this->owner = Person::processEID($eid);
	}
	
	/**
	 * Gets pursuit action owner.
	 * @return string
	 */
	public function getOwner()
	{
		return $this->owner;
	}
	
	/**
	 * Sets deadline date.
	 * @param string $datetime
	 */
	public function setDeadline($datetime)
	{
		$this->deadline = $datetime;
	}
	
	/**
	 * Gets deadline date.
	 * @return string
	 */
	public function getDeadline()
	{
		return $this->deadline;
	}
	
	/**
     * Sets status of pursuit action.
	 * @param string Status of pursuit action.
	 * @throws InvalidArgumentException
     */
	public function setStatus($status)
	{
	    if (in_array($status, self::getStatuses()) || ($status === NULL))
			$this->status = $status;
		else
			throw new InvalidArgumentException('Pursuit action status must be in set of pre defined statuses. Wrong param: ' . $status);
	}
    
    /**
     * Gets status of pursuit action.
     * @return string
     */
	public function getStatus()
	{
		return $this->status;
	}
	
    /** 
     * Sets remarks. 
     *  
     * @param string $remarks  
     */ 
    public function setRemarks($remarks) 
    { 
            $this->remarks = $remarks; 
    } 

    /** 
     * Gets remakrs. 
     *  
     * @return string 
     */ 
    public function getRemarks() 
    { 
            return $this->remarks; 
    }
	
    /**
     * Returns array of available statuses.
     * 
     * @return array
     */
	public static function getStatuses()
	{
		return array(	self::PURSUIT_ACTION_STATUS_NOT_STARTED => self::PURSUIT_ACTION_STATUS_NOT_STARTED,
						self::PURSUIT_ACTION_STATUS_IN_PROGRESS => self::PURSUIT_ACTION_STATUS_IN_PROGRESS,
						self::PURSUIT_ACTION_STATUS_COMPLETED => self::PURSUIT_ACTION_STATUS_COMPLETED,
						self::PURSUIT_ACTION_STATUS_WAITONG_ON_SOMEONE => self::PURSUIT_ACTION_STATUS_WAITONG_ON_SOMEONE,
						self::PURSUIT_ACTION_STATUS_DEFERRED => self::PURSUIT_ACTION_STATUS_DEFERRED
					);
	}
}
