<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package presenters
 */

/**
 * PursuitFilter defines structure to filter Pursuits.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class PursuitFilter extends Object
{
	/** @var string Name of pursuit */
	private $name;
	/** @var array Selected statuses */
	private $status;
	/** @var date Ends after this date */
	private $proposalSubmissionAfter;
	/** @var date Ends before this date */
	private $proposalSubmissionBefore;
	/** @var string EID of pursuit manager */
	private $pursuitManager;
	/** @var string EID of pursuit PI */
	private $pursuitPI;
	/** @var array Selected thresholds */
	private $probabilityOfWin;
	/** @var int Customer ID */
	private $customer;
	/** @var int Customer program ID */
	private $programID;
	/** @var int Customer program call ID */
	private $callID;
	
	/**
     * Sets name of pursuit.
	 * @param string Name of pursuit.
     */
	public function setName($name)
	{
	    $this->name = $name;
	}
    
    /**
     * Gets name of pursuit.
     * @return string
     */
	public function getName()
	{
		return $this->name;
	}
	
	/**
     * Sets status of pursuit.
	 * @param array Selected statuses.
	 * @throws InvalidArgumentException
     */
	public function setStatus($statuses)
	{
		foreach ($statuses as $status)
	        if (!in_array($status, Pursuit::getStatuses()) && ($status != ''))
			    throw new InvalidArgumentException('Pursuit status must be in set of pre defined statuses. Wrong param: ' . $status);
			
		$this->status = $statuses;
	}
    
    /**
     * Gets statuses of pursuit.
     * @return array
     */
	public function getStatus()
	{
		return $this->status;
	}
	
	/**
	 * Sets proposal submission date border.
	 * @param string $date
	 */
	public function setProposalSubmissionAfter($date)
	{
		$this->proposalSubmissionAfter = $date;
	}
	
	/**
	 * Gets proposal submission date border.
	 * @return string
	 */
	public function getProposalSubmissionAfter()
	{
		return $this->proposalSubmissionAfter;
	}
	
	/**
	 * Sets proposal submission date border.
	 * @param string $date
	 */
	public function setProposalSubmissionBefore($date)
	{
		$this->proposalSubmissionBefore = $date;
	}
	
	/**
	 * Gets proposal submission date border.
	 * @return string
	 */
	public function getProposalSubmissionBefore()
	{
		return $this->proposalSubmissionBefore;
	}
	
	/**
	 * Sets pursuit manager.
	 * @param string $eid
	 */
	public function setPursuitManager($eid)
	{
		$this->pursuitManager = Person::processEID($eid);
	}
	
	/**
	 * Gets pursuit manager.
	 * @return string
	 */
	public function getPursuitManager()
	{
		return $this->pursuitManager;
	}
	
	/**
	 * Sets pursuit PI.
	 * @param string $eid
	 */
	public function setPursuitPI($eid)
	{
		$this->pursuitPI = Person::processEID($eid);
	}
	
	/**
	 * Gets pursuit PI.
	 * @return string
	 */
	public function getPursuitPI()
	{
		return $this->pursuitPI;
	}
	
	/**
     * Sets threshold of probability of win.
	 * @param array Selected thresholds.
	 * @throws InvalidArgumentException
     */
	public function setProbabilityOfWin($thresholds)
	{
		foreach ($thresholds as $threshold)
	        if (!in_array($threshold, Pursuit::getProbabilityOfWinThresholds()) && ($threshold != ''))
			    throw new InvalidArgumentException('Threshold of probability of win of Pursuit must be in set of pre defined thresholds. Wrong param: ' . $threshold);
			
		$this->probabilityOfWin = $thresholds;
	}
    
    /**
     * Gets thresholds of probability of win.
     * @return array
     */
	public function getProbabilityOfWin()
	{
		return $this->probabilityOfWin;
	}
	
	/**
	 * Sets customer id.
	 * @param int $id
	 */
	public function setCustomer($id)
	{
		$this->customer = $id;
	}
	
	/**
	 * Gets customer id.
	 * @return int
	 */
	public function getCustomer()
	{
		return $this->customer;
	}
	
	/**
	 * Sets customer program id.
	 * @param int $id
	 */
	public function setProgramID($id)
	{
		$this->programID = $id;
	}
	
	/**
	 * Gets customer program id.
	 * @return int
	 */
	public function getProgramID()
	{
		return $this->programID;
	}
	
	/**
	 * Sets customer program call id.
	 * @param int $id
	 */
	public function setCallID($id)
	{
		$this->callID = $id;
	}
	
	/**
	 * Gets customer program call id.
	 * @return int
	 */
	public function getCallID()
	{
		return $this->callID;
	}
}
