<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Pursuit repository access class.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class PursuitRepository extends Object
{
	/** @var IPursuitMapper */
 	private $mapper;
	
	/** @var MultiLogger */
	private $multilog;

    public function __construct(IPursuitMapper $mapper)
    {
        $this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'Pursuit-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "STRAP Pursuit"));
    }
    
	/**
	 * Saves pursuit.
	 * @param Pursuit $pursuit
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function save(Pursuit $pursuit)
	{
		if($this->mapper->save($pursuit))
		{
			$this->multilog->log('Strap Pursuit with ID: ' . $pursuit->PursuitID . ' and Name: ' . $pursuit->Name . ' saved.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Updates pursuit.
	 * @param Pursuit $pursuit
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function update(Pursuit $pursuit)
	{
		$oldObject = $this->find($pursuit->PursuitID);
		if($this->mapper->update($pursuit))
		{
			$this->multilog->log('Strap Pursuit with ID: ' . $pursuit->PursuitID . ' and Name: ' . $pursuit->Name . ' updated. ' . $this->multilog->differences($oldObject, $pursuit), ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Deletes pursuit.
	 * @param int $id pursuit ID.
	 * @return boolean TRUE if operation ended with success, FALSE otherwise.
	 */
	public function delete($id)
	{
		if($this->mapper->delete($id))
		{
			$this->multilog->log('Strap Pursuit with ID: ' . $id . ' deleted.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
    
	/**
	 * Finds pursuit by ID.
	 * @param int $id pursuit ID.
	 * @return	Pursuit or NULL if not found.
	 */
	public function find($id)
	{
		return $this->mapper->find($id);
	}
	
	/**
	 * Finds all pursuits.
	 * @return array Array of Pursuit or empty array.
	 */
	public function findAll()
	{
		return $this->mapper->findAll();
	}
	
	/**
	 * Finds pursuits according to filter.
	 * @param PursuitFilter $filter
	 * @return array Array of Pursuit or empty array.
	 */
	public function findByFilter(PursuitFilter $filter)
	{
		return $this->mapper->findByFilter($filter);
	}
	
	/**
	 * Gets all Pursuits for given STRAP item activity.
	 * 
	 * @param int $activityID
	 * @return array Array of Pursuits where Pursuit ID is key or null if not found.
	 */
	public function findAllPursuitsByActivityID($activityID)
	{
		return $this->mapper->findAllPursuitsByActivityID($activityID);
	}
	
	/**
	 * Gets all Pursuits for given call.
	 * 
	 * @param int $callID
	 * @return array Array of Pursuits where Pursuit ID is key or null if not found.
	 */
	public function findAllPursuitsByCallID($callID)
	{
		return $this->mapper->findAllPursuitsByCallID($callID);
	}
}
