<?php

/**
 * This file is part of the ATE Tool - STRAP addon
 *
 * Copyright (c) 2009, 2013 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Repository of pursuit RED team.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class RedTeamRepository extends Object
{
	/** @var IRedTeamMapper */
    private $mapper;

	/** @var MultiLogger */
	private $multilog;
	
    public function __construct(IRedTeamMapper $mapper)
    {
        $this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'REDTeam-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "Pursuit RED team"));
    }
    
	/**
	 * Saves new RedTeam.
	 * 
	 * @param RedTeam $redTeam
	 * @return boolean
	 */
	public function save(RedTeam $redTeam)
	{
		if($this->mapper->save($redTeam))
		{
			$this->multilog->log('RedTeam member with EID: ' . $redTeam->ID . ' for pursuit: ' . $redTeam->PursuitID . ' saved.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Updates RedTeam.
	 * 
	 * @param RedTeam $redTeam
	 * @return boolean
	 */
	public function update(RedTeam $redTeam)
	{
		$oldObject = $this->find($redTeam->ID, $redTeam->PursuitID);
		if($this->mapper->update($redTeam))
		{
			$this->multilog->log('RedTeam member with EID: ' . $redTeam->ID . ' for pursuit: ' . $redTeam->PursuitID . ' updated. ' . $this->multilog->differences($oldObject, $redTeam), ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Deletes existing RedTeam member.
	 * 
	 * @param string $eid PersonEID
	 * @param int $pursuit Pursuit ID
	 * @return boolean
	 */
	public function delete($eid, $pursuitID)
	{
		if($this->mapper->delete($eid, $pursuitID))
		{
			$this->multilog->log('RedTeam member with EID: ' . $eid . ' for pursuit: ' . $pursuitID . ' deleted. ', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Gets redteam memeber by eid and purusit ID.
	 * 
	 * @param string $eid PersonEID
	 * @param int $pursuit Pursuit ID
	 * @return RedTeam or null if not found
	 */
	public function find($eid, $pursuitID)
	{
		return $this->mapper->find($eid, $pursuitID);
	}
	
	/**
	 * Gets RedTeam by purusit ID.
	 * 
	 * @param int $pursuitID Pursuit ID
	 * @return array Of RedTeam or empty array if not found
	 */
	public function findByPursuitID($pursuitID)
	{
		return $this->mapper->findByPursuitID($pursuitID);
	}
}