<?php

/**
 * This file is part of the ATE Tool - STRAP addon
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Class for accessing STRAP TechnologyRoadmapDomain from database using dibi.
 *
 * @author Jiri Pirochta <jiri.pirochta@honeywell.com>
 */
class TechnologyRoadmapDomainDibiMapper extends DibiMapper implements ITechnologyRoadmapDomainMapper
{
	/**
	 * Creates TechnologyRoadmapDomain instance from DibiRow.
	 * 
	 * @param DibiRow $row
	 * @return	TechnologyRoadmapDomain 
	 */
    private function load(DibiRow $row)
    {
		$technologyRoadmapDomain = new TechnologyRoadmapDomain();
		$technologyRoadmapDomain->Domain = $row->Domain;
		return $technologyRoadmapDomain;
    }
	
	/**
	 * Saves new STRAP TechnologyRoadmapDomain.
	 * 
	 * @param TechnologyRoadmapDomain $technologyRoadmapDomain
	 * @return boolean
	 */
	public function save(TechnologyRoadmapDomain $technologyRoadmapDomain)
	{
        try
        {
            $array = array();
			$array['Domain'] = $technologyRoadmapDomain->Domain;
            $this->conn->insert('TechnologyRoadmapDomain', $array)->execute();
			return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Insert new STRAP TechnologyRoadmapDomain with domain: " . $technologyRoadmapDomain->Domain . " failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Updates existing STRAP TechnologyRoadmapDomain.
	 * 
	 * @param TechnologyRoadmapDomain $technologyRoadmapDomain
	 * @return boolean
	 */
	public function update(TechnologyRoadmapDomain $technologyRoadmapDomain)
	{
        try
        {
            $array = array();
			$array['Domain'] = $technologyRoadmapDomain->Domain;
            $this->conn->update('TechnologyRoadmapDomain', $array)->where('[Domain] = %s', $technologyRoadmapDomain->Domain)->execute();
			return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Update of STRAP TechnologyRoadmapDomain with domain: " . $technologyRoadmapDomain->Domain . " failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}
	
	/**
	 * Deletes existing STRAP TechnologyRoadmapDomain.
	 * 
	 * @param int $domain
	 * @return boolean
	 */
	public function delete($domain)
	{
		try
		{
		    $result = $this->conn->delete('TechnologyRoadmapDomain')->where('[Domain] = %s', $domain)->execute();
			if ($result)
				Debugger::log("Strap TechnologyRoadmapDomain with domain = " . $domain . " was deleted.", Debugger::INFO);
			else
				Debugger::log("Deleting of STRAP TechnologyRoadmapDomain with domain = " . $domain . " failed.", Debugger::ERROR);
			return $result;
		}
        catch (DibiException $e)
        {
            Debugger::log("Deleting of STRAP TechnologyRoadmapDomain with domain = " . $domain . " failed: ".$e->getMessage(), Debugger::ERROR);
            return false;
        }
	}

	/**
	 * Gets STRAP TechnologyRoadmapDomain.
	 * 
	 * @param int $domain
	 * @return TechnologyRoadmapDomain or null if not found
	 */
	public function find($domain)
	{
		$technologyRoadmapDomain_db_result = $this->conn->select('*')->from('TechnologyRoadmapDomain')->where('[Domain] = %s', $domain)->fetch();
		if ($technologyRoadmapDomain_db_result !== FALSE)
			return $this->load($technologyRoadmapDomain_db_result);
		else
			return NULL;
	}

	/**
	 * Gets all STRAP TechnologyRoadmapDomain.
	 * 
	 * @return array Array of TechnologyRoadmapDomain or null if not found.
	 */
	public function findAll()
	{
        $technologyRoadmapDomain_array = array();
		$technologyRoadmapDomain_db_result = $this->conn->select('*')->from('TechnologyRoadmapDomain')->fetchAll();        
		if ($technologyRoadmapDomain_db_result !== FALSE)			
		{
			foreach ($technologyRoadmapDomain_db_result as $row)
				$technologyRoadmapDomain_array[] = $this->load($row);
			return $technologyRoadmapDomain_array;
		}
		else
			return null;
	}
}
