<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2015 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Mapper for accessing statistics from database using dibi.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class StatisticsDibiMapper extends DibiMapper implements IStatisticsMapper
{
    /**
     * Creates Statistics instance from DibiRow.
     * 
     * @param DibiRow $row
     * @return Statistics
     */
    private function load(DibiRow $row)
    {
        $stat = new Statistics();
        $stat->Id = $row->Id;
        $stat->Action = $row->Action;
        $stat->Detail = $row->Detail;
        $stat->Counter = $row->Counter;
        $stat->User = $row->User;
        
        return $stat;
    }

    /**
	 * Saves record
	 *
	 * @param Statistics $stat
	 * @return boolean True on success, false otherwise
	 */
	public function save(Statistics $stat)
	{
        $array = array();
		$array['Action'] = $stat->Action;
		$array['Detail'] = $stat->Detail;
		$array['Counter'] = $stat->Counter;
		$array['User'] = $stat->User;

		try
        {
			$db_result = $this->conn
                ->query('
                	INSERT INTO [Statistics] %v
                ', $array);

            $stat->Id = $this->conn->getInsertId();
            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Insertion of statistics failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}

	/**
	 * Updates record
	 *
	 * @param Statistics $stat
	 * @return boolean True on success, false otherwise
	 */
	public function update(Statistics $stat)
	{
        $array = array();
		$array['Action'] = $stat->Action;
		$array['Detail'] = $stat->Detail;
		$array['Counter'] = $stat->Counter;
		$array['User'] = $stat->User;

		try
        {
			$db_result = $this->conn
                ->query('
                	UPDATE [Statistics]
                	SET %a
                	WHERE [Id] = %i
                ', $array, $stat->Id);

            return true;
        }
        catch (DibiException $e)
        {
            Debugger::log("Update of statistics failed: " . $e->getMessage(), Debugger::ERROR);
            return false;
        }
	}

	/**
	 * Deletes record
	 * 
	 * @throws InvalidStateException
	 */
	public function delete()
	{
		throw new InvalidStateException('Delete method is not implemented yet.');
	}
	
	/**
	 * Finds record
	 * 
	 * @throws InvalidStateException
	 */
	public function find()
	{
		throw new InvalidStateException('Find method is not implemented yet.');
	}
	
	/**
	 * Finds all records
	 *
	 * @throws InvalidStateException
	 */
	public function findAll()
	{
		throw new InvalidStateException('FindAll method is not implemented yet.');
	}

	/**
	 * Finds record by content
	 *
	 * @param int $action
	 * @param string $detail
	 * @param string $user
	 * @return Statistict Or null if not found
	 */
	public function findByContent($action, $detail, $user)
	{
		$db_result = $this->conn
            ->query('
            	SELECT *
            	FROM [Statistics]
            	WHERE [Action] = %i AND
            	      [Detail] = %s AND
            	      [User] = %s
            ', $action, $detail, $user)
            ->fetch();

        if ($db_result === false)
        	return null;

        return $this->load($db_result);
	}
}
