<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2011 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models
 */

/**
 * Repository of task year plans.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 * @author Jiri Zajic <jiri.zajic@honeywell.com>
 */
class TaskPlanRepository extends Object
{
    /** @var ITaskPlanMapper */
 	private $mapper;
	/** @var MultiLogger */
	private $multilog;

    public function __construct(ITaskPlanMapper $mapper)
    {
        $this->mapper = $mapper;
		$this->multilog = new MultiLogger();
		$batch_id = 'TaskPlan-'.time().'-'.rand(1, 9);
		$this->multilog->add(Environment::getContext()->createLogger($batch_id, "Task Plan"));
    }	

	/**
	 * Saves task year plan.
	 * 
	 * @param TaskYearPlan $plan
	 */
	public function save(TaskYearPlan $plan)
	{
		if($this->mapper->save($plan))
		{
			$this->multilog->log('Task Plan with ID: ' . $plan->ID . ' and year: ' . $plan->Year . ' saved.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Updates task year plan.
	 * 
	 * @param TaskYearPlan $plan
	 */
	public function update(TaskYearPlan $plan)
	{
		$oldObject = $this->find($plan->ID);
		if($this->mapper->update($plan))
		{
			$this->multilog->log('Task Plan with ID: ' . $plan->ID . ' and year: ' . $plan->Year . ' updated. ' . $this->multilog->differences($oldObject, $plan), ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Deletes task year plan by ID.
	 * 
	 * @param int $plan_id
	 * @return boolean
	 */
	public function delete($plan_id)
	{
		if($this->mapper->delete($plan_id))
		{
			$this->multilog->log('Task Plan with ID: ' . $plan_id . ' deleted.', ILogger::INFO);
			return true;
		}
		else
		{
			return false;
		}
	}

	/**
     * Returns Task Plan by ID.
     * 
     * @param int $id Task Plan ID.
     * @return TaskPlan Task Plan or NULL if not found.
     */
    public function find($id)
    {
        return $this->mapper->find($id);
    }
    
	/**
	 * Returns all task year plans by task id.
	 * 
	 * @param int $id Task ID.
	 * @return	array Returns associative array of year plans (key is year), value TaskYearPlan.
	 */
	public function findTaskPlans($id)
	{
		return $this->mapper->findTaskPlans($id);
	}
	
	/**
	 * Returns all task years by task id.
	 * 
	 * @param int $id Task ID.
	 * @return	array Returns year of years.
	 */
	public function findTaskYears($id)
	{
		return $this->mapper->findTaskYears($id);
	}

	/**
	 * Returns task year plan by task id and year.
	 * 
	 * @param int $id Task ID.
	 * @param int $year year.
	 * @return	TaskYearPlan
	 */
	public function findTaskYearPlan($id, $year)
	{
		return $this->mapper->findTaskYearPlan($id, $year);
	}
}
