<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2012 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models\Import
 */

/**
 * Class for testing file by template defined in file by regular expressions.
 *
 * @author Martin Kotlar <martin.kotlar@honeywell.com>
 */
class RegexHeaderFileTester
{
	
	/** @var array of regular expressions for header testing */
    private $header_template = NULL;

	/** @var int last parsed timestamp in successfully tested input file */
    private $last_timestamp = '';
	
	/**
	 * Reads template into inner structure.
	 * @throws IOException template file cannot be opened.
	 */
	public function __construct($template_path)
	{
		$templatehandle = fopen($template_path, 'r');
		if ($templatehandle === false) 
			throw new IOException('Cannot read template in ' . $template_path);
		$this->header_template = array();
		while (!feof($templatehandle))
		{
			$line = trim(fgets($templatehandle));
			if ($line != "")
				$this->header_template[] = $line;
			else
				throw new IOException('Invalid template on line '.(count($this->header_template) + 1).' detected. Empty lines are not allowed in template file ' . $template_path);
		}
		fclose($templatehandle);
	}

	/**
	 * Test given file resource for header validity based on 
	 * defined regular expressions in source template.
	 * 
	 * Throws IOException in case of invalid input - unexpected
	 * header of file.
	 * 
	 * @param resource $file_res file to test
	 * @return boolean
	 * @throws IOException if file is not corresponding to template.
	 */
	public function testResource($filehandle)
	{
		if (!is_resource($filehandle))
		{
			throw new IOException('Given handle is not file resource.');
			return false;
		}
		if (is_null($this->header_template))
		{
			throw new IOException('Template not set up.');
			return false;
		}
		fseek($filehandle, 0);
		$contains_timestamp = FALSE;
		$this->last_timestamp = '';
		foreach ($this->header_template as $idx => $pattern)
		{
			$line = fgets($filehandle);
			if (!preg_match($pattern, $line, $match))
			{
				throw new IOException('File has an incorrect header on line ' . ($idx + 1) . '.');
				return false;
			}
			if (!$contains_timestamp && (count($match) > 1)) // probably contains time stamp
			{
				$this->last_timestamp = $match[1];
				$contains_timestamp = TRUE;
			}
		}
		return true;
	}

	/**
	 * Returns string with timestamp found in the input file.
	 * @return string or NULL if there was no successful test prior to this call.
	 */
	public function getLastFileTimestamp()
	{
		return $this->last_timestamp;
	}
	
}


