<?php

/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2014 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package models\Import
 */

/**
 * iASAP access class for Actual hours
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class ActualsReaderPO extends IASAPWrapper
{
	/** @var array Array of PO numbers used as mapping keys */
	private $po_numbers;
	
	/**
	 * @param array $po_numbers Array of PO numbers used as mapping keys
	 * @param ILogger $logger
	 */
	public function __construct($po_numbers, ILogger $logger = NULL)
	{
		parent::__construct($logger);
		
		$this->po_numbers = $po_numbers;
	}
	
	/**
	 * Gets value of po_numbers
	 * @return array
	 */
	public function getPo_numbers()
	{
		return $this->po_numbers;
	}
	
	/**
	 * Reads actual hours from iASAP.
	 * Quantity of readed data is determined by start and end date.
	 * @param DateTime $start_date
	 * @param DateTime $end_date
	 * @return array Contains for each PO number set of people.
	 *     And for each person it contains sum of actual hours in defined fiscal months.
	 *     $table[PO_number][Person_EID][Fiscal_month<0..11>] = Sum_of_actual_hours
	 *     For empty input set of PO numbers or in case of error returns empty array.
	 */
	public function read(DateTime $start_date, DateTime $end_date)
	{
		$table = array();
		
		foreach ($this->po_numbers as $po)
		{
			$db_result = $this->conn
			    ->query('
			    	SELECT [ps_employee_charge.eId] AS [EID], [ps_employee_charge.date] AS [Date],
			    	       [ps_employee_charge.hours] AS [Hours]
			    	FROM [ps_activity], [ps_employee_charge]
			    	WHERE [ps_activity.l1Description] LIKE %~like~ AND
			    	      [ps_activity.activityId] = [ps_employee_charge.fkActivityId] AND
			    	      %d <= [ps_employee_charge.date] AND [ps_employee_charge.date] <= %d
			    	ORDER BY [ps_employee_charge.eId] ASC
			    ', $po, $start_date->format('Y-m-d'), $end_date->format('Y-m-d'))
			    ->fetchAll();

			if ($db_result)
			{
				foreach ($db_result as $row)
				{
					$eid = $this->repairEID($row->EID);
                    $charge_date = new DateTime($row->Date);
					$fiscal_month = FiscalCalendar::getFiscalMonthByTimestamp((int) $charge_date->format('U'));
					if (isset($table[$po][$eid][$fiscal_month->Index]))
					{
						$table[$po][$eid][$fiscal_month->Index] += $row->Hours;
					}
					else
					    $table[$po][$eid][$fiscal_month->Index] = $row->Hours;
				}
			}
		}
		
		return $table;
	}
}
