<?php
/**
 * This file is part of the ATE Tool
 *
 * Copyright (c) 2009, 2015 Honeywell International spol. s r.o.
 *
 * For the full copyright and license information, please view
 * the file license.txt that was distributed with this source code.
 * @package utils
 */

/**
 * Provides logging of statistics actions into database.
 *
 * @author Vojtech Obrusnik <vojtech.obrusnik@honeywell.com>
 */
class StatisticsLogger implements IStatistics
{
	/** @var StatisticsRepository */
	private $repository;
	/** @var string Id of active user */
	private $user;

	/**
	 * Creates new statistics logger
	 *
	 * @param StatisticsRepository $repository
	 */    
	public function __construct(StatisticsRepository $repository)
    {
		$this->repository = $repository;
		$this->user = '' . Environment::getUser()->getId();
	}

	/**
     * Validates action range
     *
     * @param int $action Should be in range of IStatistics::*
     * @return boolean True on success, false otherwise
     */
	public function validateAction($action)
	{
		$all_actions = array(
			IStatistics::CLICK,
			IStatistics::VISIT
		);

		return in_array($action, $all_actions);
	}

	/**
	 * Logs statistics
	 * 
	 * @param string $detail
	 * @param int $action Should be in range IStatistics::*
	 * @return boolean True on success, false otherwise
	 */
    public function log($detail, $action = NULL)
    {
		if (!$this->validateAction($action))
			throw new InvalidArgumentException('Action must be in range of IStatistics::*. Wrong param: ' . $action);

		$stat = new Statistics();
		$stat->Action = $action;
		$stat->Detail = $detail;
		$stat->Counter = 1;
		$stat->User = $this->user;
		
		return $this->repository->add($stat);
    }

}