

/**
 * Ajax handle for customer selector
 * @param object element Should be reference to currently changed combo box of customer selector
 */
function handleCustomerSelector(element)
{
	// Kill function, because cannot find dependencies without input
	if (!element)
		return false;
	
	// List of part names of selector container
	var container_parts = ['Customer', 'Program', 'Call'];
	// Container name
	var container_name = element.attr('name').replace(/\[.*?\]/g, '');
	// Get part name only
	var part_name = element.attr('name').replace(new RegExp(container_name + '\\[(.*?)\\]', 'g'), '$1');
	// Value of changed element
	var value = element.val();
	
	// Request with empty value will reset all selected values in container,
	// so try to find first non-empty value on higher level and send it
	if (!value)
	{
        for (var i = container_parts.indexOf(part_name) - 1; i >= 0; i--)
		{
			value = $('select[name="'+container_name+'['+container_parts[i]+']"]').val();
			part_name = container_parts[i];
			if (value)
				break;
		}
	}	

	$.ajax({
		url: $('input[name="'+container_name+'[ajaxHandle]"]').val(), // Fetch prepared url of handle
		data: {
			id: value,
		    part_name: part_name
		},
		success: function(payload){
			if (payload && payload.customerSelectorResult)
			{
				// Clear all options from container parts except highest static level and prompts
				$('select[name*="'+container_name+'["][name!="'+container_name+'[Customer]"] option[value!=""]').each(function(){
					$(this).remove();
				});
				// Add new options
				var result = payload.customerSelectorResult;
				for (var part_name in result)
				{
					for (var id in result[part_name].values)
					{
						$('select[name="'+container_name+'['+part_name+']"][name!="'+container_name+'[Customer]"]').append($('<option>', {
							value: id,
							text: result[part_name].values[id]
						}));
					}
					// Fill according to selection
					$('select[name="'+container_name+'['+part_name+']"]').val(result[part_name].id);
				}
			}
		}
	});
}

// Updates customer selectors according to pre selected values
$(document).ready(function(){
    $('select[name*="[Call]"]').each(function(){
		handleCustomerSelector($(this));
	});
});

// Updates customer selector according to current state
$(document).on('change', 'select[name*="[Customer]"], select[name*="[Program]"], select[name*="[Call]"]', function(){
	handleCustomerSelector($(this));

// Only filter form should be resubmitted, because it is necessary to store new form state.
var closest_form = $(this).closest('form');
if (closest_form.attr('id').match(/filter/gi))
	closest_form.submit();
});